/* AC3 Mesquite module */

package mesquite.ac3.FelsContrWithChar;
/*~~  */

import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.ac3.lib.*;
import mesquite.cont.lib.*;
/** ======================================================================== */
public class FelsContrWithChar extends ContrastForChar {
	private int currentItem=0;
	private ContinuousDistribution observedStates;
	//private ContinuousDistribution observedStates2;
	//private MesquiteMenuItemSpec itemItem;
	private String itemName=null;
	private Tree lastUsedTree = null;
	private UnivariateContrastCalculator calculator = null;
	private long lastTreeVersion=-1;
	private int componentShown =0;

	//contrast option constants moved to ContrastForChar to avoid duplication and ensure everyone's on same page

	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		return true;  
	}

	/*.................................................................................................................*/
	//Peter: I compacted these
	/** returns current parameters, for logging etc..*/
	public String getBriefParameters() {
		String addendum = "";
		if (itemName!=null)
			addendum = "  (item " + itemName + ")";
		switch (componentShown) {
		case CONTRAST: 
			return "Standardized contrast" + addendum;
		case ABSCONTRAST:
			return "Absolute value of contrast";
		case RAWCONTRAST:
			return "Raw contrast" + addendum;
		case SD:
			return "Square root of branch length sum" + addendum;
		case VPRIME:
			return "v Prime" + addendum;
		case NODEVALUE:
			return "Estimated Node Value" + addendum;
		case CORRECTEDHEIGHT:
			return "Corrected Height" + addendum;
		case TIPS:
			return "Raw tip value" + addendum;
		case RESIDUAL:
			return "Residual" + addendum;
		case ABSRESIDUAL:
			return "Absolute value of residual" + addendum;
		case RAWHEIGHT:
			return "Uncorrected height of base node" + addendum;
		case ACDCP:
			return "Positivized Contrasts for Categorical Characters" + addendum; 
		} 
		return "Error: unknown contrast request";	
	}
	/*.................................................................................................................*/
	/** returns current parameters, for logging etc..*/
	public String getParameters() {
		String addendum = "";
		if (itemName!=null)
			addendum = "  (item " + itemName + ")";
		switch (componentShown) {
		case CONTRAST: 
			return "Standardized contrast" + addendum;
		case ABSCONTRAST:
			return "Absolute value of standardized contrast";
		case RAWCONTRAST:
			return "Raw (uncorrected) contrast" + addendum;
		case SD:
			return "Square root of sum of corrected branch lengths" + addendum;
		case VPRIME:
			return "v Prime" + addendum;
		case NODEVALUE:
			return "Estimated value of base node" + addendum;
		case CORRECTEDHEIGHT:
			return "Corrected height of base node" + addendum;
		case TIPS:
			return "Observed states of terminal taxa" + addendum;
		case RESIDUAL:
			return "Residual" + addendum;
		case ABSRESIDUAL:
			return "Absolute value of residual" + addendum;
		case RAWHEIGHT:
			return "Uncorrected height of base node" + addendum;
		case ACDCP:
			return "Positivized Contrasts for Categorical Characters" + addendum; 
		} 
		return "Error: unknown contrast request";	
	}
	/*.................................................................................................................*/
	public Snapshot getSnapshot(MesquiteFile file) {
		//TODO: allow change in assignTask, etc.
		Snapshot temp = new Snapshot();
		temp.addLine("setItem " + (currentItem));
		return temp;
	}
	/*.................................................................................................................*/
	public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
		if (checker.compare(this.getClass(), "Sets which item (from a multi-item continuous matrix) is used", "[number of item]", commandName, "setItem")) {
			int ic = MesquiteInteger.fromString(arguments);
			if (!MesquiteInteger.isCombinable(ic) && observedStates!=null){
				int numItems =observedStates.getNumItems();
				String[] items = new String[numItems];
				for (int i=0; i<items.length; i++){
					if (StringUtil.blank(observedStates.getItemName(i)))
						items[i] = "(unnamed)";
					else
						items[i]= observedStates.getItemName(i);
				}
				ic = ListDialog.queryList(containerOfModule(), "Item for contrasts", "Calculate contrasts for item:", MesquiteString.helpString, items, 0);
			}
			if (!MesquiteInteger.isCombinable(ic))
				return null;
			if (observedStates !=null && observedStates instanceof ContinuousDistribution) {
				if ((ic>=0) && (ic<=observedStates.getNumItems()-1)) {
					currentItem = ic;
					parametersChanged(commandRec);
				}
			}
		}
		else if (checker.compare(this.getClass(), "Sets the component to be calculated to Felsenstein's Contrasts", null, commandName, "showContrast")) {
			setOptionWithChangeParameters(CONTRAST,commandRec);
		}
		else if (checker.compare(this.getClass(), "Sets the component to be calculated to Absolute Value of Felsenstein's Contrasts", null, commandName, "showAbsContrast")) {
			setOptionWithChangeParameters(ABSCONTRAST,commandRec);
		}
		else if (checker.compare(this.getClass(), "Sets the component to be calculated to Raw Contrasts", null, commandName, "showRawContrast")) {
			setOptionWithChangeParameters(RAWCONTRAST,commandRec);
		}
		else if (checker.compare(this.getClass(), "Sets the component to be calculated to Standard Deviation", null, commandName, "showSD")) {
			setOptionWithChangeParameters(SD,commandRec);
		}
		else if (checker.compare(this.getClass(), "Sets the component to be calculated to V Prime", null, commandName, "showVPrime")) {
			setOptionWithChangeParameters(VPRIME,commandRec);
		}
		else if (checker.compare(this.getClass(), "Sets the component to be calculated to Estimated Node Values", null, commandName, "showNodeValue")) {
			setOptionWithChangeParameters(NODEVALUE,commandRec);
		}
		else if (checker.compare(this.getClass(), "Sets the component to be calculated to Corrected Height", null, commandName, "showCorrectedHeight")) {
			setOptionWithChangeParameters(CORRECTEDHEIGHT,commandRec);
		}
		else if (checker.compare(this.getClass(), "Sets the component to be calculated to Raw Height", null, commandName, "showRawHeight")) {
			setOptionWithChangeParameters(RAWHEIGHT,commandRec);
		}
		else 
			return super.doCommand(commandName, arguments, commandRec, checker);
		return null;
	}

	/* Peter: Other calls would need to be aware that parametersChanged might be called and hence do other parameter settings first before parametersChanged is called. 
	Wayne: I tried this, but it causes a hang in DiagnosticChart (try changing the call on line 709 to setOptionWithChangeParameters).  Not sure how to proceed.*/
	public void setOption(int option, CommandRecord commandRec){
		componentShown = option;
		//parametersChanged(commandRec);
	}

	/**
	@param int specifies the component (contrast related calculation) to be displayed
	@param CommandRec passed on to parametersChanged
	 */
	public void setOptionWithChangeParameters(int option, CommandRecord commandRec){
		componentShown = option;
		parametersChanged(commandRec);
	}

	/**
	stub?
	 */
	public void setXCharForReport(int xChar, CommandRecord commandRec) {      
	}

	public void setYCharForReport(int yChar, CommandRecord commandRec) {
	}

	/*Wayne: The following is my best guess as to how to protect take control against multiple calls.  It seems to fix the problem with duplicate
   	items on the submenu - which was one of the problems Ted noticed - but extra analysis is still being dumped on the text pane. 
   	/*Peter: this should be protected against being called more than once -- i.e., menu items should be added only if not yet exist, and should be deleted if !take */


	private MesquiteMenuItemSpec showContrastItem = null;
	private MesquiteMenuItemSpec showAbsContrastItem = null;
	private MesquiteMenuItemSpec showRawContrastItem = null;
	private MesquiteMenuItemSpec showSDItem = null;
	private MesquiteMenuItemSpec showVPrimeItem = null;
	private MesquiteMenuItemSpec showNodeValueItem = null;
	private MesquiteMenuItemSpec showCorrectedHeightItem = null;
	MesquiteSubmenuSpec mss = null;


	public void takeControl(boolean take){
		if (take){
			MesquiteCommand itemChoiceCommand = MesquiteModule.makeCommand("setItem",  (Commandable)this);
			if (mss == null)
				mss = addSubmenu(null, "Contrast Component", null);
			if (showContrastItem == null) {
				showContrastItem = addItemToSubmenu(null, mss, "Felsenstein's Contrast", makeCommand("showContrast", this));
				showAbsContrastItem = addItemToSubmenu(null, mss, "Absolute Value of Felsenstein's Contrast", makeCommand("showAbsContrast", this));		
				showRawContrastItem = addItemToSubmenu(null, mss, "Raw (uncorrected) Contrast", makeCommand("showRawContrast", this));
				showSDItem = addItemToSubmenu(null, mss, "Standard Deviation", makeCommand("showSD", this));
				showVPrimeItem = addItemToSubmenu(null, mss, "Vprime", makeCommand("showVPrime", this));
				showNodeValueItem = addItemToSubmenu(null, mss, "Estimated Node Value", makeCommand("showNodeValue", this));
				showCorrectedHeightItem = addItemToSubmenu(null, mss, "Corrected Height", makeCommand("showCorrectedHeight", this));
			}
		}
		else if ((mss != null) && (showContrastItem != null)) {
			if (showContrastItem != null) {
				deleteMenuItem(showContrastItem);
				showContrastItem = null;
				deleteMenuItem(showAbsContrastItem);
				showAbsContrastItem = null;
				deleteMenuItem(showRawContrastItem);
				showRawContrastItem = null;
				deleteMenuItem(showSDItem);
				showSDItem = null;
				deleteMenuItem(showVPrimeItem);
				showVPrimeItem = null;
				deleteMenuItem(showNodeValueItem);
				showNodeValueItem = null;
				deleteMenuItem(showCorrectedHeightItem);
				showCorrectedHeightItem = null;
			}
		}	
	}
	/*.................................................................................................................*/
	public void employeeParametersChanged(MesquiteModule employee, MesquiteModule source, Notification notification, CommandRecord commandRec) {
		observedStates = null;//to force recalculation
		super.employeeParametersChanged(module, source, notification, commandRec);
	}
	/** Called to provoke any necessary initialization.  This helps prevent the module's intialization queries to the user from
   	happening at inopportune times (e.g., while a long chart calculation is in mid-progress)*/
	public void initialize(Tree tree, CommandRecord commandRec){
	}

	// Holds and caches a NoPolyTree expansion of the tree.
	NoPolyTree localTree;
	private boolean[] localDeleted;  //holds corresponding deletion array

	/*.................................................................................................................*/
	public double[] calculateStandardDeviations(Tree tree,  CommandRecord commandRec) {
		if (tree == null)
			return null;
		if (tree == lastUsedTree && lastTreeVersion == tree.getVersionNumber() && calculator != null)
			return calculator.getStdDev();

		calculator = new UnivariateContrastCalculator(localTree,observedStates);
		return calculator.stdDevCalculation();
	}

	/*.................................................................................................................*/
	public void calculateNumbers(Tree tree, CharacterDistribution charDistribution, NumberArray result, MesquiteString resultString, CommandRecord commandRec) {
		observedStates = null;
		//modif here
		NumberArray nodesToRecalc=null;
		if(result!=null)
			nodesToRecalc=result.cloneArray();

		//end modif
		if (result==null || tree == null)
			return;
		result.deassignArray();
		if (!(charDistribution instanceof ContinuousDistribution))
			return;
		if (tree instanceof NoPolyTree){
			localTree = (NoPolyTree) tree;
		}
		else {
			localTree = new NoPolyTree((MesquiteTree)tree);
		}
		observedStates = (ContinuousDistribution)charDistribution;
		itemName=null;
		lastUsedTree = tree;
		lastTreeVersion = tree.getVersionNumber();
		int root = localTree.getRoot(localDeleted);
		if (observedStates != null) {
			calculator = new UnivariateContrastCalculator(localTree,observedStates);
			if (componentShown == TIPS){  // no need to calculate contrasts, just copy values from x
				double [] x = calculator.fillTips(root);
				for (int i=0; i<x.length; i++) {
					result.setValue(i, x[i]);
				}
			}
			else { 
				switch (componentShown) {
				case SD: {
					double [] stdDev = calculator.stdDevCalculation();
					//	if (!calculator.Ok()) {
					//		AlertDialog.notice(null,"Missing Contrast values", "I have encountered missing contrast values");
					//	}
					for (int i=0; i<stdDev.length; i++) 
						result.setValue(i, stdDev[i]);
					
					if(nodesToRecalc!=null && MesquiteDouble.isCombinable(nodesToRecalc.getDouble(0))){
						double[] res=calculator.correctedStdDevCalculation(nodesToRecalc);
						for (int j=0;j<res.length;j+=2)
							result.setValue((int)res[j], res[j+1]);
					}
					break;
				}
				case CONTRAST: {
					double [] contrasts = calculator.contrastCalculation();
					//	if (!calculator.Ok()) {
					//		AlertDialog.notice(null,"Missing Contrast values", "I have encountered missing contrast values");
					//	}
					for (int i=0; i<contrasts.length; i++) 
						result.setValue(i, contrasts[i]);
					if(nodesToRecalc!=null && MesquiteDouble.isCombinable(nodesToRecalc.getDouble(0))){
						double[] res=calculator.contrastToRecalculate(nodesToRecalc);
						for (int j=0;j<res.length;j+=2)
							result.setValue((int)res[j], res[j+1]);
					}
					break;
				}
				case ABSCONTRAST: {
					double [] contrasts = calculator.contrastCalculation();
					//	if (!calculator.Ok()) {
					//		AlertDialog.notice(null,"Missing Contrast values", "I have encountered missing contrast values");
					//	}
					for (int i=0; i<contrasts.length; i++) {
						if (contrasts[i] < 0)
							result.setValue(i, -1*contrasts[i]);
						else
							result.setValue(i, contrasts[i]);
					}
					if(nodesToRecalc!=null){
						double[] res=calculator.contrastToRecalculate(nodesToRecalc);
						if(res != null)
						for (int j=0;j<res.length;j+=2){
							if(res[j+1]>=0)
							result.setValue((int)res[j], res[j+1]);
						else
							result.setValue((int)res[j], -1*res[j+1]);
						}
					}
					
					break;
				}
				case ACDCP: {
					double [] contrasts = calculator.contrastCalculation();
					for (int i=0; i<contrasts.length; i++) {
						/*if (contrasts[i] < 0)
									result.setValue(i, -1*contrasts[i]);
								else*/
						result.setValue(i, contrasts[i]);
					}
					if(nodesToRecalc!=null){
						double[] res=calculator.contrastToRecalculate(nodesToRecalc);
						for (int j=0;j<res.length;j+=2){
							result.setValue((int)res[j], res[j+1]);
						}
					}
					break;

				}
				case  RAWCONTRAST: {
					double [] raw = calculator.rawContrastCalculation();
					//	if (!calculator.Ok()) {
					//		AlertDialog.notice(null,"Missing Contrast values", "I have encountered missing contrast values");
					//	}
					for (int i=0; i<raw.length; i++) 
						result.setValue(i, raw[i]);
					
					if(nodesToRecalc!=null){
						double[] res=calculator.contrastToRecalculate(nodesToRecalc);
						for (int j=0;j<res.length;j+=2)
							result.setValue((int)res[j], res[j+1]);
					}
					break;
				}
				case VPRIME: {
					double [] vPrime = calculator.vPrimeCalculation();
					for (int i=0; i<vPrime.length; i++) 
						result.setValue(i, vPrime[i]);
					break;
				}
				case CORRECTEDHEIGHT:
				case	RAWHEIGHT: {
					double [] height = calculator.heightCalculation(componentShown);   
					// If a node has a reconstructed value, but no contrast, 
					// its value is really a pass through and should not be
					// used in calculations...
					for (int i=0; i<height.length; i++) 
						result.setValue(i, height[i]);
					break;
				}
				case NODEVALUE: {
					double [] x = calculator.nodeValueCalculation();

					// If a node has a reconstructed value, but no contrast,
					// its value is really a pass through and should not be
					// used in calculations...
					for (int i=0; i<x.length; i++) {
						if (localTree.nodeIsInternal(i)) {
							if (MesquiteDouble.isCombinable(x[i]))  { // a value was set, so i specifies a node
								int left = tree.firstDaughterOfNode(i);
								int right = tree.lastDaughterOfNode(i);
								if (MesquiteDouble.isCombinable(x[left]) && MesquiteDouble.isCombinable(x[right]))
									result.setValue(i, x[i]);
							}
						}
						else
							result.setValue(i,x[i]);  // it's a tip so either it does or it doesn't have a value.
					}
					break;
				}
				}
			}
		}
	}

	/*.................................................................................................................*/
	public void calculateBiVariateNumbers(Tree tree, 
			ContinuousDistribution charDistribution, 
			ContinuousDistribution charDistribution2, 
			NumberArray result, 
			MesquiteString resultString, 
			CommandRecord commandRec) {
		observedStates = null;
		//observedStates2 = null;
		if (result==null || tree == null)
			return;
		result.deassignArray();
		if (!(charDistribution instanceof ContinuousDistribution) ||
				!(charDistribution2 instanceof ContinuousDistribution))
			return;
		if (tree instanceof NoPolyTree)
			localTree = (NoPolyTree) tree;
		else 
			localTree = new NoPolyTree((MesquiteTree)tree);
		observedStates = (ContinuousDistribution)charDistribution;
		//observedStates2 = (ContinuousDistribution)charDistribution2;
		itemName=null;
		lastUsedTree = tree;
		lastTreeVersion = tree.getVersionNumber();
		int root = localTree.getRoot();
		if (observedStates != null) {
			calculator = new UnivariateContrastCalculator(localTree,observedStates);
			if (componentShown == TIPS){  // no need to calculate contrasts, just copy values from x
				double [] x = calculator.fillTips(root);
				for (int i=0; i<x.length; i++) {
					result.setValue(i, x[i]);
				}
			}
			else { 
				switch (componentShown) {
				case SD: {
					double [] stdDev = calculator.stdDevCalculation();
					//	if (!calculator.Ok()) {
					//		AlertDialog.notice(null,"Missing Contrast values", "I have encountered missing contrast values");
					//	}
					for (int i=0; i<stdDev.length; i++) 
						result.setValue(i, stdDev[i]);
					break;
				}
				case CONTRAST: {
					double [] contrasts = calculator.contrastCalculation();
					//	if (!calculator.Ok()) {
					//		AlertDialog.notice(null,"Missing Contrast values", "I have encountered missing contrast values");
					//	}
					for (int i=0; i<contrasts.length; i++) 
						result.setValue(i, contrasts[i]);
					break;
				}
				case ABSCONTRAST: {
					double [] contrasts = calculator.contrastCalculation();
					//	if (!calculator.Ok()) {
					//		AlertDialog.notice(null,"Missing Contrast values", "I have encountered missing contrast values");
					//	}
					for (int i=0; i<contrasts.length; i++) {
						if (contrasts[i] < 0)
							result.setValue(i, -1*contrasts[i]);
						else
							result.setValue(i, contrasts[i]);
					}
					break;
				}
				case  RAWCONTRAST: {
					double [] raw = calculator.rawContrastCalculation();
					//	if (!calculator.Ok()) {
					//		AlertDialog.notice(null,"Missing Contrast values", "I have encountered missing contrast values");
					//	}
					for (int i=0; i<raw.length; i++) 
						result.setValue(i, raw[i]);
					break;
				}
				case VPRIME: {
					double [] vPrime = calculator.vPrimeCalculation();
					for (int i=0; i<vPrime.length; i++) 
						result.setValue(i, vPrime[i]);
					break;
				}
				case CORRECTEDHEIGHT:
				case	RAWHEIGHT: {
					double [] height = calculator.heightCalculation(componentShown);   
					// If a node has a reconstructed value, but no contrast, 
					// its value is really a pass through and should not be
					// used in calculations...
					for (int i=0; i<height.length; i++) 
						result.setValue(i, height[i]);
					break;
				}
				case NODEVALUE: {
					double [] x = calculator.nodeValueCalculation();

					// If a node has a reconstructed value, but no contrast,
					// its value is really a pass through and should not be
					// used in calculations...
					for (int i=0; i<x.length; i++) {
						if (localTree.nodeIsInternal(i)) {
							if (MesquiteDouble.isCombinable(x[i]))  { // a value was set, so i specifies a node
								int left = tree.firstDaughterOfNode(i);
								int right = tree.lastDaughterOfNode(i);
								if (MesquiteDouble.isCombinable(x[left]) && MesquiteDouble.isCombinable(x[right]))
									result.setValue(i, x[i]);
							}
						}
						else
							result.setValue(i,x[i]);  // it's a tip so either it does or it doesn't have a value.
					}
					break;
				}
				case ACDCP: {

				}
				}
			}
		}
	}


	/*.................................................................................................................*/
	public String getName() {
		return "Contrast calculations for AC3";
	}
	/*.................................................................................................................*/
	public String getVersion() {
		return "1.08";
	}
	/*.................................................................................................................*/
	public boolean isPrerelease() {
		return false;
	}
	/** returns an explanation of what the module does.*/
	public String getExplanation() {
		return "Calculates Felsenstein's contrasts and other related statistics for continuous-valued characters on a tree.";
	}
	/*.................................................................................................................*/
	public String getAuthors() {
		return "Franois Gao, Jonathan Corcy, Michel Laurin (based on PDAP)";
	}

}

