/* AC3 Mesquite module */

package mesquite.ac3.RootedRegression;
/*~~  */

import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.ac3.CIReport.CIReport;
import mesquite.ac3.SPRootReconstruction.SPRootReconstruction;
import mesquite.ac3.lib.*;
import mesquite.categ.lib.CategoricalDistribution;
import mesquite.cont.lib.*;
//import JSci.maths.statistics.*;


/* ======================================================================== */
public class RootedRegression extends RootRegressionCalculator {
	Tree myTree = null;
	TreeReference lastTreeReference;
	String DiagnosticString;
	AC32CTStatPak myStats = null;
	AC32CTStatPakSource sp2cts;
	MesquiteBoolean verboseQueryMode;   
	MesquiteChart chart;
    AC3TreeWinAsstC chartModule = null;
	CIReport CIReportTask; 
	CategoricalDistribution observedStates1 = null;
	ContinuousDistribution observedStates2 = null;

	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		verboseQueryMode = new MesquiteBoolean(false);
		loadPreferences();
		//verboseQueryMode.setValue(false);
		lastTreeReference = null;
		//addMenuItem("Root Calculation Query Mode...", makeCommand("setVerboseQueryMode",  (Commandable)this));
		addMenuItem("Set CI/PI width...", makeCommand("setwidth",  (Commandable)this)); 
		addMenuItem("Set Vh...", makeCommand("setvh",  (Commandable)this)); 
		//addMenuItem("Set Polytomy DF Reduction...", makeCommand("setDFReduction",  (Commandable)this)); 
		addMenuItem("Show CI/PI columns (text window)",makeCommand("showCIPIColumns", (Commandable)this));
		addMenuItem("Add a value to the CI/PI table",makeCommand("addCIPIvalues",(Commandable)this));
		addMenuItem(null, "Write CI/PI table to a file", makeCommand("writeCI", (Commandable)this));
		sp2cts = (AC32CTStatPakSource)hireNamedEmployee(commandRec, AC32CTStatPakSource.class, "#SPRootReconstruction");
		//myStats = sp2cts.getStatPak();	
		CIReportTask = (CIReport)hireNamedEmployee(commandRec, CIReport.class, "#CIReport");
		if (CIReportTask == null)
			return sorry(commandRec, getName() + " couldn't start because no CI reporter module obtained."); 		

 		return true;
  	}
	/*-----------------------------------------*/
	public void processPreferencesFromFile (String[] prefs) {
		if (prefs!=null && prefs.length>0)
			verboseQueryMode.setValue("v".equals(prefs[0]));
	}
	/*-----------------------------------------*/
	public String[] preparePreferencesForFile () {
		String[] prefs= new String[1];
		if (verboseQueryMode.getValue())
			prefs[0] = "v";
		else
			prefs[0] = "n";
		return prefs;
	}
  	 
	/*-----------------------------------------*/ 
  	public Snapshot getSnapshot(MesquiteFile file) {
   	 	Snapshot temp = new Snapshot();
  	 	temp.addLine("setwidth " + myStats.getWidth1());
  	 	temp.addLine("setwidth2 " + myStats.getWidth2());
  	 	temp.addLine("setvh " + myStats.getVh());
  	 	temp.addLine("setDFReduction " + myStats.getDFReduction());
  	 	return temp;
  	}
	/*-----------------------------------------*/

	/**
	 @return String the columns' description
	 This just generates the description of the columns in the CI/PI file.  It's
     broken out to make doCommand easier to follow.
    */
	private String makeCIPIcolumnString() {
	    double width1 = 0.95;
        double width2 = 0.90;
        if (myStats != null) {
        	width1 = myStats.getWidth1();
            width2 = myStats.getWidth2();
        }
        return("The Confidence and Prediction interval (CI/PI) value dump on the text pane contains the following columns:\n\n" +
	           " 1.  Tip Name\n" +
               " 2.  X Value\n" +
               " 3.  Observed Y Value (or -99999 for generated X values)\n" +
               " 4.  Predicted Y Value (Yhat)\n" +
               " 5.  Lower " + width1*100 + "% Confidence Interval\n" + 
               " 6.  Upper " + width1*100 + "% Confidence Interval\n" +
               " 7.  Lower " + width1*100 + "% Prediction Interval\n" +
               " 8.  Upper " + width1*100 + "% Prediction Interval\n" +
               " 9.  Lower " + width2*100 + "% Confidence Interval\n" +
               "10.  Upper " + width2*100 + "% Confidence Interval\n" +
               "11.  Lower " + width2*100 + "% Prediction Interval\n" +
               "12.  Upper " +width2*100 + "% Prediction Interval\n");
	}
	
	
	/** 
	The dispatcher for mesquite/script commands
	*/
	public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
		if (checker.compare(this.getClass(),"Sets the first width for the confidence and prediction intervals","[a confidence width e.g., 0.95]",commandName,"setwidth1")){
			if (myStats == null)
				return null;
			MesquiteInteger pos = new MesquiteInteger(0);
			double newWidth= MesquiteDouble.fromString(arguments, pos);
			if (!MesquiteDouble.isCombinable(newWidth))
				myStats.querySetWidth();
			else
				myStats.setWidth1(newWidth);
			parametersChanged(commandRec);
		}
		else if (checker.compare(this.getClass(),"Sets the second width for the confidence and prediction intervals","[a confidence width e.g., 0.90]",commandName,"setwidth2")){
			if (myStats == null)
				return null;
			MesquiteInteger pos = new MesquiteInteger(0);
			double newWidth= MesquiteDouble.fromString(arguments, pos);
			if (!MesquiteDouble.isCombinable(newWidth))
				myStats.querySetWidth();
			else 
				myStats.setWidth2(newWidth);
			parametersChanged(commandRec);
		}
		else if (checker.compare(this.getClass(),"Sets both widths for the confidence and prediction intervals","[two confidence widths e.g., 0.95 0.90]",commandName,"setwidth")){
			if (myStats == null)
				return null;
			MesquiteInteger pos = new MesquiteInteger(0);
			double newWidth1= MesquiteDouble.fromString(arguments, pos);
			MesquiteInteger pos2 = null;
			double newWidth2 = MesquiteDouble.unassigned;			
			if (MesquiteDouble.isCombinable(newWidth1)) {
				pos2 = new MesquiteInteger(arguments.indexOf(' ',0));
				if (pos2.getValue()>0)
					newWidth2 = MesquiteDouble.fromString(arguments,pos2);
			}
			if (!MesquiteDouble.isCombinable(newWidth1) ||
					!MesquiteDouble.isCombinable(newWidth2))
				myStats.querySetWidth();
			else {
				myStats.setWidth1(newWidth1);
				myStats.setWidth2(newWidth2);
			}
			parametersChanged(commandRec);
		}
		else if (checker.compare(this.getClass(),"Sets the tip to root distance for added nodes in prediction interval regression estimates","[a real number]",commandName,"setvh")) {
			if (myStats == null)
				return null;
			MesquiteInteger pos = new MesquiteInteger(0);
			double newVh= MesquiteDouble.fromString(arguments, pos);
			if (!MesquiteDouble.isCombinable(newVh))
				myStats.querySetVh();
			else 
				myStats.setVh(newVh);
			parametersChanged(commandRec);
		}
		else if (checker.compare(this.getClass(),"Sets the degrees of freedom reduced because of polytomies","[number of dof]",commandName,"setDFReduction")) {
			if (myStats == null)
				return null;
			MesquiteInteger pos = new MesquiteInteger(0);
			int dfReduction= MesquiteInteger.fromFirstToken(arguments, pos);
			if (!MesquiteInteger.isCombinable(dfReduction))
				myStats.queryDFReduce(myTree, true);
			else
				myStats.setDFReduction(dfReduction);
			parametersChanged(commandRec);
		}
		else if (checker.compare(this.getClass(), "Sets the query mode to verbose or not", "[on/off]", commandName, "setVerboseQueryMode")) {
			if (StringUtil.blank(arguments)){
				MesquiteBoolean ok = MesquiteBoolean.queryCheckBox(containerOfModule(), "Set Verbose Query Preference", "In verbose mode you will be asked for settings for AC3 root reconstruction frequently; otherwise you change the settings using menu items", "Use Verbose Mode", verboseQueryMode.getValue());
				if (!ok.isUnassigned()) {
					verboseQueryMode.setValue(ok.getValue());
					storePreferences();
				}
			}
		}
		else if (checker.compare(this.getClass(),"Write CI/PI table to a file",null,commandName,"writeCI")) {
			if (myStats == null)
				return null;
			CIReportTask.setStatPak(myStats);
			MesquiteFile CIfile = new MesquiteFile();
			CIReportTask.writeReport(CIfile,observedStates1,observedStates2,myTree,arguments,commandRec);	
		}
		else if (checker.compare(this.getClass(),"Show CI/PI columns (text window)","",commandName, "showCIPIColumns")){
			String description = makeCIPIcolumnString();
			AlertDialog.notice(null,"CI/PI Columns",description);
		}
		else if(checker.compare(this.getClass(),"Add a value to the CI/PI table","[a real number]",commandName,"addCIPIValues")) {
			double newValue = MesquiteDouble.queryDouble(null,"Add CI/PI value","Enter x (horizontal trait) value to add to the CI/PI table",0.0);
			myStats.addCIPIValue(new MesquiteDouble(newValue));
		}   		
		else
			return super.doCommand(commandName, arguments, commandRec, checker);
		return null;
	}
		
    /*-----------------------------------------*/
    public void setChartModule(AC3TreeWinAsstC module) {
        chartModule = module;
        myStats = sp2cts.getStatPak(chartModule.getDFReduction());
    }

    /*-----------------------------------------*/    
    /**
     * Simply returns the statPak so the charter can access the slope and other
     * interesting results.
     */
    public AC32CTStatPak getStatPak() {
        if (chartModule == null){
            MesquiteMessage.warnProgrammer("No chartModule set for "+this);
            return null;
        }
        if (myStats == null){
            myStats = sp2cts.getStatPak(chartModule.getDFReduction());
        }
        return myStats;
    }


	public void setChart(MesquiteChart chart){
		this.chart = chart;
	}
	/*-----------------------------------------*/
	/** This is a nop for now - maybe remove this from the abstract class?? */
	public void setOption(int option) {
	}

	/*.................................................................................................................*/
	public void calculateNumbers(Tree tree, 
						     CharacterDistribution charDistribution1,
						     CharacterDistribution charDistribution2,
						     NumberArray result,
						     MesquiteString resultString, 
						     CommandRecord commandRec) {
		//MesquiteDouble conSlope = new MesquiteDouble();
		int root;   // don't initialize until tree is known good
		//Tree dataReducedTree = null;
		
		if (result==null || tree == null)
			return;
		if (!(charDistribution1 instanceof CategoricalDistribution) ||!(charDistribution2 instanceof ContinuousDistribution))
			return;
		result.zeroArray();
		observedStates1 = (CategoricalDistribution)charDistribution1;
		observedStates2 = (ContinuousDistribution)charDistribution2;
		if ((lastTreeReference == null) || (tree instanceof MesquiteTree && !((MesquiteTree)tree).sameTreeVersions(lastTreeReference, true, true))) { // Peter: tighter restrictions; any change will provoke query or reset

			// first create and initialize the new rootStatPak
			myStats = sp2cts.getStatPak(chartModule.getDFReduction());	
			// now set the tree so doCalculations will do the right thing...
			myStats.setTree(tree);
			root = tree.getRoot();
			myStats.countInternalNodes(tree,root);  // count the internal nodes, to set number
			int zlb = myStats.countZLBs(tree,root);     // Zero-Length-Branches are what AC3 calls "polytomies"; count to inform the user
			// now ask the user to set df Reductions
            if (!commandRec.scripting()){
                if (verboseQueryMode.getValue()){
                        myStats.queryDFReduce(tree, true);
                        chartModule.setDFQueryDone(true);
                        myStats.querySetWidth();
                        myStats.querySetVh();
                }
                else if ((!chartModule.getDFQueryDone() && zlb > 0)){
                    myStats.queryDFReduce(tree, true);
                    chartModule.setDFQueryDone(true);
                    myStats.querySetWidth();
                    myStats.querySetVh();
                }
            }
		}
		myTree = tree;
		lastTreeReference = ((MesquiteTree)tree).getTreeReference(lastTreeReference);
		if ((observedStates1 != null) || (observedStates2 != null)) {
			NumberArray xArray = new NumberArray(); 
			NumberArray yArray = new NumberArray();
			myStats.setObserved1(observedStates1);
			myStats.setObserved2(observedStates2);
			boolean munch = myStats.doCalculations(xArray,yArray,chart, commandRec);
			// Results are never set - we just leave the tip data untouched for display
			if (munch)
				chart.munch(commandRec);
			if (resultString != null)
				resultString.setValue("CI and PI around Regression through root");
		}
	}



	/*.................................................................................................................*/

	/*.................................................................................................................*/
  	 public Class getCharacterClass() {
  	 	return ContinuousState.class;
  	 }
	/*.................................................................................................................*/
  	 public CompatibilityTest getCompatibilityTest() {
  	 	return new ContinuousStateTest();
  	 }
   
 	/** returns current parameters, for logging etc..*/
	/*.................................................................................................................*/
 	public String getParameters() {
		return "Regression through reconstructed root";
   	 }

	/*.................................................................................................................*/
    	 public String getName() {
		return "Regression through Reconstructed Root Value for AC3";
   	 }
    /*.................................................................................................................*/
    public String getAuthors() {
        return "Franois Gao , Jonathan Corcy, Michel Laurin (based on PDAP)" ;
    }
    /*.................................................................................................................*/
    public String getVersion() {
        return "1.08";
    }
	/*.................................................................................................................*/
  	public boolean isPrerelease() {
		return false;
   	}
	/*.................................................................................................................*/
  	public String getExplanation() {
		return "Calculates regression passing through a reconstruction of characters at the root.";
   	}

}
