/* AC3 Mesquite module */

package mesquite.ac3.SPOriDiagnostics;
/*~~  */

import java.awt.*;

import mesquite.ac3.lib.*;
import mesquite.lib.*;


public class SPOriDiagnostics extends AC32CTStatPakSource  {
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		return true;
	}

	public AC32CTStatPak getStatPak(MesquiteInteger dfr){
	   return new oriStatPak(dfr);
	}
	
	public String getName(){
		return "Regression through Origin StatPak Source for AC3";
	}
		
	/*.................................................................................................................*/
    public String getAuthors() {
        return "Franois Gao , Jonathan Corcy, Michel Laurin (based on PDAP)" ;
    }
    /*.................................................................................................................*/
	/** returns module version */
    public String getVersion() {
        return "1.08(2)";
    }
	/*.................................................................................................................*/
  	/** returns true when a module is prerelease */
  	public boolean isPrerelease() {
		return false;
   	}
	/*.................................................................................................................*/
   	public boolean isSubstantive() {
   		return false;  // the statPak is substantive, but I don't think this module is...
   	}
	/*.................................................................................................................*/
	public String getExplanation() {
		return "Calculates summary statistics and regression through the origin; used for contrast vs. contrast plots";
	}
}

class oriStatPak extends AC32CTStatPak{
	
	private double        r2;    				        //r squared value for least squares regression
	private double        ts;    					    //T statistic
	private double        f;    					    //F statistic
	private int        	  conypos = 0;                  //count of positive contrasts
	private int        	  cony0 = 0;                    //count of negative contrasts
	private int        	  conyneg = 0;                  //count of contrasts == 0
	private double        pvalue = MesquiteDouble.unassigned;  //significance level of regression
	private int			  unusedNumber=0;
	private NumberArray	  contrastWithNodes;
	
	public oriStatPak(MesquiteInteger dfr) {
		super(dfr);
	}
	
	
	/** positivize by flipping the sign of corresponding elements of data1 and data2 if the data1 element is negative */
	private void positivize(NoPolyTree tree, double[] data1,double [] data2,int node) {
		for (int daughter = tree.firstDaughterOfNode(node); tree.nodeExists(daughter); daughter = tree.nextSisterOfNode(daughter))
			positivize(tree, data1, data2, daughter);
			if (MesquiteDouble.isCombinable(data1[node]) && MesquiteDouble.isCombinable(data2[node])) {
				if (data1[node] < 0) {	
					data1[node] *= -1;
					data2[node] *= -1;
				}
			}
	}

	
	private void calcBounds(Tree tree, double [] data0, double [] data1, int node) {
		if (tree.nodeIsInternal(node))
		 	for (int daughter = tree.firstDaughterOfNode(node); tree.nodeExists(daughter); daughter = tree.nextSisterOfNode(daughter)) {
				     calcBounds(tree, data0, data1, daughter);
		 	}
			if (MesquiteDouble.isCombinable(data0[node]) &&
			   MesquiteDouble.isCombinable(data1[node])) {
			if (data0[node] > max[0])	
				max[0] = data0[node];
			if (data1[node] > max[1])
				max[1] = data1[node];
			if (data0[node] < min[0])
				min[0] = data0[node];
			if (data1[node] < min[1])
				min[1] = data1[node];
		}
	}	
	
	private void calcVars(Tree tree, double[] contrasts1, double contrasts2 []) {
		number = 0;
		variance[0] = 0.0;
		variance[1] = 0.0;
        cov = 0.0;
		varianceSums(tree,contrasts1,contrasts2,tree.getRoot());
		variance[0] /= getNumber()-1;  			
		variance[1] /= getNumber()-1; 
		cov /= getNumber()-1;
	}
	
	/** This calculates the sums for getVars */
	private void varianceSums(Tree tree, double[] contrasts1, double[] contrasts2, int node) {
		for (int daughter = tree.firstDaughterOfNode(node); tree.nodeExists(daughter); daughter = tree.nextSisterOfNode(daughter))
			varianceSums(tree, contrasts1, contrasts2, daughter);
		if (tree.nodeIsInternal(node) && 
				MesquiteDouble.isCombinable(contrasts1[node]) &&
				MesquiteDouble.isCombinable(contrasts2[node])) {
			number++;
			variance[0] += contrasts1[node]*contrasts1[node];
			variance[1] += contrasts2[node]*contrasts2[node];
			cov += contrasts1[node]*contrasts2[node];
		}
		else if (tree.nodeIsInternal(node) && 
				(MesquiteDouble.isCombinable(contrasts1[node])  ||
				MesquiteDouble.isCombinable(contrasts2[node]))) {
			unusedNumber++;
		}
	}
	
	
	/**
	@param tree the tree to traverse
	@param contrasts1 the contrasts in variable1 (horizontal), only used for validity checking
	@param contrasts2 the contrasts in variable2 (vertical), the values actually counted
	@param node controls recursion in the tree
 	This method counts the number of contrasts that are greater than,
    less than, or equal to zero and updates the appropriate fields of the
 	statpak.  The first contrast array is just used to avoid counting points that
 	are only defined in one variable.
 	*/
    //PEM 16-Nov-2003
	private void countContrasts(Tree tree, double[] contrasts1, double[] contrasts2, int node) {
		for (int daughter = tree.firstDaughterOfNode(node); tree.nodeExists(daughter); daughter = tree.nextSisterOfNode(daughter))
			countContrasts(tree, contrasts1, contrasts2, daughter);
		if (tree.nodeIsInternal(node) && 
				MesquiteDouble.isCombinable(contrasts1[node]) &&
				MesquiteDouble.isCombinable(contrasts2[node])) {
			if (contrasts2[node]>0)
				conypos++;
			else
				if (contrasts2[node]<0)
				conyneg++;
				else cony0++;
		}
	}

	
	private double[] contrasts1, contrasts2;

	/**Do any calculations needed*/
	
	//calculations for screen 15
	public boolean doCalculations(NumberArray array, NumberArray array2,
			boolean screen15, MesquiteChart chart, CommandRecord commandRec) {
		//UnivariateContrastCalculator continuousCalculator;
		//AC3ContrastCalculator categoricalCalculator;
		min[0] = MesquiteDouble.infinite;
		min[1] = MesquiteDouble.infinite;
		max[0] = MesquiteDouble.negInfinite;
		max[1] = MesquiteDouble.negInfinite;
		contrastWithNodes=null;
		unusedNumber = 0;
		resetChartBounds(chart);    // this seems to be necessary for the charts to scale properly
		contrasts1=new double[array.getSize()];
		contrasts2=new double[array2.getSize()];
		//continuousCalculator = new BivariateContrastCalculator(tree, obs1, obs2);
		//categoricalCalculator= new AC3ContrastCalculator(tree,obs1);
		//continuousCalculator = new UnivariateContrastCalculator(tree,obs2);
		for (int i =0;i< array.getSize();i++)
		contrasts1[i] = array.getDouble(i);
		for (int i =0;i< array.getSize();i++)
			contrasts2[i] = array2.getDouble(i);
		/*if(categoricalCalculator.getNodesToRecalculate()!=null){
			continuousCalculator = new UnivariateContrastCalculator(tree,obs2,categoricalCalculator.getNodesToRecalculate());
			contrasts2 = continuousCalculator.contrastCalculation();
			double[] res=continuousCalculator.contrastToRecalculate(categoricalCalculator.getNodesToRecalculate());
		for (int j=0;j<res.length;j+=2){
				contrasts2[(int)res[j]]=res[j+1];
			}
		else{
		
		
		}*/
		positivize(tree,contrasts1,contrasts2,root);
		calcBounds(tree,contrasts1,contrasts2,root);
		calcVars(tree,contrasts1,contrasts2);

        
		if ((variance[0] != 0) && (variance[1] != 0))
			corr = cov/Math.sqrt(variance[0]*variance[1]);
		else
			corr = 0;
		if (variance[0] != 0)  {
			lsregok = true;
			ls = cov/variance[0];
			lsy = 0.0;
		}
		else
			lsregok = false;
        // reset the contrast counts
        conypos = 0;
        conyneg = 0;
        cony0 = 0;
		countContrasts(tree,contrasts1,contrasts2,root);  // need xValues for filtering
		finishStatOrg();
        tStats = new NRtDistribution(df);
        if (Math.abs(corr) <1 && df > 0)
            if (ts<0)
                pvalue= tStats.cumulative(ts);
            else
                pvalue = 1-tStats.cumulative(ts);
		else
			pvalue = MesquiteDouble.unassigned;
		return false;   // really check this out PEM 1-Oct-2003
		
	}

	//calculations for screen 14
	public boolean doCalculations(NumberArray xValues, NumberArray yValues, MesquiteChart chart, CommandRecord commandRec){
		//BivariateContrastCalculator continuousCalculator;
		UnivariateContrastCalculator continuousCalculator;
		AC3ContrastCalculator categoricalCalculator;
		min[0] = MesquiteDouble.infinite;
		min[1] = MesquiteDouble.infinite;
		max[0] = MesquiteDouble.negInfinite;
		max[1] = MesquiteDouble.negInfinite;
		contrastWithNodes=null;
		unusedNumber = 0;
		resetChartBounds(chart);    // this seems to be necessary for the charts to scale properly
		
		//continuousCalculator = new BivariateContrastCalculator(tree, obs1, obs2);
		categoricalCalculator= new AC3ContrastCalculator(tree,obs1);
		continuousCalculator = new UnivariateContrastCalculator(tree,obs2);
		contrasts1 = categoricalCalculator.contrastCalculation();
		contrasts2 = continuousCalculator.contrastCalculation();
		if(categoricalCalculator.getNodesToRecalculate()!=null){
			contrastWithNodes=categoricalCalculator.getNodesToRecalculate().cloneArray();
		double[] res=continuousCalculator.contrastToRecalculate(contrastWithNodes);
		double[] stdDev=continuousCalculator.getStdDev();
		double[] result=continuousCalculator.correctedStdDevCalculation(contrastWithNodes);
			for (int j=0;j<result.length;j+=2)
				stdDev[(int)result[j]]= result[j+1];
		for(int k=0;k<contrasts1.length;k++){
			if(MesquiteDouble.isCombinable(contrasts1[k]) && stdDev[k]!=0)
			contrasts1[k]=contrasts1[k]/stdDev[k];
		}
		for (int j=0;j<res.length;j+=2){
			//if((double)res[j+1]!=0)
				contrasts2[(int)res[j]]=(double)res[j+1];}
		}
		/*if(categoricalCalculator.getNodesToRecalculate()!=null){
			continuousCalculator = new UnivariateContrastCalculator(tree,obs2,categoricalCalculator.getNodesToRecalculate());
			contrasts2 = continuousCalculator.contrastCalculation();
			double[] res=continuousCalculator.contrastToRecalculate(categoricalCalculator.getNodesToRecalculate());
		for (int j=0;j<res.length;j+=2){
				contrasts2[(int)res[j]]=res[j+1];
			}
		else{
		
		
		}*/
		positivize(tree,contrasts1,contrasts2,root);
		calcBounds(tree,contrasts1,contrasts2,root);
		calcVars(tree,contrasts1,contrasts2);

        
		if ((variance[0] != 0) && (variance[1] != 0))
			corr = cov/Math.sqrt(variance[0]*variance[1]);
		else
			corr = 0;
		if (variance[0] != 0)  {
			lsregok = true;
			ls = cov/variance[0];
			lsy = 0.0;
		}
		else
			lsregok = false;
        // reset the contrast counts
        conypos = 0;
        conyneg = 0;
        cony0 = 0;
		countContrasts(tree,contrasts1,contrasts2,root);  // need xValues for filtering
		finishStatOrg();
        tStats = new NRtDistribution(df);
        if (Math.abs(corr) <1 && df > 0)
            if (ts<0)
                pvalue= tStats.cumulative(ts);
            else
                pvalue = 1-tStats.cumulative(ts);
		else
			pvalue = MesquiteDouble.unassigned;
		return false;   // really check this out PEM 1-Oct-2003
		
	}





	// This method finishes the calculation of the three regression models (ordinary least squares, major axis
	// and reduced major axis) displayed by screen 9, and now available elsewhere in Mesquite.
	public void finishStatOrg()  {        
	    if (corr == 0) 
	        lsregok = false;
	    if (lsregok) 
	        rma = ls/corr;
	    if (((corr > 0) && (rma < 0)) ||
	            ((corr < 0) && (rma > 0)))
	        rma *= -1;
	    rmay = 0.0;
	    double d = Math.sqrt(((variance[0]+variance[1])*(variance[0]+variance[1])) - 4*(variance[0]*variance[1]-cov*cov));
	    double lambda1 = (variance[0]+variance[1]+d)/2;
	    // Sokal and Rolf 1981 set Y1 for vertical axis, for example on 595-596.
	    if (lambda1 == variance[1]) {
	        if (cov >0)
	            ma =  Double.POSITIVE_INFINITY;
	        else ma = Double.NEGATIVE_INFINITY;
	    }
	    else ma = cov/(lambda1-variance[1]);	// This is correct variance[1] is vertical axis
	    may = 0.0;
	    r2 = corr*corr;
	    if (r2 != 1) {
	        ts = corr*Math.sqrt((getNumberMinus1())/(1-r2));
	        f = ts*ts;
	    }
        df = getNumberMinus1()-getDFReduction();
	}
	
	
	public double getPValue() {
		return pvalue;
	}


/*............................................................*/
// Now that the statPak is set up, harvest the result string.

 	public String flst() {
 		StringBuffer results = new StringBuffer(300);
 		results.append("Summary statistics for regression [of contrasts] through the origin.\n\n");
 		results.append("Total number of contrasts: " + (getNumber()+unusedNumber) + "\n");
 		results.append("Number of used contrasts: " + getNumber() + "\n");
 		results.append("Number of unused contrasts: " + unusedNumber + "\n\n");
 		results.append("List of recalculated nodes: ");
 		if(contrastWithNodes!=null){
 			for (int i=0;i<contrastWithNodes.getSize();i+=3){
 				if (contrastWithNodes.getInt(i)!=0){
 					boolean leftNodeIsTerm=tree.nodeIsTerminal(contrastWithNodes.getInt(i+1));
 					boolean rightNodeIsTerm=tree.nodeIsTerminal(contrastWithNodes.getInt(i+2));
 					if(leftNodeIsTerm && rightNodeIsTerm)
 						results.append("\n\tTerminal taxa: "+ tree.getNodeLabel(contrastWithNodes.getInt(i+1))+"(Node "+ contrastWithNodes.getInt(i+1)+"), "+tree.getNodeLabel(contrastWithNodes.getInt(i+2)) +"(Node "+ contrastWithNodes.getInt(i+2)+") were contrasted at: Node "+contrastWithNodes.getInt(i));
 					else if(leftNodeIsTerm && !rightNodeIsTerm)
 						results.append("\n\tTerminal Taxon: "+ tree.getNodeLabel(contrastWithNodes.getInt(i+1)) +"(Node "+ contrastWithNodes.getInt(i+1)+") and internal node: Node "+ contrastWithNodes.getInt(i+2) +" were contrasted at: Node "+contrastWithNodes.getInt(i));
 					else if(!leftNodeIsTerm && rightNodeIsTerm)
 						results.append("\n\tTerminal Taxon: "+ tree.getNodeLabel(contrastWithNodes.getInt(i+2)) +"(Node "+ contrastWithNodes.getInt(i+2)+") and internal node: Node "+ contrastWithNodes.getInt(i+1) +" were contrasted at: Node "+contrastWithNodes.getInt(i));
 					else
 						results.append("\n\tInternal nodes: Node "+ contrastWithNodes.getInt(i+1) +", Node "+ contrastWithNodes.getInt(i+2) +" were contrasted at: Node "+contrastWithNodes.getInt(i));
 					}
 				else
 					break;
 			}
 		}
 		else results.append("None");
 		 		
 		results.append("\n\nMinimum of X,Y coordinates: " + min[0] + ", " + min[1] + "\n");
 		results.append("Maximum of X,Y coordinates: " + max[0] + ", " + max[1] + "\n");
 		results.append("Mean of X,Y coordinates: zero, zero\n");
 		results.append("Variance of X,Y coordinates: " + variance[0] + ", " + variance[1] + "\n");
 		results.append("Covariance: " + cov + "\n");
 		results.append("Pearson Product-Moment Correlation Coefficient: " + corr + "\n");
 		results.append("Reduced Major Axis.");
 		if (lsregok) {
 			results.append("   Slope: " + rma + "\n");
 			results.append("   Y  Intercept: zero\n");
 		}
 		else {
 			results.append("    Slope: undefined \n");
 			results.append("    Y Intercept: undefined \n");
 		}
 		results.append("Major Axis. \n");
 		results.append("   Slope: " + ma + "\n");
 		results.append("   Y Intercept: zero\n");
 		results.append("Least Squares Regression.\n");
 		if (lsregok) {
 			results.append("   Slope: " + ls + "\n");
 			results.append("   Intercept: zero\n");
 			results.append("   R Squared: " + r2 + "\n");
 		}
 		else {
 			results.append("   Slope: undefined\n");
 			results.append("   Intercept: undefined\n");
 			results.append("   R Squared: undefined\n");
 		}
 		if (Math.abs(corr) <1) {
 			results.append("t: "+ ts + "\n");
 			results.append("F: " + f + "\n");
 			results.append("d.f.: " + df + "\n");
 			if (df > 0) {
 			    double tail1;
 			    if (ts<0)
 			        tail1= tStats.cumulative(ts);
 			    else
 			        tail1 = 1-tStats.cumulative(ts);
 			    results.append("p-value:\n");
 			    results.append("   2-tailed: " + (2.0*tail1) + "\n");
 			    results.append("   1-tailed: " +  tail1 + "\n");
 			}
 		}
 		else {
 			results.append("t: undefined\n");
 			results.append("F: undefined\n");
 		}
 		results.append("Number of Y contrasts > 0 : " + conypos + "\n");
 		results.append("Number of Y contrasts < 0 : " + conyneg + "\n");
 		results.append("Number of Y contrasts = 0 : " + cony0 + "\n");
 		double signPvalue = SignTest.signTest(conypos,conyneg);
 		if (signPvalue > 0.5)
 			results.append("Sign test 2-tailed : 1.0\n");
 		else
 			results.append("Sign test 2-tailed : " + 2.0*signPvalue + "\n");
 		results.append("          1-tailed : " + signPvalue + "\n\n\n");
 		return results.toString();
 	}

	// This method returns text to display in the legend window.
	public String getLegendText() {
		StringBuffer results = new StringBuffer(200);
		results.append("Number of contrasts: " + getNumber() + "\n");
		results.append("Pearson Product-Moment Correlation Coefficient: " + corr + "\n");
		if (Math.abs(corr) <1) {
	    	   	if (df > 0)
	    	   		results.append("Two tailed p-value: " + (2.0*(1-tStats.cumulative(Math.abs(ts)))) + "\n");
		}
		return results.toString();
	}
	
	public void setDrawingOption(int option){
		// stub for now
	}

	/**draw on the chart*/
	// This draws the colored line segments for the three different regression models that will
	// be dumped below.  Unlike ScatterRegDiagnostics, these should all go through the origin, so
	// this method starts the lines at the origin, rather than min[x], which should be positive,
	// (or rarely zero in real data sets) since contrasts are positivized.
	public void drawOnChart(Graphics g, MesquiteChart chart){
		Charter c = chart.getCharter();
		int startX = c.xToPixel(0,chart);     
		int startYOLS =  c.yToPixel(ls*min[0],chart);
		int endX = c.xToPixel(max[0],chart);
		int endYOLS = c.yToPixel(ls*max[0],chart);
		int startYRMA =  c.yToPixel( rma*min[0],chart);
		int endYRMA = c.yToPixel(rma*max[0],chart);
		int startYMA =  c.yToPixel(ma*min[0],chart);
		int endYMA = c.yToPixel(ma*max[0],chart);
		int extraLineWidth = 1;

		g.setColor(Color.red);
		g.drawLine(startX, startYRMA,endX,endYRMA);
		// conditional handles drawing thicker lines
		if (Math.abs(endYRMA-startYRMA)>(endX-startX))   // abs(slope) > 1
			g.drawLine(startX+extraLineWidth,startYRMA,endX+extraLineWidth,endYRMA);
		else	
			g.drawLine(startX,startYRMA+extraLineWidth,endX,endYRMA+extraLineWidth);
		g.setColor(Color.green);
		g.drawLine(startX,startYMA,endX,endYMA);
		// conditional handles drawing thicker lines
		if (Math.abs(endYMA-startYMA)>(endX-startX))   // abs(slope) > 1
			g.drawLine(startX+extraLineWidth,startYMA,endX+extraLineWidth,endYMA);
		else	
			g.drawLine(startX,startYMA+extraLineWidth,endX,endYMA+extraLineWidth);
		// Draw the OLS line last so if it and the MA line are "almost" the same, the OLS line "wins."
		g.setColor(Color.black);
		g.drawLine(startX, startYOLS,endX,endYOLS);
		if (Math.abs(endYOLS-startYOLS)>(endX-startX))   // abs(slope) > 1
			g.drawLine(startX+extraLineWidth,startYOLS,endX+extraLineWidth,endYOLS);
		else	
			g.drawLine(startX,startYOLS+extraLineWidth,endX,endYOLS+extraLineWidth);
	}


	
	
	// These are stubs for now...
	public void clearCIPIlist() {
		// TODO Auto-generated method stub
		
	}


	public void querySetVh() {
		// TODO Auto-generated method stub
		
	}


	public void setVhToMeanHeight(Tree tree) {
		// TODO Auto-generated method stub
		
	}





}
