/* AC3 Mesquite module */

package mesquite.ac3.SPRegDiagnostics;
/*~~  */

import java.awt.*;
import java.util.*;

import mesquite.ac3.lib.*;
import mesquite.cont.lib.EigenAnalysis;
import mesquite.lib.*;
import mesquite.lib.duties.*;
import JSci.maths.statistics.*;    // for p-value calculations


public class SPRegDiagnostics extends StatPakSource  {
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		return true;
	}

	public AC3StatPak getStatPak(){
		return new regStatPak();
	}
	
	public String getName(){
		return "Regression Diagnostics StatPak Source for AC3";
		}
		
    /*.................................................................................................................*/
    public String getAuthors() {
        return "Franois Gao , Jonathan Corcy, Michel Laurin (based on PDAP)" ;
    }
    /*.................................................................................................................*/
	/** returns module version */
    public String getVersion() {
        return "1.08(2)";
    }
	/*.................................................................................................................*/
  	 public boolean isPrerelease() {
		return false;
   	}
	/*.................................................................................................................*/
   	public boolean isSubstantive() {
   		return false;  // the statPak is substantive, but I don't think this module is...
   	}
	public String getExplanation() {
		return "Calculates summary statistics and regression coefficients for a scatterplot.";
	}
}



class regStatPak extends AC3StatPak {

         double     r2;    		//r squared value for least squares regression
         double     ts;    		//T statistic
         double     f;    		//F statistic
         double 	pvalue = MesquiteDouble.unassigned;
         int			unusedNumber=0;
         
         
	public regStatPak(){
		super();
	}

	public boolean doCalculations(NumberArray xValues, NumberArray yValues, MesquiteChart chart, CommandRecord commandRec){
	    // here you can put the main calculations.  
	    resetChartBounds(chart);
	    MesquiteDouble sx = new MesquiteDouble(0.0);
	    MesquiteDouble sx2 = new MesquiteDouble(0.0);
	    MesquiteDouble sy = new MesquiteDouble(0.0);
	    MesquiteDouble sxy = new MesquiteDouble(0.0);
	    calcBounds(xValues,yValues);
	    getMeans(xValues,yValues);
	    mean[0] /= getNumber();
	    mean[1] /= getNumber();		
	    getVars(xValues,yValues);
	    variance[0] /= getNumberMinus1();
	    variance[1] /= getNumberMinus1();
	    corr /= getNumberMinus1();
	    cov = corr;
	    if ((variance[0] > 0)  && (variance[1] > 0))
	    		corr = corr/Math.sqrt(variance[0]*variance[1]);
	    else
	        corr = 0;
	    getSums(xValues,yValues,sx,sx2,sy,sxy);
	    if ((getNumber()*sx2.getValue()-sx.getValue()*sx.getValue()) != 0) {
	        lsregok = true; 
	        double d = 1/(getNumber()*sx2.getValue()-sx.getValue()*sx.getValue());
	        ls = d*(getNumber()*sxy.getValue()-sx.getValue()*sy.getValue());
	        lsy = d*(sx2.getValue()*sy.getValue()-sx.getValue()*sxy.getValue());
	    }
	    else
	        lsregok = false;
	    finishStat();			

	    if (Math.abs(corr) <1 && df > 0)
	        if (ts<0)
	            pvalue= tStats.cumulative(ts);
	        else
	            pvalue = 1-tStats.cumulative(ts);
	    else
	        pvalue = MesquiteDouble.unassigned;
	    return false;
	}

 	public double getPValue(){
 		return pvalue;
 	}
	public void getMeans(NumberArray data0, NumberArray data1)
	{
		for (int i = 0; i < data0.getSize(); i++)
		if (data0.isCombinable(i) && data1.isCombinable(i)) {
			mean[0] += data0.getDouble(i);
			mean[1] += data1.getDouble(i);
		}
		else if(data0.isCombinable(i) || data1.isCombinable(i))
			unusedNumber++;
	}

	public void getVars(NumberArray data0, NumberArray data1)
	{
	for (int i = 0; i < data0.getSize(); i++)
		if (data0.isCombinable(i) && data1.isCombinable(i)) {
			double dev0 = mean[0] - data0.getDouble(i);
			double dev1 = mean[1] - data1.getDouble(i);
			variance[0] += dev0*dev0;
			variance[1] += dev1*dev1;
			corr += dev0*dev1;
		}
	}

	// This calculates the rma and the y intercept for rma called rmay, ma and
	// the y intercept for the ma called may, r squared called r2, t statistic
	// called ts, the f statistic, and the degrees of freedom called df.
	public void finishStat() {	
    		if (variance[0]>0) {
     			rma = Math.sqrt(variance[1]/variance[0]);
        		if (corr<0) // make sure the sign of the reduced major axis slope agrees with 
           		rma *= -1;  // that of the correlation
        		rmay = mean[1]-rma*mean[0];
    		}
    		double d = Math.sqrt(((variance[0]+variance[1])*(variance[0]+variance[1])) - 4*(variance[0]*variance[1]-cov*cov));
		double lambda1 = (variance[0]+variance[1]+d)/2;
		
		// Sokal and Rolf 1981 set Y1 for vertical axis, for example on 595-596.
		if (lambda1 == variance[1]) {
			if (cov >0)
				ma =  Double.POSITIVE_INFINITY;
			else ma = Double.NEGATIVE_INFINITY;
		}
		else ma = cov/(lambda1-variance[1]);	// This is correct variance[1] is vertical axis
   		may = mean[1]-ma*mean[0];
    		r2 = corr*corr;
    		if (r2<1.0) {
    			ts = corr*Math.sqrt((getNumber()-2)/(1-r2));
        		f = ts*ts;  
    		}
    		df = getNumber()-2;   // nominal guess
		if (df > 0)
			tStats = new  NRtDistribution(df);  // have a df estimate, so now is time to make tStats	
	}

	/*............................................................*/
	// Now that the statPak is set up, harvest the result string.
	public String flst() {

		StringBuffer results = new StringBuffer(200);
		results.append("Summary statistics for regression on scatterplot.\n\n");
		results.append("Total number of data points: " + (getNumber()+unusedNumber) + "\n");
		results.append("Number of valid data points (used in calculations): " + getNumber() + "\n");
		results.append("Number of invalid data points: " + unusedNumber + "\n");
		results.append("Minimum of X,Y coordinates: " + min[0] + ", " + min[1] + "\n");
		results.append("Maximum of X,Y coordinates: " + max[0] + ", " + max[1] + "\n");
		results.append("Mean of X,Y coordinates: " + mean[0] + ", " + mean[1] + "\n");
		results.append("Variance of X,Y coordinates: " + variance[0] + ", " + variance[1] + "\n");
		results.append("Covariance: " + cov + "\n");
		results.append("Pearson Product-Moment Correlation Coefficient: " + corr + "\n");
		results.append("Reduced Major Axis.\n");
		if (lsregok) {
			results.append("   Slope: " + rma + "\n");
			results.append("   Y  Intercept: " + rmay + "\n");
		}
		else {
			results.append("    Slope: undefined \n");
			results.append("    Y Intercept: undefined \n");
		}
		results.append("Major Axis.\n");
		results.append("   Slope: " + ma + "\n");
		results.append("   Y Intercept: " + may + "\n");
		results.append("Least Squares Regression.\n");
		if (lsregok) {
			results.append("   Slope: " + ls + "\n");
			results.append("   Intercept: " + lsy + "\n");
			results.append("   R Squared: " + r2 + "\n");
		}
		else {
			results.append("   Slope: undefined\n");
			results.append("   Intercept: undefined\n");
			results.append("   R Squared: undefined\n");
		}
		if (Math.abs(corr) <1) {
           		results.append("t: "+ ts + "\n");
           		results.append("F: " + f + "\n");
   		  	results.append("d.f.: " + df + "\n");
    	   		if (df > 0) {
                    double tail1;
                    if (ts<0)
                        tail1= tStats.cumulative(ts);
                    else
                        tail1 = 1-tStats.cumulative(ts);
    	   			results.append("p-value:\n");
    	   			results.append("   2-tailed: " + (2.0*tail1) + "\n");
    	   			results.append("   1-tailed: " +  tail1 + "\n");
    	   		}
    		}
    		else {
    			results.append("t: undefined\n");
       			results.append("F: undefined\n");
    		}
    		results.append("\n\n");
		return results.toString();
	}
    	/*---------------------*/
	public String getLegendText() {
		StringBuffer results = new StringBuffer(200);
		results.append("Number of data points: " + getNumber() + "\n");
		results.append("Pearson Product-Moment Correlation Coefficient: " + corr + "\n");
		if (Math.abs(corr) <1) {
	   		if (df > 0)
	    	   		results.append("Two tailed p-value: "   + (2.0*(1-tStats.cumulative(Math.abs(ts)))) + "\n");
		}
		return results.toString();
	}
	
	public void setDrawingOption(int option){
		//stub for now
	}
	
	/**draw on the chart*/
	// This draws the line for the OLS regression models that will
	// be dumped below.  In this module, lines are drawn over the range of x values, compare with
	// the method in ScatterOriDiagnostics.
	public void drawOnChart(Graphics g, MesquiteChart chart){
		Charter c = chart.getCharter();
		int startX = c.xToPixel(min[0],chart);
		int startYOLS =  c.yToPixel(lsy + ls*min[0],chart);
		int endX = c.xToPixel(max[0],chart);
		int endYOLS = c.yToPixel(lsy + ls*max[0],chart);
		// currently not showing these...
		//int startYRMA =  c.yToPixel(rmay + rma*min[0],chart); 
		//int endYRMA = c.yToPixel(rmay + rma*max[0],chart);
		//int startYMA =  c.yToPixel(may + ma*min[0],chart);
		//int endYMA = c.yToPixel(may + ma*max[0],chart);
		int extraLineWidth = 1;

		g.setColor(Color.black);
		g.drawLine(startX, startYOLS,endX,endYOLS);
		if (Math.abs(endYOLS-startYOLS)>(endX-startX))   // abs(slope) > 1
			g.drawLine(startX+extraLineWidth,startYOLS,endX+extraLineWidth,endYOLS);
		else	
			g.drawLine(startX,startYOLS+extraLineWidth,endX,endYOLS+extraLineWidth);		
	}

	public boolean doCalculations(NumberArray xArray, NumberArray yArray, boolean screen15, MesquiteChart chart, CommandRecord commandRec) {
		// TODO Auto-generated method stub
		return false;
	}

} //class regStatPak
