/* Mesquite Class Library.  Copyright 2001 D. Maddison & W. Maddison.  Version 0.98.  July 2001.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)

AC3 reporter dialog is based on the Mesquite Exporter Dialog, 
and is Copyright 2002-2003 P. E. Midford.
*/
package mesquite.ac3.lib;

import java.awt.*;
import mesquite.lib.*;



/*===============================================*/
/** Report Options dialog box class*/
public class AC3ReporterDialog extends ExtensibleDialog {
	AC3FileReporter fileReporter;
	static final String reportString = "Write";
	static final String cancelString = "Cancel";
	Choice reportDelimiter;
	Choice tableDelimiter;
    Checkbox convertSpacesCheckBox;
	Checkbox writeOnlySelectedDataCheckBox;
	Checkbox writeOnlySelectedTaxaCheckBox;
	
	public AC3ReporterDialog (AC3FileReporter fileReporter, Frame parent, String title, MesquiteInteger buttonPressed) {
		super(parent, title, buttonPressed);
		this.fileReporter = fileReporter;
		setDefaultButton("Write");
	}
	/*.................................................................................................................*/
	/** 
     * This method adds a pop-up list of end of line delimiters to the dialog box.  Mesquite
	 * currently supports Mac, Windows, and Unix delimiters, with the default being the choice
	 * appropriate for the current system.
     */
	public void addLineEndPopUpPanel () {
		reportDelimiter = addPopUpMenu("End of line character:", "Current System Default","MacOS (CR)","Windows (CR+LF)","UNIX (LF)",fileReporter.lineDelimiter);
	}
	/** 
     * This method adds a pop-up list of table entry delimiters to the dialog box.
	 */
	public void addTableDelimiterPopUpPanel() {
		tableDelimiter = addPopUpMenu("Table entry delimiters:", "Tab", "Comma", "Space", "NewLine",fileReporter.tableDelimiter);
	}
    public void addSpaceConversionCheckBox(){
        convertSpacesCheckBox = this.addCheckBox("Convert spaces to underscore ('_') in node names", fileReporter.convertSpaces);
    }
	/*.................................................................................................................*/
	/** This method handles setting up the panels for the dialog box.  This is the long form, with
	 * all the toggles passed in.
     */
	public void addDefaultPanels (boolean dataSelected, boolean taxaSelected, boolean hasTables) {
		fileReporter.writeOnlySelectedTaxa = false;
		fileReporter.writeOnlySelectedData = false;
        fileReporter.convertSpaces = true;
		if (dataSelected) 
			 writeOnlySelectedDataCheckBox = this.addCheckBox("write only selected data", fileReporter.writeOnlySelectedData);
		else
			writeOnlySelectedDataCheckBox = new Checkbox();
			
		if (taxaSelected) 
			 writeOnlySelectedTaxaCheckBox = this.addCheckBox("write only selected taxa", fileReporter.writeOnlySelectedTaxa);
		else
			writeOnlySelectedTaxaCheckBox = new Checkbox();
		if (hasTables)
			addTableDelimiterPopUpPanel();
		addLineEndPopUpPanel();
        addSpaceConversionCheckBox();
		addPrimaryButtonRow(reportString, cancelString);
		prepareAndDisplayDialog();
	}
	/*.................................................................................................................*/
	//* This version has no passed toggles, so to decide whether to ask about table delimiters
	// it queries the reporter.
	public void addDefaultPanels () {
		if (fileReporter.hasTables())
			addTableDelimiterPopUpPanel();
		addLineEndPopUpPanel();
        addSpaceConversionCheckBox();
		addPrimaryButtonRow(reportString, cancelString);
		prepareAndDisplayDialog();
	}
	/*.................................................................................................................*/
	public void buttonHit(String buttonLabel, Button button) {
		super.buttonHit(buttonLabel, button);
		if (buttonLabel.equalsIgnoreCase(reportString)) {
			fileReporter.lineDelimiter = reportDelimiter.getSelectedIndex();
			fileReporter.tableDelimiter = tableDelimiter.getSelectedIndex();
		}
	}
	/*.................................................................................................................*/
	public static int query(AC3FileReporter fileReporter, Frame parent, String title) {
		MesquiteInteger buttonPressed = new MesquiteInteger(1);
		AC3ReporterDialog reportDialog = new AC3ReporterDialog(fileReporter, parent, title,buttonPressed);
//		Font font = new Font ("Dialog", Font.PLAIN, 12);
		reportDialog.addDefaultPanels();
		reportDialog.dispose();
		return buttonPressed.getValue();
	}
	/*.................................................................................................................*/
	public int query() {
		return buttonPressed.getValue();
	}
	/*.................................................................................................................*/
	public int query(boolean dataSelected, boolean taxaSelected,boolean convertSpaces) {
		if (dataSelected) 
			fileReporter.writeOnlySelectedData = writeOnlySelectedDataCheckBox.getState();
		if (taxaSelected) 
			fileReporter.writeOnlySelectedTaxa = writeOnlySelectedTaxaCheckBox.getState();
		if (convertSpaces)
            fileReporter.convertSpaces = convertSpacesCheckBox.getState();
		return buttonPressed.getValue();
	}
}
