/* AC3 Mesquite module */

package mesquite.ac3.AC3DiagnosticChart;

import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;
import mesquite.parsimony.ParsAncestralStates.ParsAncestralStates;
import mesquite.parsimony.lib.CategParsimonyModel;
import mesquite.parsimony.lib.OrderedModel;
import mesquite.parsimony.lib.ParsAncStatesForModel;
import mesquite.ac3.lib.*;
import mesquite.categ.lib.*;
import mesquite.cont.lib.*;

/* ======================================================================== */
/**=== Class AC3DiagnosticChart.  ===*/

public class AC3DiagnosticChart extends AC3TreeWinAsstC implements ChartListener  {
	public ContrastForChar contrastsStatsTask;
	public CharSourceCoordObed characterSourceTask,characterSourceTask2,characterSourceTask3;
	public ResidualsCalculator residualsTask;
	public DrawChart charterTask;
	//modif
//	public ParsAncestralStates ac3States; 
	public ParsAncStatesForModel ac3Model; 
	//end modif
	private Tree tree;
	public int currentX = 0;
	public int currentY = 1;
	public NodeLabeller nsLabeller;
	boolean separateAxes = true;
	public TreeDisplay treeDisplay;
	boolean showTreeLabels = false;
	MesquiteString screenName;
	public MesquiteCommand setXCommand;
	public MesquiteCommand setYCommand;
	Taxa currentTaxa;
	AC3ChartWindow window;
	//CharacterModel charModel;

	public int screen = 1;     // This is the first screen shown when the diagnostic chart is shown
	MesquiteString charSourceName,charSourceName2,charSourceName3;
	MesquiteCommand  cstC,cstC2,cstC3;
	MesquiteBoolean[] screenBoolean;
	public AC3ScatterAsst diagnosticsTask;
	MesquiteBoolean showLegend;
	MesquiteBoolean dfQueryDone;
	MesquiteInteger dfReduction;
	
	//modif
	MesquiteSubmenuSpec resori = null;
	private MesquiteMenuItemSpec showContinuousContinuousItem = null;
	private MesquiteMenuItemSpec showContinuousDiscreteItem = null;

	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, boolean hiredByName) {
		charterTask = (DrawChart)hireNamedEmployee(DrawChart.class, "#Scattergram");
		if (charterTask == null)
			return sorry(getName() + " couldn't start because no charting module obtained.");
		//modified here
		/*ac3States=(Ac3States)hireNamedEmployee(commandRec,CharStatesForNodes.class,"#Ac3States");
 		if (ac3States == null)
			return sorry(commandRec, getName() + " couldn't found ac3 module");
		ac3States=(ParsAncestralStates)hireEmployee(commandRec,ParsAncestralStates.class,"ParsAncestralStates");
		if (ac3States == null)
			return sorry(commandRec, getName() + " couldn't found ParsAncestralStates for ac3 module");*/
		ac3Model=(ParsAncStatesForModel)hireNamedEmployee(ParsAncStatesForModel.class,"#ParsimonyOrdered");
		if (ac3Model == null)
			return sorry(getName() + " couldn't found Model for Ancestral States Reconstruction for ac3 module");
		//charModel=new OrderedModel();
		//end modif

		characterSourceTask = (CharSourceCoordObed)hireCompatibleEmployee(CharSourceCoordObed.class,ContinuousState.class, "Source of continuous characters (for AC3 chart)");
		if (characterSourceTask == null ) {
			return sorry(getName() + " couldn't start because no source of characters obtained");
		}

		characterSourceTask2 = (CharSourceCoordObed)hireCompatibleEmployee(CharSourceCoordObed.class,CategoricalState.class, "Source of categorical characters (for AC3 chart)");
		if (characterSourceTask2 == null ) {
			return sorry(getName() + " couldn't start because no source of characters obtained");
		}
		
		characterSourceTask3 = (CharSourceCoordObed)hireEmployee(CharSourceCoordObed.class, "Source of categorical and continuous characters (for AC3 chart) needed to contrast Residuals of contrast regression vs. X contrast values");
		if (characterSourceTask3 == null ) {
			return sorry(getName() + " couldn't start because no source of characters obtained");
		}

		charSourceName = new MesquiteString(characterSourceTask.getName());
		charSourceName2 = new MesquiteString(characterSourceTask2.getName());
		charSourceName3 = new MesquiteString(characterSourceTask3.getName());

		cstC3 =  makeCommand("setCharacterSource3",  (Commandable)this);
		cstC2 =  makeCommand("setCharacterSource2",  (Commandable)this);
		cstC =  makeCommand("setCharacterSource",  (Commandable)this);

		characterSourceTask3.setHiringCommand(cstC3);
		characterSourceTask2.setHiringCommand(cstC2);
		characterSourceTask.setHiringCommand(cstC);

		screenBoolean = new MesquiteBoolean[16];
		for (int i=0; i<16; i++)
			screenBoolean[i] = new MesquiteBoolean(false);
		screenBoolean[screen].setValue(true);
		showLegend = new MesquiteBoolean(true);
		dfQueryDone = new MesquiteBoolean(false);
		dfReduction = new MesquiteInteger(0);

		/*
		addMenuItem( "Next Character", makeCommand("nextCharacter",  (Commandable)this));
		addMenuItem( "Previous Character", makeCommand("previousCharacter",  (Commandable)this));
		addMenuItem( "Choose Character", makeCommand("chooseCharacter",  (Commandable)this));
		 */

		setXCommand = makeCommand("setX", (Commandable)this);
		setYCommand = makeCommand("setY", (Commandable)this);
		//ntxC =makeCommand("setScreen",  (Commandable)this);

		setScreen(screen);
		screenName = new MesquiteString();

		window =  new AC3ChartWindow(this);
		setModuleWindow(window);
		window.setWindowSize(400,400);

		contrastsStatsTask = (ContrastForChar)hireNamedEmployee(ContrastForChar.class, "#FelsContrWithChar");
		if (contrastsStatsTask == null)
			return sorry(getName() + " couldn't start because no calculating module obtained.");

		residualsTask = (ResidualsCalculator)hireNamedEmployee(ResidualsCalculator.class, "#ContrastResiduals");
		if (residualsTask == null)
			return sorry(getName() + " couldn't start because no calculating module for residuals obtained.");
		addCheckMenuItem(null, "Abs. Contrast vs. Standard Deviation (1,2)", makeCommand("setScreen", this).setDefaultArguments("1"), screenBoolean[1]); //one char scroll; ScatterRegDiagnostics
		addCheckMenuItem(null, "Abs. Contrast vs. Est. Node Value (3,4)", makeCommand("setScreen", this).setDefaultArguments("3"), screenBoolean[3]);//one char scroll; ScatterRegDiagnostics
		addCheckMenuItem(null, "Abs. Contrast vs. Node Height (5,6)", makeCommand("setScreen", this).setDefaultArguments("5"), screenBoolean[5]);//one char scroll; ScatterRegDiagnostics
		addCheckMenuItem(null, "Est. Node Value vs. Node Height (7,8)", makeCommand("setScreen", this).setDefaultArguments("7"), screenBoolean[7]);//one char scroll; ScatterRegDiagnostics
		//addCheckMenuItem(null, "Y Contrasts vs. X Contrasts (positivized) (9)", makeCommand("setScreen", this).setDefaultArguments("9"), screenBoolean[9]);//two char scroll; ScatterOriDiagnostics
		addCheckMenuItem(null, "Root Node Reconstructions (9A)", makeCommand("setScreen",this).setDefaultArguments("12"), screenBoolean[12]);//two char scroll; RootReconstruction with error bars
		addCheckMenuItem(null, "Conf. & Pred. Intervals (9B)",makeCommand("setScreen",this).setDefaultArguments("13"), screenBoolean[13]); //two char scroll; Regression with CI and PI intervals
		addCheckMenuItem(null, "Residual vs. Contrast (10)", makeCommand("setScreen", this).setDefaultArguments("10"), screenBoolean[10]);//two char scroll; ScatterResXDiagnost
		addCheckMenuItem(null, "Abs. Residual vs. Node Height (11)", makeCommand("setScreen", this).setDefaultArguments("11"), screenBoolean[11]);//two vert char scroll; ScatterRegDiagnostics
		 
		//modif
		addCheckMenuItem(null, "Continuous Vs Categorical (positivized) (14)",makeCommand("setScreen",this).setDefaultArguments("14"),screenBoolean[14]);//two chars; acdcp regression
		addCheckMenuItem(null,"Multiple Regression (15)",makeCommand("setScreen",this).setDefaultArguments("15"),screenBoolean[15]);//two chars; acdcp regression
		//end modif

		addMenuItem("-", null);
		addCheckMenuItem(null, "Show Legend", makeCommand("toggleShowLegend",  (Commandable)this), showLegend);
		switch (screen) {

		//modif
		case 14: {
			screenName.setValue("Contrast of Y(continuous) variable vs. positivized contrast of X(categorical) variable");
			break;
		}
		case 15: {
			screenName.setValue("(Multiple Regression) Residuals of contrast regression vs. X contrast values");
			break;
		}
		//end modif
		case 1: {
 				screenName.setValue("Absolute value of contrasts vs. their standard deviation");
 				break;
 				}
 			case 3: {
 				screenName.setValue("Absolute value of contrast vs. estimated value of their node");
 				break;
 				}
 			case 5: {
 				screenName.setValue("Absolute value of contrast vs. height of their node");
 				break;
 				}
 			case 7: {
 				screenName.setValue("Estimated value of associated node vs. height of the node");
 				break;
 				}
 			case 9: {
 				screenName.setValue("Contrasts of Y variable vs. positivized contrasts of X variable");
 				break;
 				}
 			case 12: {
 				screenName.setValue("Reconstructed root node value");
 				break;
 				}
 			case 13: {
 				screenName.setValue("Regression through root with confidence and predication intervals");
 				break;
 				}
 			case 10: {
 				screenName.setValue("Residuals of contrast regression vs. X contrast values");
 				break;
 				}
 			case 11: {
 				screenName.setValue("Absolute value of contrast residual vs. height of node");
 				break;
 				}

		default:
			screenName.setValue(contrastsStatsTask.getName()); 
		}
		if (!MesquiteThread.isScripting()) 
			window.setVisible(true);
		//The following was turned off because it needs too much work
		if (false && getEmployer() instanceof TreeWindowMaker) {
			addMenuItem( "Tree labels", makeCommand("toggleTreeLabels",  (Commandable)this));
			treeDisplay = ((TreeWindowMaker)getEmployer()).getTreeDisplay();
			nsLabeller = new NodeLabeller(this, treeDisplay, (AC3ChartWindow)window);
			nsLabeller.setShowLabels(showTreeLabels);

			treeDisplay.addExtra(nsLabeller);
		}
		makeMenu("AC3.Chart");
		resetContainingMenuBar();
		resetAllWindowsMenus();
		return true;
	}

	public void employeeQuit(MesquiteModule m){
		iQuit();
	}
	/*.................................................................................................................*/
	/*.................................................................................................................*/
	public void windowGoAway(MesquiteWindow whichWindow) {
		whichWindow.hide();
		whichWindow.dispose();
		if (nsLabeller != null)
			nsLabeller.turnOff();
		iQuit();
	}
	/*...................................................................................................................*/
	public void pointMouseDown(MesquiteChart chart, int whichPoint, MesquiteNumber valueX, MesquiteNumber valueY, int x, int y, int modifiers, String message){
		if (nsLabeller==null) {
			if (message == null)
				message = "";
			else
				message += "\n";
			int i = whichPoint;
			if (tree.nodeInTree(i) && tree.nodeIsInternal(i)) {
				message  += "Node " + i;
				if (tree instanceof MesquiteTree && ((MesquiteTree)tree).getNodeLabel(i)!=null)
					message += " (" +  ((MesquiteTree)tree).getNodeLabel(i) + ")";
				String[] results = ((AC3ChartWindow)window).getStrings(i);
				if (results !=null)
					for (int j = 0; j<results.length; j++)
						message += "\n" + results[j];  
				chart.showQuickMessage(whichPoint, x, y, message);
				return;
			}

		}
		((AC3ChartWindow)window).pointTouched(tree, whichPoint);
	}
	/*...................................................................................................................*/
	public void pointMouseUp(MesquiteChart chart, int whichPoint, int x, int y, int modifiers, String message){
		chart.hideQuickMessage();
		((AC3ChartWindow)window).pointReleased(tree, whichPoint);
	}
	/*.................................................................................................................*/
	public long toExternal(long i){
		return(CharacterStates.toExternal((int)i));
	}
	/*.................................................................................................................*/
	public long toInternal(long i){
		return(CharacterStates.toInternal((int)i));
	}
	/*.................................................................................................................*/
	public long getMin(){
		return 0;
	}
	/*.................................................................................................................*/
	/** returns the maximum index for a character on this chart */
	public long getMax(){
		if (characterSourceTask==null || currentTaxa==null)
			return 0;
		return characterSourceTask.getNumberOfCharacters(currentTaxa)-1;
	}
	//modif 
	public long getMax2(){
		if (characterSourceTask2==null || currentTaxa==null)
			return 0;
		return characterSourceTask2.getNumberOfCharacters(currentTaxa)-1;
	}
	public long getMax3(){
		if (characterSourceTask3==null || currentTaxa==null)
			return 0;
		return characterSourceTask3.getNumberOfCharacters(currentTaxa)-1;
	}
	//end modif

	/*.................................................................................................................*/
	/** This saves the state of the chart to a Snapshot */
	public Snapshot getSnapshot(MesquiteFile file) { 
		Snapshot temp = new Snapshot();
		if (window ==null)
			return null;
		temp.addLine( "getCharacterSource " , characterSourceTask);
		temp.addLine( "getCharacterSource2 " , characterSourceTask2);
		temp.addLine( "getCharacterSource3 " , characterSourceTask3);
		temp.addLine("setScreen " + screen); 

		temp.addLine("setX " + toExternal(currentX)); 
		temp.addLine("setY " + toExternal(currentY)); 

		temp.addLine("toggleShowLegend " + showLegend.toOffOnString());
		temp.addLine("toggleTreeLabels " + MesquiteBoolean.toOffOnString(showTreeLabels)); 
		temp.addLine("toggleQueryDone " + dfQueryDone.toOffOnString());
		Snapshot fromWindow = window.getSnapshot(file);
		temp.addLine("getWindow");
		temp.addLine("tell It");
		temp.incorporate(fromWindow, true);
		temp.addLine("endTell");
		temp.addLine("showWindow");
		return temp;
	}
	/*.................................................................................................................*/
	/** fireAssistantIfNeeded is a helper method for hireIfNeeded.  The ordering of the tests in the conditional
	    were reversed to take advantage of "short-circuiting" && by putting the cheaper test first */
	void fireAssistantIfNeeded(String name, String toBeHired){
		if (!name.equalsIgnoreCase(toBeHired) && charterTask.findEmployeeWithName(name)!=null)
			charterTask.doCommand("fireEmployee", name,CommandChecker.defaultChecker);
	}
	/** hireIfNeeded makes sure the appropriate calculation assistant is hired, after firing any that are
	  inappropriate.  */
	Object hireIfNeeded(String name){
		fireAssistantIfNeeded("#AC3ScatterRegDiagnostics", name);
		fireAssistantIfNeeded("#AC3ScatterResXDiagnostics", name);
		fireAssistantIfNeeded("#AC3ScatterOriDiagnostics", name);
		fireAssistantIfNeeded("#AC3ScatterRootCIPI", name);
		fireAssistantIfNeeded("#AC3ScatterRootRegCIPI",name);
		
		if (charterTask.findEmployeeWithName(name)==null) {
			Object obj = charterTask.doCommand("newAssistant", name,CommandChecker.defaultChecker);
			if (obj !=null && obj instanceof ScatterRegisterableAsst) {
				((ScatterRegisterableAsst)obj).setChartModule(this);
			}
			return obj;
		}
		else {
			return charterTask.findEmployeeWithName(name);
		}
	}
	/*.................................................................................................................*/
	/** This method sets the screen, updates the boolean array that handles check marks in the menu
	and hires the appropriate diagnostics employee */
	Object setScreen(int ic){
		screenBoolean[screen].setValue(false);
		screen = ic;
		screenBoolean[screen].setValue(true);
		Object hired = null;
		switch (ic){
		case 1:
				case 3:
				case 5:
				case 7:
				case 11: {
					hired = hireIfNeeded("#AC3ScatterRegDiagnostics");
					break;
				}

		//modif
				case 15:
		case 14:{	
			hired = hireIfNeeded("#AC3ScatterOriDiagnostics");
			break;
		}
		//end modif

		case 9: {	
					hired = hireIfNeeded("AC3#ScatterOriDiagnostics");
					break;
				}
				case 10: {
					hired = hireIfNeeded("#AC3ScatterResXDiagnostics");
					break;
				}
				case 12: {   //screen 9A
					hired = hireIfNeeded("#AC3ScatterRootCIPI");
					break;
				}
				case 13: {	//screen 9B
					hired = hireIfNeeded("#AC3ScatterRootRegCIPI");
					break;
				}
						
		}
		diagnosticsTask = (AC3ScatterAsst)hired;
		return hired;
	}
	MesquiteInteger pos = new MesquiteInteger();

	/*.................................................................................................................*/
	public Object doCommand(String commandName, String arguments, CommandChecker checker) {

		if (checker.compare(this.getClass(), "Sets module supplying continuous characters", "[name of module]", commandName, "setCharacterSource")) {//temporary
			return characterSourceTask.doCommand(commandName, arguments, checker);
		}

		else if (checker.compare(this.getClass(), "Sets whether or not the legend is shown", "[on = show; off]", commandName, "toggleShowLegend")) {
			showLegend.toggleValue(parser.getFirstToken(arguments));
			window.setLegendVisible(showLegend.getValue());
		}
		else if (checker.compare(this.getClass(), "Returns module supplying characters", null, commandName, "getCharacterSource")) {
			return characterSourceTask;
		}
		//modif
		else if (checker.compare(this.getClass(), "Returns module supplying characters", null, commandName, "getCharacterSource2")) {
			return characterSourceTask2;
		}
		else if (checker.compare(this.getClass(), "Returns module supplying characters", null, commandName, "getCharacterSource3")) {
			return characterSourceTask3;
		}
		//end modif
		//modif comments
		else if (checker.compare(this.getClass(), "Sets the screen shown", "[screen number, one of 1,3,5,7,9-13,14,15]", commandName, "setScreen")) {
			//end modif  
			int ic = MesquiteInteger.fromString(arguments);
			if (validScreen(ic)) {               
				incrementMenuResetSuppression();
				Object o = setScreen(ic);
				doCounts();
				getModuleWindow().contentsChanged();
				decrementMenuResetSuppression();
				return o;
			}
		}
		else if (checker.compare(this.getClass(), "Sets which item is shown on the x axis (appropriate where the calculator can supply a series of items, e.g. characters)", "[item number]", commandName, "setX")) {
			int ic = (int)(MesquiteInteger.fromString(arguments));
			if (!MesquiteInteger.isCombinable(ic))
				ic = MesquiteInteger.queryInteger(containerOfModule(), "Choose ", "choose:", 1); //TODO have something more intelligent here
			if (!MesquiteInteger.isCombinable(ic))
				return null;
			int icInternal = ic-1;
			if (MesquiteInteger.isCombinable(icInternal) && (icInternal>=0) && (icInternal<=getMax())) {
				currentX=icInternal;
				if (!MesquiteThread.isScripting()){
					doCounts();
				}
			}
		}
		else if (checker.compare(this.getClass(), "Sets which item is shown on the y axis (appropriate where the calculator can supply a series of items, e.g. characters)", "[item number]", commandName, "setY")) {
			int ic = (MesquiteInteger.fromString(arguments));
			if (!MesquiteInteger.isCombinable(ic))
				ic = MesquiteInteger.queryInteger(containerOfModule(), "Choose an ", "choose:", 1); //TODO have something more intelligent here
			if (!MesquiteInteger.isCombinable(ic))
				return null;
			int icInternal = ic-1;
			if (MesquiteInteger.isCombinable(icInternal) && (icInternal>=0) && (icInternal<=getMax())) {
				currentY=icInternal;
				if (!MesquiteThread.isScripting()){
					doCounts();
				}
			}
		}
		else if (checker.compare(this.getClass(), "Sets whether or not the tree labels are shown", "[on or off]", commandName, "toggleTreeLabels")) {
			//TODO: use MesquiteBoolean checkmark
			if (StringUtil.blank(arguments))
				showTreeLabels = !showTreeLabels;
			else {
				String s = ParseUtil.getFirstToken(arguments, pos);
				if ("on".equalsIgnoreCase(s))
					showTreeLabels = true;
				else if  ("off".equalsIgnoreCase(s))
					showTreeLabels = false;
			}
			if (nsLabeller!=null)
				nsLabeller.setShowLabels(showTreeLabels);

			if (treeDisplay !=null)
				treeDisplay.pleaseUpdate(false);
		} 
		else if (checker.compare(this.getClass(),"Sets the degrees of freedom adjustment", "[integer]",commandName, "setDFReduction")) {
			int dfr = (int)(MesquiteInteger.fromString(arguments));
			if (!MesquiteInteger.isCombinable(dfr))
				MesquiteMessage.warnProgrammer("setQueryDone invoked without parameter");
			else
				;
		}         
		else if (checker.compare(this.getClass(),"Sets the ask once control flag", "[on/off]",commandName, "toggleQueryDone")) {
			if (StringUtil.blank(arguments)) //not a menu command..
				MesquiteMessage.warnProgrammer("setQueryDone invoked without parameter");
			else
				dfQueryDone.toggleValue(parser.getFirstToken(arguments));
		}
		else
			return super.doCommand(commandName, arguments, checker);
		return null;
	}

	/* This checks whether the screenNumber specifies a valid screen; if new screens are added, this needs to be updated */
	private boolean validScreen(int screenNumber) {
		boolean result = false;
		if (MesquiteInteger.isCombinable(screenNumber)) {
			if (screenNumber == 1 || screenNumber == 3 || screenNumber == 5 || screenNumber == 7 || screenNumber == 9)
				result = true;
			if (screenNumber == 10 || screenNumber == 11 || screenNumber == 12 || screenNumber == 13 || screenNumber == 14 )//updated for 14
				result = true;
			if (screenNumber == 15)//updated for 15
				result = true;
		}
		return result;
	}



	long oldTreeID = -1;
	long oldTreeVersion = 0;
	/*.................................................................................................................*/
	public void setTree(Tree tree) {
		if (tree==null)
			return;
		if (!((MesquiteTree)tree).allLengthsAssigned(false)) {
			AlertDialog.notice(null,"Missing Branch Lengths", "The tree you want to analyze contains one or more branches with unassigned branch lengths. " +
			"You should go back to the tree window and specify branch lengths (e.g set to 1, Pagel's method, etc.).");
			//iQuit();
			//window.dispose();
			//return;
		}
		if (this.tree !=null && this.tree instanceof NoPolyTree && this.tree !=tree) { // about to set new tree
			((NoPolyTree)this.tree).disconnect();
		}
		if (tree instanceof NoPolyTree)
			this.tree=tree;
		else 
			this.tree = new NoPolyTree((MesquiteTree)tree);
		if (this.tree instanceof Selectionable)
			charterTask.pointsAreSelectable(true, (Selectionable)this.tree, false);
		currentTaxa = this.tree.getTaxa();
		if (oldTreeID==-1) {
			if (contrastsStatsTask==null || residualsTask == null)
				return;
			setDFQueryDone(false);
			contrastsStatsTask.initialize(tree);
			residualsTask.initialize(tree);
			characterSourceTask.initialize(tree.getTaxa());

			//modif
			characterSourceTask2.initialize(tree.getTaxa());
			characterSourceTask3.initialize(tree.getTaxa());
			//end modif

			doCounts();
		}
		else if (tree.getID() != oldTreeID || tree.getVersionNumber() != oldTreeVersion) {
			setDFQueryDone(false);
			doCounts();
		}
		else {
			if (window!=null &&((AC3ChartWindow)window).getChart()!=null ) {
				((AC3ChartWindow)window).getChart().getField().repaint(); //for selection Only
				((AC3ChartWindow)window).getChart().repaint();
			}
		}
		oldTreeID = tree.getID();
		oldTreeVersion = tree.getVersionNumber();
	}

	/*.................................................................................................................*/
	public void employeeParametersChanged(MesquiteModule employee, MesquiteModule source, Notification notification) {
		if (contrastsStatsTask!=null && residualsTask!=null) {
			doCounts();
		}
	}
	/*.................................................................................................................*/
	int numItemsX(){
		if (tree==null)
			return 0;
		return 1;
		//contrastsStatsTask.getNumberOfCharacters(tree, commandRec);
	}
	/*.................................................................................................................*/
	int numItemsY(){
		if (tree==null)
			return 0;
		return 1;
	}
	/*.................................................................................................................*/
	public CharacterDistribution getObserved1(){
		return ((AC3ChartWindow)window).observedStates;
	}
	public CharSourceCoordObed getCharSource1(){
		return this.characterSourceTask;
	}
	public CharSourceCoordObed getCharSource2(){
		return this.characterSourceTask2;
	}
	/*.................................................................................................................*/
	public CharacterDistribution getObserved2(){
		return ((AC3ChartWindow)window).observedStates2;
	}
	public int getScreen(){
		return this.screen;
	}
	/*.................................................................................................................*/
	public Tree getTree(){
		return tree;
	}

	/*.................................................................................................................*/
	public boolean getDFQueryDone(){
		return dfQueryDone.getValue();
	}

	/*.................................................................................................................*/
	public void setDFQueryDone(boolean value){
		dfQueryDone.setValue(value);
	}

	/*.................................................................................................................*/
	public MesquiteInteger getDFReduction(){
		return dfReduction;
	}

	/*.................................................................................................................*/
	public void doCounts() {
		((AC3ChartWindow)window).blankChart();
		((AC3ChartWindow)window).setTree(tree);
		((AC3ChartWindow)window).recalcChart();
		window.contentsChanged();
	}
	/*.................................................................................................................*/
	public String getName() {
		return "AC3 Diagnostic Chart";
	}
	/*.................................................................................................................*/
	public String getAuthors() {
		return "Franois Gao, Jonathan Corcy, Michel Laurin (based on PDAP)" ;
	}
	/*.................................................................................................................*/
	public String getVersion() {
		return "1.08";
	}
	/*.................................................................................................................*/

	public boolean showCitation(){
		return true;
	}
	/*.................................................................................................................*/
	public boolean isPrerelease() {
		return false;
	}
	/*.................................................................................................................*/
	public boolean requestPrimaryChoice() {
		return true;
	}

	/*.................................................................................................................*/
	/** returns an explanation of what the module does.*/
	public String getExplanation() {
		return "Shows values for nodes of tree via a scattergram." ;
	}
	/*.................................................................................................................*/
	public void endJob() {
		if (treeDisplay !=null && nsLabeller !=null)
			treeDisplay.removeExtra(nsLabeller);
		super.endJob();
	}
}

/* ======================================================================== */
/**=== Local Class BannerPanel.  ===*/

class BannerPanel extends MesquitePanel{
	public void paint(Graphics g){
		g.setColor(Color.cyan);
		g.drawString("AC3: Analysis of Continous vs. Categorical Contrasts", 16, getBounds().height-4);
	}
}
/*========================================================*/
class NodeLabeller extends TreeDisplayExtra {
	LabelsAtNodes labelsAtNodes;
	AC3ChartWindow chartWindow;
	Color brightGreen, brightbrightGreen;
	boolean showLabels = true;
	int labelDrawn = -1;

	public NodeLabeller (MesquiteModule ownerModule, TreeDisplay treeDisplay, AC3ChartWindow chartWindow) {
		super(ownerModule, treeDisplay);
		this.chartWindow =chartWindow;
		brightGreen = new Color((float)0.4, (float)1.0, (float)0.4);
		brightbrightGreen = new Color((float)0.6, (float)1.0, (float)0.6);
	}
	/*.................................................................................................................*/
	public void drawOneLabel(int N, Tree tree, Graphics g) {
		makeSureLabelsReady(tree);
		int nodeX = treeDisplay.getTreeDrawing().x[N];
		int nodeY = treeDisplay.getTreeDrawing().y[N];
		MesquiteLabel c = (MesquiteLabel)labelsAtNodes.getPanel(N);

		c.setColor(brightbrightGreen); 
		c.setText("Node " + N);  
		String[] results = chartWindow.getStrings(N);
		for (int i = 0; i<results.length; i++)
			c.addLine(results[i]);  

		c.setVisible(true);
		c.repaint();
		int w = c.getWidth(g);
		if (w+nodeX+16>treeDisplay.getBounds().width)
			c.setLocation(treeDisplay.getBounds().width- w, nodeY + 18);
		else
			c.setLocation(nodeX + 16, nodeY + 18);
		g.setColor(brightGreen);
		g.setXORMode(Color.white);
		g.drawLine(nodeX, nodeY, c.getBounds().x, c.getBounds().y);
		g.drawLine(nodeX, nodeY+1, c.getBounds().x, c.getBounds().y + 1);
		g.drawLine(nodeX, nodeY+2, c.getBounds().x, c.getBounds().y + 2);
		g.drawLine(nodeX, nodeY+3, c.getBounds().x, c.getBounds().y + 3);
		g.setPaintMode();
		g.setColor(Color.black);
		labelDrawn = N;
	}
	/*.................................................................................................................*/
	public void hideOneLabel(int N, Graphics g) {
		MesquiteLabel c = (MesquiteLabel)labelsAtNodes.getPanel(N);
		if (c==null)
			return;
		c.setVisible(false);
		if (g==null)
			return;
		int nodeX = treeDisplay.getTreeDrawing().x[N];
		int nodeY = treeDisplay.getTreeDrawing().y[N];
		g.setColor(brightGreen);
		g.setXORMode(Color.white);
		g.drawLine(nodeX, nodeY, c.getBounds().x, c.getBounds().y);
		g.drawLine(nodeX, nodeY+1, c.getBounds().x, c.getBounds().y + 1);
		g.drawLine(nodeX, nodeY+2, c.getBounds().x, c.getBounds().y + 2);
		g.drawLine(nodeX, nodeY+3, c.getBounds().x, c.getBounds().y + 3);
		g.setPaintMode();
		labelDrawn = -1;
	}
	/*.................................................................................................................*/
	private void drawLabels(int N, Tree tree, Graphics g) {
		for (int d = tree.firstDaughterOfNode(N); tree.nodeExists(d); d = tree.nextSisterOfNode(d))
			drawLabels(d, tree, g);
		drawOneLabel(N, tree, g);
	}
	/*.................................................................................................................*/
	public void hideLabels() {
		if (labelsAtNodes==null)
			return;
		Graphics g = treeDisplay.getGraphics();
		for (int N = 0; N<labelsAtNodes.getNumNodes(); N++) {
			if (N== labelDrawn)
				hideOneLabel(N, g);
			else
				labelsAtNodes.getPanel(N).setVisible(false);
		}
		if (g!=null)
			g.dispose();
	}
	/*.................................................................................................................*/
	public void setShowLabels(boolean show) {
		showLabels = show;
		if (labelsAtNodes!=null && !showLabels)
			hideLabels();

	}
	/*.................................................................................................................*/
	private void makeSureLabelsReady(Tree tree){
		if (labelsAtNodes==null) {
			labelsAtNodes = new LabelsAtNodes(ownerModule, tree.getNumNodeSpaces(), treeDisplay);
		}
		else if (labelsAtNodes.getNumNodes()!=tree.getNumNodeSpaces() ) {
			labelsAtNodes.resetNumNodes(tree.getNumNodeSpaces());
		}
	}
	/*.................................................................................................................*/
	public void drawOnTree(Tree tree, int drawnRoot, Graphics g) {
		if (showLabels) {
			makeSureLabelsReady(tree);
			drawLabels(drawnRoot, tree, g);
			g.setColor(Color.black);
		}
	}
	/*.................................................................................................................*/
	public void printOnTree(Tree tree, int drawnRoot, Graphics g) {
		drawOnTree(tree, drawnRoot, g);
	}
	/*.................................................................................................................*/
	public void setTree(Tree tree) {
	}
	/*.................................................................................................................*/
	public void turnOff() {
		super.turnOff();
		if (labelsAtNodes!=null)
			labelsAtNodes.dispose();
	}
}

/*========================================================*/
class AC3ChartWindow extends ChartWindow {
	BannerPanel banner;
	int bannerHeight = 24;
	private int windowWidth=0;
	private int windowHeight=0;
	private AC3DiagnosticChart ownerModule;
	private NumberArray checkArray,xArray, yArray, zArray;
	private boolean weightedBranches = true;
	String nameA, nameB;
	Charter charter;
	AC3Legend legend;
	Tree tree;
	DoubleMiniScroll scrollBox;
	MesquiteChart chart;
//	ContinuousDistribution observedStates, observedStates2;
//	CategoricalDistribution observedCStates,observedCStates2;
	ContinuousDistribution observedStates2;
	CategoricalDistribution observedStates;
	CharacterDistribution observedStates3;

	public AC3ChartWindow ( AC3DiagnosticChart ownerModule) {
		super(ownerModule, true); //infobar
		this.ownerModule = ownerModule;
		charter = ownerModule.charterTask.createCharter(ownerModule);
		charter.setShowNames(true);
		chart=new MesquiteChart(ownerModule, 100, 0, charter);
		setChart(chart);
		xArray = new NumberArray(3);
		//checkArray=xArray.cloneArray();
		yArray = new NumberArray(3);
		zArray = new NumberArray(3);
		chart.deassignChart();
		addToWindow(chart);
		chart.setVisible(true);
		chart.setBackground(getBackground());
		scrollBox = new DoubleMiniScroll(ownerModule.setXCommand, ownerModule.setYCommand, 0,0,0,0,0,0);
		chart.add(scrollBox);
		scrollBox.setYTitle("Y");
		scrollBox.setXTitle("X");
		//scrollBox.setLocation(10, getHeight()-10);
		scrollBox.setLocation(2, getHeight()-24- scrollBox.getBounds().height);
		banner = new BannerPanel();
		addToWindow(banner);
		banner.setLocation(0, 0);
		banner.setFontSize(18);
		banner.setVisible(true);
		banner.setBackground(Color.blue);
		legend = new AC3Legend();
		chart.getField().add(legend);
		legend.setVisible(true);
		resetTitle();
	}

	/*.................................................................................................................*/
	/** When called the window will determine its own title.  MesquiteWindows need
	to be self-titling so that when things change (names of files, blocks, etc.)
	they can reset their titles properly*/
	public void resetTitle(){
		setTitle("AC3 Diagnostic Chart");
	}

	void setLegendVisible(boolean vis){
		if (legend!=null)
			legend.setVisible(vis);
	}
	/*.................................................................................................................*/
	public Snapshot getSnapshot(MesquiteFile file) { 
		Snapshot temp = super.getSnapshot(file);
		temp.addLine( "getLegend");
		temp.addLine("tell It");

		temp.incorporate(legend.getSnapshot(file), true);
		temp.addLine("endTell");
		return temp;
	}
	/*.................................................................................................................*/
	public Object doCommand(String commandName, String arguments, CommandChecker checker) { 
		if (checker.compare(this.getClass(), "Returns the legend", null, commandName, "getLegend")) {
			return legend;
		}
		else 
			return super.doCommand(commandName, arguments, checker);
	}
	/*.................................................................................................................*/
	private void resetScrolls(){

		// one character scroll
		if (ownerModule.screen == 1 ||
				ownerModule.screen == 3 ||
				ownerModule.screen == 5 ||
				ownerModule.screen == 7 || 
				ownerModule.screen == 11) {
			scrollBox.setYTitle("Continuous Character");
			scrollBox.setXTitle("Categorical Character");
			int min = (int)ownerModule.toExternal(ownerModule.getMin());
			int max = (int)ownerModule.toExternal(ownerModule.getMax());
			int max2 = (int)ownerModule.toExternal(ownerModule.getMax2());
			scrollBox.setXValues(min,  (int)ownerModule.toExternal(ownerModule.currentX), max2);
			scrollBox.setYValues(min,  (int)ownerModule.toExternal(ownerModule.currentY), max);
			scrollBox.setYVisible(true);
			scrollBox.setXVisible(true);
		}
		// two character scroll, one for X and one for Y
		else if (ownerModule.screen == 9 ||ownerModule.screen == 10 ) {
			scrollBox.setXTitle("Character");
			scrollBox.setYTitle("Character");
			int min = (int)ownerModule.toExternal(ownerModule.getMin());
			int max = (int)ownerModule.toExternal(ownerModule.getMax());
			scrollBox.setXValues(min,  (int)ownerModule.toExternal(ownerModule.currentX), max);
			scrollBox.setXVisible(true);
			scrollBox.setYValues(min,  (int)ownerModule.toExternal(ownerModule.currentY), max);
			scrollBox.setYVisible(true);
		}
		else if (ownerModule.screen == 15){
			scrollBox.setXTitle("Character");
			int min = (int)ownerModule.toExternal(ownerModule.getMin());
			int max3 = (int)ownerModule.toExternal(ownerModule.getMax3());
			scrollBox.setXValues(min,  (int)ownerModule.toExternal(ownerModule.currentX), max3);
			scrollBox.setXVisible(true);
			scrollBox.setYVisible(false);
		}
		//		 two character scroll, one for X and one for Y for acdcp diagnostic chart
		else if (ownerModule.screen == 14 ) {
			scrollBox.setXTitle("Character");
			scrollBox.setYTitle("Character");
			int min = (int)ownerModule.toExternal(ownerModule.getMin());
			int max = (int)ownerModule.toExternal(ownerModule.getMax());
			int max2 = (int)ownerModule.toExternal(ownerModule.getMax2());
			scrollBox.setXValues(min,  (int)ownerModule.toExternal(ownerModule.currentX), max2);
			scrollBox.setXVisible(true);
			scrollBox.setYValues(min,  (int)ownerModule.toExternal(ownerModule.currentY), max);
			scrollBox.setYVisible(true);
		}
		// two character scroll, both for Y
		else {
			scrollBox.setXTitle("Character");
			scrollBox.setYTitle("Character");
			int min = (int)ownerModule.toExternal(ownerModule.getMin());
			int max = (int)ownerModule.toExternal(ownerModule.getMax());
			scrollBox.setXValues(min,  (int)ownerModule.toExternal(ownerModule.currentX), max);
			scrollBox.setXVisible(true);
			scrollBox.setYValues(min,  (int)ownerModule.toExternal(ownerModule.currentY), max);
			scrollBox.setYVisible(true);
		}

		/*/X (horizontal)
		if (ownerModule.contrastsStatsTask instanceof Incrementable){
			Incrementable inc = (Incrementable)ownerModule.contrastsStatsTask;
			scrollBox.setXTitle("Character");
			int min = (int)toExternal(0);
			int max = (int)toExternal(inc.getMax(commandRec));
			scrollBox.setXValues(min,  (int)toExternal(ownerModule.currentX), max);
			scrollBox.setXVisible(true);
		}
		else {
			scrollBox.setXValues(0,0,0);
			scrollBox.setXVisible(false);
		}
		//Y (vertical)
		if (ownerModule.contrastsStatsTask instanceof Incrementable){
			Incrementable inc = (Incrementable)ownerModule.contrastsStatsTask;
			scrollBox.setYTitle("Character");
			int min = (int)toExternal(0);
			int max = (int)toExternal(inc.getMax(commandRec));
			scrollBox.setYValues(min,  (int)toExternal(ownerModule.currentY), max);
			scrollBox.setYVisible(true);
		}
		else {
			scrollBox.setYValues(0,0,0);
			scrollBox.setYVisible(false);
		}
		 */
	}
	/*.................................................................................................................*/
	public void setVisible(boolean vis){
		windowResized();
		super.setVisible(vis);
	}
	/*.................................................................................................................*
	public void show(){
		setVisible(true);
	}
	/*.................................................................................................................*/
	public void setCharter(Charter charter) {
		charter.setShowNames(true);
		chart.setCharter(charter);
	}
	/*.................................................................................................................*/
	public String[] getStrings(int index) {
		return new String[] {"X (" + ownerModule.contrastsStatsTask.getParameters() +
				"): " + xArray.toString(index), 
				"Y (" + ownerModule.contrastsStatsTask.getParameters() + 
				"): " + yArray.toString(index)};
	}
	/*.................................................................................................................*/
	public void setTree(Tree tree) {
		if (this.tree !=null && this.tree instanceof NoPolyTree && this.tree !=tree) { // about to set new tree
			((NoPolyTree)this.tree).disconnect();
		}
		if (tree instanceof NoPolyTree){
			this.tree=tree;
		}
		else {
			this.tree = new NoPolyTree((MesquiteTree)tree);
		}
		if (this.tree instanceof Selectionable)
			ownerModule.charterTask.pointsAreSelectable(true, (Selectionable)this.tree, false);
	}
	/*.................................................................................................................*/
	public void recalcChart() {
		chart.deassignChart();
		if (ownerModule.contrastsStatsTask != null && ownerModule.residualsTask != null) {
			MesquiteNumber resultX = new MesquiteNumber();
			MesquiteNumber resultY = new MesquiteNumber();
			//MesquiteNumber resultZ = new MesquiteNumber();
			resetScrolls();
			int numNodes = tree.getNumNodeSpaces();
			xArray.resetSize(numNodes);
			checkArray=xArray.cloneArray();
			yArray.resetSize(numNodes);
			//zArray.resetSize(numNodes);
			//if (ownerModule.contrastsStatsTask instanceof Incrementable)
			//	((Incrementable)ownerModule.contrastsStatsTask).setCurrent(ownerModule.currentX, commandRec); 
			//x values
			int iChar = ownerModule.currentX;  // actually this is *only* true for bivariate plots (screens 9&10).
			int iChar2= ownerModule.currentX;
			int iChar3= ownerModule.currentX;
			switch (ownerModule.screen){
			case 1:{
				ownerModule.contrastsStatsTask.setOption(ContrastForChar.SD);
				iChar = ownerModule.currentY;    //currentY is the character specified in the single scroll - both axes are functions of this.
				iChar2=ownerModule.currentX;
				legend.setMessage("Plot of absolute value of contrast vs. standard deviation");
				break;
			}
			case 3:{
				ownerModule.contrastsStatsTask.setOption(ContrastForChar.NODEVALUE);
				iChar = ownerModule.currentY; 
				iChar2=ownerModule.currentX;
				legend.setMessage("Plot of absolute value of contrast vs. estimated value of associated node");
				break;
			}
			case 5:{
				ownerModule.contrastsStatsTask.setOption(ContrastForChar.CORRECTEDHEIGHT);
				iChar = ownerModule.currentY; 
				iChar2=ownerModule.currentX;
				legend.setMessage("Plot of absolute value of contrast vs. height of associated node");
				break;
			}
			case 7:{
				ownerModule.contrastsStatsTask.setOption(ContrastForChar.CORRECTEDHEIGHT);
				iChar = ownerModule.currentY; 
				iChar2=ownerModule.currentX;
				legend.setMessage("Plot of estimated value of associated node vs. its height");
				break;
			}
			case 9:{
				ownerModule.contrastsStatsTask.setOption(ContrastForChar.CONTRAST);
				iChar = ownerModule.currentX;   // character specified in the horizontal scroll 
				legend.setMessage("Plot of contrasts vs. positivized contrasts");
				break;
			}
			case 10:{
				ownerModule.contrastsStatsTask.setOption(ContrastForChar.ABSCONTRAST);
				iChar = ownerModule.currentX;  // character specified in the horizontal scroll
				legend.setMessage("Plot of residuals from screen 9 vs. contrast values");
				break;
			}
			case 11:{
				ownerModule.contrastsStatsTask.setOption(ContrastForChar.CORRECTEDHEIGHT);
				iChar = ownerModule.currentX; 
				legend.setMessage("Plot of absolute value of residual vs. height of associated node");
				break;
			}
			case 12:{
				ownerModule.contrastsStatsTask.setOption(ContrastForChar.TIPS);
				iChar = ownerModule.currentX; 
				legend.setMessage("Plot of tip values and confidence intervals for reconstructed root values");//Peter: what of this depends on diagnostics being active?
				break;
			}
			case 13:{
				ownerModule.contrastsStatsTask.setOption(ContrastForChar.TIPS);
				iChar = ownerModule.currentX;  //?
				legend.setMessage("Plot of phylogenetically correct (OLS) regression and " +  //Peter: what of this depends on diagnostics being active?
						"associated CI's (red-95%, orange-90%) and PI (green-95%, cyan-90%), "+
						"mapped onto the original tip data space. (see Garland and Ives 2000, Am. Nat. " + 
				"155:346-364).");
			}
			//modified here
			case 14:{
				ownerModule.contrastsStatsTask.setOption(ContrastForChar.SD);
				iChar = ownerModule.currentX; 
				legend.setMessage("Plot of positivized continuous contrast vs categorical contrasts");
			}
			case 15:{
				ownerModule.contrastsStatsTask.setOption(ContrastForChar.CONTRAST);
				iChar3 = ownerModule.currentX; 
				legend.setMessage("Multiple Regression");
			}
			}

			if (ownerModule.screen ==14 ) { 
				//end modif	
					ownerModule.contrastsStatsTask.setOption(ContrastForChar.SD);
					observedStates = (CategoricalDistribution)ownerModule.characterSourceTask2.getCharacter(tree, iChar);
				CharacterHistory ch = null;
				ch=	observedStates.adjustHistorySize(tree, ch);
				ch.adjustSize(tree);
				ch.setObservedStates(observedStates);
			//	observedStates.getDefaultModel(proj, paradigm)
				//ownerModule.ac3States.calculateStates(tree, observedStates, ch, null , commandRec);
				ownerModule.ac3Model.calculateStates(tree, observedStates, ch, null,null, null);
				observedStates =(CategoricalDistribution)ch;
				AC3ContrastCalculator calcul=new AC3ContrastCalculator(tree,observedStates);
				double[] result= calcul.contrastCalculation();
				if(calcul.getNodesToRecalculate()!=null){
				checkArray=calcul.getNodesToRecalculate().cloneArray();
				yArray=calcul.getNodesToRecalculate().cloneArray();
				}
				ownerModule.contrastsStatsTask.calculateNumbers(tree,observedStates2, checkArray, null);  // This does the work for the x-axis
				for (int i=0;i<result.length;i++){
					if(MesquiteDouble.isCombinable(result[i]) && checkArray.getDouble(i)!=0)
						xArray.setValue(i, result[i]/checkArray.getDouble(i));
					else
						xArray.setValue(i,result[i]);
				}
				checkArray=xArray.cloneArray();
				
					
					}
			else if (ownerModule.screen ==15 ) { 
				//end modif	
				try{
				observedStates3 = (CategoricalDistribution)ownerModule.characterSourceTask3.getCharacter(tree, iChar3);
				CharacterHistory ch = null;
				ch=	observedStates3.adjustHistorySize(tree, ch);
				ch.adjustSize(tree);
				ch.setObservedStates(observedStates3);
				ownerModule.ac3Model.calculateStates(tree, observedStates3, ch, null,null, null);
				observedStates3 =(CategoricalDistribution)ch;
				AC3ContrastCalculator calcul=new AC3ContrastCalculator(tree,(CategoricalDistribution)observedStates3);
				double[] result= calcul.contrastCalculation();
				if(calcul.getNodesToRecalculate()!=null){
					yArray=calcul.getNodesToRecalculate().cloneArray();
				checkArray=xArray.cloneArray();
				}
				ownerModule.contrastsStatsTask.setOption(ContrastForChar.SD);
				ownerModule.contrastsStatsTask.calculateNumbers(tree,observedStates2, checkArray, null);  // This does the work for the x-axis
				for (int i=0;i<result.length;i++){
					if(MesquiteDouble.isCombinable(result[i]) && checkArray.getDouble(i)!=0)
						xArray.setValue(i, result[i]/checkArray.getDouble(i));
					else
						xArray.setValue(i,result[i]);
				}
				checkArray=xArray.cloneArray();
				}
				catch(ClassCastException e){
					ownerModule.contrastsStatsTask.setOption(ContrastForChar.CONTRAST);
					observedStates3 = (ContinuousDistribution)ownerModule.characterSourceTask3.getCharacter(tree, iChar3);
					ownerModule.contrastsStatsTask.calculateNumbers(tree,(ContinuousDistribution)observedStates3, xArray, null);  // This does the work for the x-axis
					chart.setXAxisName("Contrasts of X variable");   //set to correct name
					checkArray=xArray.cloneArray();
				}
					}
			
			else if (ownerModule.screen ==12 || ownerModule.screen==13 ) { 
				observedStates = (CategoricalDistribution)ownerModule.characterSourceTask2.getCharacter(tree, iChar);
				CharacterHistory ch = null;
				ch=	observedStates.adjustHistorySize(tree, ch);
				ch.adjustSize(tree);
				ch.setObservedStates(observedStates);
				//ownerModule.ac3States.calculateStates(tree, observedStates, ch, null, commandRec);
				ownerModule.ac3Model.calculateStates(tree, observedStates, ch, null,null, null);
				observedStates =(CategoricalDistribution)ch;
				AC3ContrastCalculator calcul=new AC3ContrastCalculator(tree,observedStates);
				double[] result= calcul.fillTips(tree.getRoot());
				for (int i=0;i<result.length;i++)
					xArray.setValue(i, result[i]);
				checkArray=xArray.cloneArray();
				chart.setXAxisName("AC3 X categorical states");
				//end modif
					}
			else		{	//if one character calculation, don't get character unless needed
				observedStates = (CategoricalDistribution)ownerModule.characterSourceTask2.getCharacter(tree, iChar2);
				CharacterHistory ch = null;
				ch=	observedStates.adjustHistorySize(tree, ch);
				ch.adjustSize(tree);
				ch.setObservedStates(observedStates);
				//ownerModule.ac3States.calculateStates(tree, observedStates, ch, null, commandRec);
				ownerModule.ac3Model.calculateStates(tree, observedStates, ch, null,null, null);
				observedStates =(CategoricalDistribution)ch;
				AC3ContrastCalculator calcul=new AC3ContrastCalculator(tree,observedStates);
				double[] result= calcul.contrastCalculation();
				for (int i=0;i<result.length;i++)
				checkArray.setValue(i, result[i]);
				if(calcul.getNodesToRecalculate()!=null){
					yArray=calcul.getNodesToRecalculate().cloneArray();
					xArray=yArray.cloneArray();
				}
				//end modif
				observedStates2 = (ContinuousDistribution)ownerModule.characterSourceTask.getCharacter(tree, iChar);
				ownerModule.contrastsStatsTask.calculateNumbers(tree, observedStates2, xArray, null);  // This does the work for the x-axis
				//modif
				//end modif
				chart.setXAxisName(ownerModule.contrastsStatsTask.getParameters());   //set to correct name
				}
			if (ownerModule.contrastsStatsTask instanceof Incrementable)
				((Incrementable)ownerModule.contrastsStatsTask).setCurrent(ownerModule.currentY);
			//y values
			switch (ownerModule.screen){
			case 1:
			case 3:
			case 5: {
				ownerModule.contrastsStatsTask.setOption(ContrastForChar.ABSCONTRAST);
				break;
			}
			case 7:{
				ownerModule.contrastsStatsTask.setOption(ContrastForChar.NODEVALUE);
				break;
			}
		/*	case 14:{
				ownerModule.contrastsStatsTask.setOption(ContrastForChar.ACDCP,commandRec);
				break;
			}*/
			case 14:
			case 9:{
				ownerModule.contrastsStatsTask.setOption(ContrastForChar.CONTRAST);
				break;
			}
			case 15:
			case 10:{
				ownerModule.contrastsStatsTask.setOption(ContrastForChar.RESIDUAL);
				ownerModule.residualsTask.setOption(0);//raw
				break;
			}
			case 11:{
				ownerModule.contrastsStatsTask.setOption(ContrastForChar.ABSRESIDUAL);
				ownerModule.residualsTask.setOption(1);//absolute
				break;
			}
			case 12:
			case 13:{
				ownerModule.contrastsStatsTask.setOption(ContrastForChar.TIPS);
				break;
			}
			}
			if (true ) { 
				//if one character calculation, don't get character unless needed  TODO: make this work
				observedStates2 = (ContinuousDistribution)ownerModule.characterSourceTask.getCharacter(tree, ownerModule.currentY);
				//modif here

			}
			switch (ownerModule.screen) {
			case 1:
			case 3:
			case 5:
			case 7:
			case 9:
			case 14:{
				//ownerModule.contrastsStatsTask.setXCharForReport(ownerModule.currentX,commandRec); 
				//ownerModule.contrastsStatsTask.setYCharForReport(ownerModule.currentY,commandRec); 
				ownerModule.contrastsStatsTask.calculateNumbers(tree, observedStates2, yArray, null);  // This does the work for the y-axis
				break;
			}
			//modif
			case 15:
			case 10:
			case 11: {

				ownerModule.residualsTask.calculateNumbers(tree, observedStates, observedStates2, yArray, null);
				break;
			}
			case 12:
			case 13: {
				ownerModule.contrastsStatsTask.setXCharForReport(ownerModule.currentX); 
				ownerModule.contrastsStatsTask.setYCharForReport(ownerModule.currentY); 
				ownerModule.contrastsStatsTask.calculateNumbers(tree, observedStates2, yArray, null);
				break;
			}
			}
			// maybe call it here??

			chart.setYAxisName(ownerModule.contrastsStatsTask.getParameters());  //TODO: this should use string passed back from calculateNumbers ???
			boolean showTips;
			showTips = false;
			if ((ownerModule.screen == 12) || (ownerModule.screen == 13)) 
				showTips = true;
			for (int i=0; i<numNodes; i++) {
				if (tree.nodeInTree(i) && (showTips || tree.nodeIsInternal(i))) { //todo: have toggle for internal only
					if ((ownerModule.screen == 12) || (ownerModule.screen == 13)) {
						xArray.placeValue(i, resultX);
						yArray.placeValue(i, resultY);
					}
					else{
					if(checkArray.isCombinable(i))
					xArray.placeValue(i, resultX);
					else
						checkArray.placeValue(i,resultX);
					yArray.placeValue(i, resultY);}
					if (ownerModule.screen == 9 || ownerModule.screen == 14 || ownerModule.screen == 15){ //positivizing for screen 9
						chart.setXAxisName("Positivized Contrasts - CAUTION if any values are zero");   //force to correct name
						if (ownerModule.screen != 15)
						chart.setYAxisName("Contrasts Positivized on the X variable (Garland et al. 1992)");
						else 
							chart.setYAxisName("Residuals of Source 1 vs Source 2");
						if (resultX.isNegative()) {
							resultY.changeSign();
							resultX.abs();
						}
					}
					//zArray.placeValue(i, resultZ);

					int point = chart.addPoint(resultX, resultY);
					//int point = chart.addPoint(resultX, resultY, resultZ);
					String label = tree.getNodeLabel(i);
					if (label == null)
						chart.setName(point, Integer.toString(i));
					else
						chart.setName(point, label);
					//chart.setCategory(point, nodeJustInternal(tree, i));
				}
				else { //these are here to ensure number of points is in parallel with numbering of nodes, for selection and other purposes
					resultX.setToUnassigned();
					int point = chart.addPoint(resultX, resultX);
				}
			}
			xArray.deassignArray();
			yArray.deassignArray();
			contentsChanged();
			chart.munch();
			if (ownerModule.diagnosticsTask != null) {
				String s = ownerModule.diagnosticsTask.getLegendText();
				if (!StringUtil.blank(s))
					legend.appendToMessage("\n\n" + s);
			}
		}
		if (ownerModule.treeDisplay!=null)
			ownerModule.treeDisplay.pleaseUpdate(false);
	}
	/*.................................................................................................................*/
	public void pointTouched(Tree tree, int which) {
		if (ownerModule.nsLabeller==null) {
			return;
		}
		//the following looks wrong; which should be the node number directly; no need to search
		int count = 0;
		Graphics g = ownerModule.treeDisplay.getGraphics();
		if (g!=null) {
			for (int i=0; i<tree.getNumNodeSpaces(); i++) {
				if (tree.nodeInTree(i) && tree.nodeIsInternal(i)) {
					if (count == which) {
						ownerModule.nsLabeller.drawOneLabel(i, tree, g);
					}
					count++;
				}
			}
			g.dispose();
		}

	}
	/*.................................................................................................................*/
	public void pointReleased(Tree tree, int which) {
		if (ownerModule.nsLabeller==null)
			return;
		if (ownerModule.nsLabeller.showLabels)
			return;
		int count = 0;
		Graphics g = ownerModule.treeDisplay.getGraphics();
		if (g!=null) {
			for (int i=0; i<tree.getNumNodeSpaces(); i++) {
				if (tree.nodeInTree(i) && tree.nodeIsInternal(i)) {
					if (count == which) {
						ownerModule.nsLabeller.hideOneLabel(i, g);
					}
					count++;
				}
			}
			g.dispose();
		}
		ownerModule.nsLabeller.hideLabels();
	}

	/*............................................................................................................*/
	public void windowResized() {
		if (MesquiteWindow.checkDoomed(this))
			return;
		if (chart!=null && (windowWidth!=getWidth() || windowHeight!=getHeight())) {
			windowWidth=getWidth();
			windowHeight=getHeight();
			if (chart!=null){
				chart.setLocation(0, bannerHeight+20);
				chart.setChartSize(windowWidth-20, windowHeight-20 - bannerHeight);
			}
			if (scrollBox !=null)
				scrollBox.setLocation(2, getHeight()-24- bannerHeight- scrollBox.getBounds().height);
			if (banner !=null)
				banner.setSize(windowWidth-20, bannerHeight);
		}
		MesquiteWindow.uncheckDoomed(this);
	}

	public void dispose(){
		if (tree !=null && tree instanceof NoPolyTree)
			((NoPolyTree)tree).disconnect();
		super.dispose();
	}
}

/** This defines the legend box for the plot
 */
class AC3Legend extends ResizableLegend {
	TextArea specsBox;
	private static final int grabHeight = 6;
	private static final int bottomHeight = 4;
	private static final int defaultSpecsHeight = (12 + MesquiteModule.textEdgeCompensationHeight) * 3;
	private static final int defaultLegendWidth=142;
	private static final int defaultLegendHeight=grabHeight + defaultSpecsHeight + 2 + bottomHeight;
	private int specsHeight = defaultSpecsHeight;
	public AC3Legend() {
		super(defaultLegendWidth, defaultLegendHeight);
		setVisible(false);
		setOffsetX(0);
		setOffsetY(0);

		setBackground(Color.white);
		setLayout(null);
		specsBox = new TextArea(" ", 2, 2, TextArea.SCROLLBARS_VERTICAL_ONLY);
		specsBox.setEditable(false);
		add(specsBox);


		setSize(legendWidth,legendHeight);
		specsBox.setVisible(true);
	}

	public void paint(Graphics g) {
		if (MesquiteWindow.checkDoomed(this))
			return;
		g.setColor(Color.cyan);
		g.drawRect(0, 0, legendWidth-1, legendHeight-1);
		g.fillRect(0, 0, legendWidth-1, grabHeight);
		g.fillRect(legendWidth-grabHeight, legendHeight-grabHeight, grabHeight, grabHeight);
		g.setColor(Color.black);

		MesquiteWindow.uncheckDoomed(this);
	}

	public void sizeChanged(int w, int h){
		specsHeight  = h - (grabHeight + 2 + bottomHeight);
		specsBox.setBounds(1,grabHeight,w-2, specsHeight);
		legendHeight = h;
		legendWidth = w;

	}

	public void setSize(int w, int h){
		sizeChanged(w, h);
		super.setSize(w, h);
	}
	public void setBounds(int x, int y, int w, int h){
		sizeChanged(w, h);
		super.setBounds(x, y, w, h);
	}
	public void setMessage(String s) {
		specsBox.setText(s);
	}
	public void appendToMessage(String s) {
		specsBox.setText(specsBox.getText() + s);
	}
}
