/* AC3 Mesquite module */

package mesquite.ac3.AC3ScatterOriDiagnostics;

import java.awt.*;
import java.applet.Applet;
import java.util.*;

import mesquite.ac3.lib.*;
import mesquite.categ.lib.CategoricalDistribution;
import mesquite.cont.lib.ContinuousDistribution;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;
import JSci.maths.statistics.*;    // for p-value calculations

public class AC3ScatterOriDiagnostics extends ScatterRegisterableAsst  {
    Vector extras;
    //modif
   public AC32CTStatPakSource sps;
    public MesquiteBoolean showLines;
    public AC3FileReporter FICReportTask;

    public AC3TreeWinAsstC chartModule;
    //modif end
    public MesquiteBoolean paneHasFICData,paneHasContrastData,paneHasStdContrastData;
    public MesquiteBoolean spaceConvertFICNames;

    private AC32CTStatPak myStats = null;
    public MesquiteBoolean verboseQueryMode;   

    public boolean startJob(String arguments, Object condition, boolean hiredByName) {
        extras = new Vector();
        showLines = new MesquiteBoolean(true);
        paneHasFICData = new MesquiteBoolean(false);
        paneHasContrastData = new MesquiteBoolean(false);
        paneHasStdContrastData = new MesquiteBoolean(false);
        spaceConvertFICNames = new MesquiteBoolean(true);
        addCheckMenuItem(null, "Show Regression Lines", makeCommand("toggleShowLines",  (Commandable)this), showLines);
        addMenuItem(null, "Generate File of Independent Contrasts (FIC)", makeCommand("writeFIC", (Commandable)this));
        addCheckMenuItem(null, "Show data from File of Inpendent Contrasts in text pane", makeCommand("toggleFICinPane",(Commandable)this),paneHasFICData);
        addCheckMenuItem(null, "Show standardized contrasts of continuous characters and raw contrasts of discrete character in text pane", makeCommand("toggleContrastsinPane",(Commandable)this),paneHasContrastData);
        addCheckMenuItem(null, "Show Standardized contrast values in text pane", makeCommand("toggleStdContrastsinPane",(Commandable)this),paneHasStdContrastData);
        addMenuItem(null,"Show columns in File of Independent Contrasts (FIC)",makeCommand("showFIC",(Commandable)this));
        addMenuItem(null, "Root Calculation Query Mode...", makeCommand("setVerboseQueryMode",  (Commandable)this));
        addMenuItem(null, "Set Polytomy DF Reduction...", makeCommand("setDFReduction",  (Commandable)this)); 
        //addMenuItem(null, "Generate File containing Distance Matrix",makeCommand("writeDistances",(Commandable)this));
        sps = (AC32CTStatPakSource)hireNamedEmployee(AC32CTStatPakSource.class, "#SPOriDiagnostics");
        verboseQueryMode = new MesquiteBoolean(false);
        loadPreferences();
        FICReportTask = (AC3FileReporter)hireNamedEmployee(AC3FileReporter.class, "#FICReport");
        if (FICReportTask == null)
            return sorry(getName() + " couldn't start because no FIC reporter module obtained.");       
        return true;
    }

    /*.................................................................................................................*/	
    public String getLegendText(){
        String s = "";
        if (extras.size() == 1) {
            SODExtra extra = (SODExtra)extras.elementAt(0);
            String b = extra.myStats.getLegendText();
            if (!StringUtil.blank(b))
                s +=  b + "\n";
        }
        //if (s.length() >0)
        //	s += "\n";
        s += "Regression lines through origin: \n" +
        "    Black is ordinary least squares (OLS) \n" +
        "    Green is major axis (MA) \n" +
        "    Red is reduced major axis \n";
        return s;
    }
    
    /*-----------------------------------------*/
    public void processPreferencesFromFile(String[] prefs) {
        if (prefs != null && prefs.length > 0) {
            verboseQueryMode.setValue("v".equals(prefs[0]));
        }
    }

    /*-----------------------------------------*/
    public String[] preparePreferencesForFile() {
        String[] prefs = new String[1];
        if (verboseQueryMode.getValue())
            prefs[0] = "v";
        else
            prefs[0] = "n";
        return prefs;
    }

    /*-----------------------------------------*/
    public Snapshot getSnapshot(MesquiteFile file) {
        Snapshot temp = new Snapshot();
        temp.addLine("toggleShowLines " + showLines.toOffOnString());
        return temp;
    }

    /*.................................................................................................................*/
    /**
	@param String commandName
	@param String argments
	@param CommandRecord commandRec
	@param CommandChecker checker
	Dispatches a script command
     */
    public Object doCommand(String commandName, String arguments, CommandChecker checker) {
        if (checker.compare(this.getClass(), "Sets whether or not the regression line is shown", "[on = show; off]", commandName, "toggleShowLines")) {
            showLines.toggleValue(parser.getFirstToken(arguments));
            parametersChanged();
        }
        else if (checker.compare(this.getClass(), "Generate File of Independent Contrasts (FIC)",null,commandName,"writeFIC")) {
            MesquiteFile FICfile = new MesquiteFile();
            FICReportTask.writeReport(FICfile,chartModule.getObserved1(),chartModule.getObserved2(),chartModule.getTree(),arguments);	
        }
        else if (checker.compare(this.getClass(),"Sets whether or not data from File of Independent Contrasts is shown in the text pane","[on = show; off]",commandName,"toggleFICinPane")) {
            paneHasFICData.toggleValue(parser.getFirstToken(arguments));
            if (paneHasFICData.getValue()){
                spaceConvertFICNames.setValue(AlertDialog.query(chartModule.containerOfModule(),"Convert Spaces?", "Convert Spaces in node names to underscores (_); useful if pasting into Excel","Convert","Leave Unchanged"));
            }
            parametersChanged();
        }	
        else if (checker.compare(this.getClass(),"Sets whether or not Contrasts are shown in the text pane","[on = show; off]",commandName,"toggleContrastsinPane")) {
            paneHasContrastData.toggleValue(parser.getFirstToken(arguments));
           // if (paneHasContrastData.getValue()){
             //   spaceConvertFICNames.setValue(AlertDialog.query(chartModule.containerOfModule(),"Convert Spaces?", "Convert Spaces in node names to underscores (_); useful if pasting into Excel","Convert","Leave Unchanged"));
            //}
            parametersChanged();
        }	
        else if (checker.compare(this.getClass(),"Sets whether or not Standardized Contrasts are shown in the text pane","[on = show; off]",commandName,"toggleStdContrastsinPane")) {
            paneHasStdContrastData.toggleValue(parser.getFirstToken(arguments));
           // if (paneHasContrastData.getValue()){
             //   spaceConvertFICNames.setValue(AlertDialog.query(chartModule.containerOfModule(),"Convert Spaces?", "Convert Spaces in node names to underscores (_); useful if pasting into Excel","Convert","Leave Unchanged"));
            //}
            parametersChanged();
        }	
        else if (checker.compare(this.getClass(), "Gets correl coef", null, commandName, "getCorrelationCoefficient")) {
            if (extras.size() == 1){
                SODExtra extra = (SODExtra)extras.elementAt(0);
                return new MesquiteNumber(extra.getValue(commandName));
            }
        }
        else if (checker.compare(this.getClass(), "Gets p value", null, commandName, "getPValue")) {
            if (extras.size() == 1){
                SODExtra extra = (SODExtra)extras.elementAt(0);
                return new MesquiteNumber(extra.getValue(commandName));
            }
        }
        else if (checker.compare(this.getClass(), "Show columns in File of Independent Contrasts (FIC)",null,commandName,"showFIC")) {
            String description = 
                "FIC files contain the following columns:\n\n" +
                " 1.  Name of Contrast as Node - Node (e.g., 'Tm - Ta')\n" +
                " 2.  Name of Node at Base of Contrast\n" +
                " 3.  Unstandardized Contrast for Trait 1\n" +
                " 4.  Unstandardized Contrast for Trait 2\n" +
                " 5.  Standard Deviation of Contrast\n" +
                "     = square root of sum of corrected branch lengths\n" +
                " 6.  Nodal Value for Trait 1\n" +
                " 7.  Nodal Value for Trait 2\n" +
                " 8.  Uncorrected Height of Base Node from Root of Tree\n" +
                " 9.  Corrected Height of Base Node from Root of Tree\n" +
                "		  (as used in screens 5, 6, 7, 8, and 11)\n" +
                "10.  Average Uncorrected Height of Daughter Nodes\n" +
                "11.  Average Corrected Height of Daughter Nodes\n" +
                "12.  Name of First Daughter Branch (Descendant-Ancestor)\n" +
                "13.  Corrected Length of First Daughter Branch\n" +
                "14.  Name of Second Daughter Branch (Descendant-Ancestor)\n" +
                "15.  Corrected Length of Second Daughter Branch\n";
            AlertDialog.notice(null,"FIC Columns",description);
        }
        else if (checker.compare(this.getClass(),"Sets the number of DF's reduced because of polytomies","[number of dof]", commandName, "setDFReduction")) {
            if (myStats == null){
               MesquiteMessage.warnProgrammer("ScatterOriDiagnostics statPak should not be null");
               return null;
            }
            MesquiteInteger pos = new MesquiteInteger(0);
            int dfReduction = MesquiteInteger.fromFirstToken(arguments, pos);
            if (!MesquiteInteger.isCombinable(dfReduction))
                myStats.queryDFReduce(chartModule.getTree(), true);
            else
                myStats.setDFReduction(dfReduction);
            parametersChanged();
        }
        else if (checker.compare(this.getClass(),"Sets the query mode to verbose or not", "[on/off]",commandName, "setVerboseQueryMode")) {
            if (StringUtil.blank(arguments)) {
                MesquiteBoolean ok = MesquiteBoolean.queryCheckBox(
                            containerOfModule(),
                            "Set Verbose Query Preference",
                            "In verbose mode you will be asked for settings for AC3 root reconstruction frequently; otherwise you change the settings using menu items",
                            "Use Verbose Mode", verboseQueryMode.getValue());
                if (!ok.isUnassigned()) {
                    verboseQueryMode.setValue(ok.getValue());
                    storePreferences();
                }
            } 
            else {
                verboseQueryMode.toggleValue(parser.getFirstToken(arguments));
                storePreferences();
            }
        }
        else
            return super.doCommand(commandName, arguments, checker);
        return null;
    }

    /*-----------------------------------------*/
    public boolean canHireMoreThanOnce () {
        return false;
    }

    /*-----------------------------------------*/
    public void setChartModule(AC3TreeWinAsstC module){
        chartModule = module;
        myStats = sps.getStatPak(chartModule.getDFReduction());
    }

    /** */
    public ChartExtra createExtra(MesquiteChart chart){
        ChartExtra s =  new SODExtra(this, chart, myStats);
        extras.addElement(s);
        return s;
    }

    /** */
    public void endJob(){
        if (extras!=null){
            for (int i=0; i<extras.size(); i++){
                ChartExtra extra = (ChartExtra)extras.elementAt(i);
                extra.turnOff();
            }
        }
        super.endJob();
    }
    /*.................................................................................................................*/
    public String getName(){
        return "Scattergram Diagnostics using Regression through the Origin for AC3";
    }

    /*.................................................................................................................*/
    public String getAuthors() {
        return "Franois Gao, Jonathan Corcy, Michel Laurin (taken from PDAP)" ;
    }
    /*.................................................................................................................*/
    public String getVersion() {
        return "1.08";
    }

    /** returns an explanation of what the module does.*/
    public String getExplanation() {
        return "Calculates statistics for scattergram regression lines forced through the origin." ;
    }

    /*.................................................................................................................*/
    public boolean isPrerelease() {
        return false;
    }
}


class SODExtra extends ChartExtra {
    AC32CTStatPak myStats;
    // first create and initialize the new rootStatPak
    AC32CTStatPak oldStats = null;
    Tree lastTree;
    long lastVersion;

    AC3ScatterOriDiagnostics ownerModule;
    public SODExtra(AC3ScatterOriDiagnostics ownerModule, MesquiteChart chart, AC32CTStatPak stats){
        super(ownerModule, chart);
        this.ownerModule = ownerModule;
        myStats = stats;
    }

    /**Do any calculations needed*/
    public boolean doCalculations(){
        if (chart!=null && ownerModule.chartModule != null){
            Tree tree = ownerModule.chartModule.getTree();
/*            if (tree != lastTree){
                lastTree = tree;
                lastVersion = tree.getVersionNumber();
                ownerModule.chartModule.setDFQueryDone(false);
            }
            else if (lastVersion != tree.getVersionNumber()){
                lastVersion = tree.getVersionNumber();
                ownerModule.chartModule.setDFQueryDone(false);                
            }
*/                
            int root = tree.getRoot();
            AC3TreeWinAsstC cM = ownerModule.chartModule;
            // ugly band-aid
            if (myStats == null)
                myStats = ownerModule.sps.getStatPak(cM.getDFReduction());

            //pass the tree 'context' into the statPak so it can properly handle missing data
            myStats.setTree(tree);
            myStats.setObserved1((CategoricalDistribution)cM.getObserved1());
            myStats.setObserved2((ContinuousDistribution)cM.getObserved2());
            myStats.countInternalNodes(tree, root); // count the internal nodes,
            // to set number
            int zlb = myStats.countZLBs(tree, root); // Zero-Length-Branches are what AC3 calls "polytomies"; count to
                                           // inform the user
            // now ask the user to set df Reductions
            if (!MesquiteThread.isScripting()){
                if (ownerModule.verboseQueryMode.getValue() ||
                    (!ownerModule.chartModule.getDFQueryDone() && zlb > 0))
                    myStats.queryDFReduce(tree, true);
                ownerModule.chartModule.setDFQueryDone(true);
            }
            if(cM.getScreen()!=15){
                boolean munch = myStats.doCalculations(chart.getXArray(), chart.getYArray(), chart);
                if (munch)
                    chart.munch();
                }
                else{
                    boolean munch = myStats.doCalculations(chart.getXArray(), chart.getYArray(),true, chart);
                    if (munch)
                        chart.munch();
                    }
        }
        return false;
    }

    /**
	@param String commandName specifies the field from the statPak to return
	@return double the value requested or MesquiteDouble.unassigned
     */
    public double getValue(String commandName) {
        if (commandName.equals("getCorrelationCoefficient")) {
            return myStats.getCorrelationCoefficient();
        }
        if (commandName.equals("getPValue")) {
            return myStats.getPValue();
        }
        else return MesquiteDouble.unassigned;
    }

    /**draw on the chart*/
    // This draws the colored line segments for the three different regression models that will
    // be dumped below.  Unlike ScatterRegDiagnostics, these should all go through the origin, so
    // this method starts the lines at the origin, rather than min[x], which is should be positive,
    // since contrasts are positivized (and should rarely be zero).
    public void drawOnChart(Graphics g){
        if (myStats!=null && ownerModule.showLines.getValue())
            myStats.drawOnChart(g, chart);
    }

    /**print on the chart*/
    public void printOnChart(Graphics g){
        drawOnChart(g);
    }

    public String writeOnChart(){
        if (myStats!=null) {
        	String tmp="",tmp2="",tmp3="";
        	
            if (ownerModule.paneHasFICData.getValue()) {
                ownerModule.FICReportTask.pushConvertSpaces(ownerModule.spaceConvertFICNames.getValue());
                tmp = ownerModule.FICReportTask.reportToString(ownerModule.chartModule.getObserved1(),
                        ownerModule.chartModule.getObserved2(),
                        ownerModule.chartModule.getTree(),
                        "\n",
                        " ");	
                ownerModule.FICReportTask.popConvertSpaces();
             //   return tmp + "\n\n" + myStats.flst();
            }
           if (ownerModule.paneHasContrastData.getValue()) {
               // ownerModule.FICReportTask.pushConvertSpaces(ownerModule.spaceConvertFICNames.getValue());
                tmp2 = ownerModule.FICReportTask.reportContrastToString(ownerModule.chartModule.getObserved1(),
                        ownerModule.chartModule.getCharSource1(),
                        ownerModule.chartModule.getTree(),
                        "\n",
                        " ");	
                ownerModule.FICReportTask.popConvertSpaces();
            //    return tmp + "\n\n" + myStats.flst();
            }
           if (ownerModule.paneHasStdContrastData.getValue()) {
               // ownerModule.FICReportTask.pushConvertSpaces(ownerModule.spaceConvertFICNames.getValue());
                tmp3 = ownerModule.FICReportTask.reportContrastToString(ownerModule.chartModule.getObserved1(),
                        ownerModule.chartModule.getCharSource1(),
                        ownerModule.chartModule.getTree(),
                        "\n",
                        "\t");	
                ownerModule.FICReportTask.popConvertSpaces();
            //    return tmp + "\n\n" + myStats.flst();
            }
            //else
                return tmp + "\n\n"+tmp2+"\n\n"+tmp3+myStats.flst();
           
        }		
        else return "";
    }

    /**to inform ChartExtra that cursor has just entered point*/
    public void cursorEnterPoint(int point, int exactPoint, Graphics g){
        //Debugg.println("enter point " + point);
    }
    /**to inform ChartExtra that cursor has just exited point*/
    public void cursorExitPoint(int point, int exactPoint, Graphics g){
        //Debugg.println("exit point " + point);
    }
    /**to inform ChartExtra that cursor has just touched point*/
    public void cursorTouchPoint(int point, int exactPoint, Graphics g){
        //Debugg.println("touch point " + point);
    }

}