/* AC3 Mesquite module */

package mesquite.ac3.AC3ScatterRegDiagnostics;

import java.awt.*;
import java.util.*;

import mesquite.ac3.lib.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;
import JSci.maths.statistics.*;    // for p-value calculations


public class AC3ScatterRegDiagnostics extends AC3ScatterAsst  {
	Vector extras;
	StatPakSource sps;
	public boolean startJob(String arguments, Object condition, boolean hiredByName) {
		extras = new Vector();
		sps = (StatPakSource)hireNamedEmployee(StatPakSource.class, "#SPRegDiagnostics");
		
		return true;
	}
	/** 
	@return String containing legend text
	*/
	public String getLegendText(){
		return "Black line is ordinary least squares regression.";
	}
	/**
	@return boolean - always false
	*/
	public boolean canHireMoreThanOnce () {
		return false;
	}
	/**
	@param String commandName
	@param String argments
	@param CommandRecord commandRec
	@param CommandChecker checker
	Dispatches a script command
	*/
	public Object doCommand(String commandName, String arguments, CommandChecker checker) {
 		if (checker.compare(this.getClass(), "Gets correl coef", null, commandName, "getCorrelationCoefficient")) {
 			if (extras.size() == 1){
 				SRDExtra extra = (SRDExtra)extras.elementAt(0);
 				return new MesquiteNumber(extra.getValue(commandName));
 			}
 		}
 		else if (checker.compare(this.getClass(), "Gets p value", null, commandName, "getPValue")) {
 			if (extras.size() == 1){
 				SRDExtra extra = (SRDExtra)extras.elementAt(0);
 				return new MesquiteNumber(extra.getValue(commandName));
 			}
 		}
 		else
 			return super.doCommand(commandName, arguments, checker);
 		return null;
	}

	/**
	*/
	public ChartExtra createExtra(MesquiteChart chart){
		ChartExtra s = new SRDExtra(this, chart, sps);
		extras.addElement(s);
		return s;
	}
	
	/** returns module name */
	public String getName(){
		return "Scattergram Regression Diagnostics for AC3";
	}
		
    /*.................................................................................................................*/
    public String getAuthors() {
    	return "Franois Gao, Jonathan Corcy, Michel Laurin (taken from PDAP)" ;
    }
    /*.................................................................................................................*/
	/** returns module version */
    public String getVersion() {
    	return "1.08";
    }
	/*.................................................................................................................*/
  	public boolean isPrerelease() {
		return false;
   	}
	/*.................................................................................................................*/
	public boolean isSubstantive() {
		return false;
	}
	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Calculates statistics and regression lines for values on a scattergram." ;
   	 }
	/*.................................................................................................................*/
	/** cleans up extras when the job ends */
	public void endJob(){
		if (extras != null) {
			for (int i=0; i<extras.size(); i++){
				ChartExtra extra = (ChartExtra)extras.elementAt(i);
				extra.turnOff();
			}
		}
		super.endJob();
	}
}

/** */
class SRDExtra extends ChartExtra {
	AC3StatPak myStats;
	StatPakSource statPakSource;
	public SRDExtra(MesquiteModule ownerModule, MesquiteChart chart, StatPakSource sps){
		super(ownerModule, chart);
		statPakSource = sps;
	}
	/**Do any calculations needed*/
	public boolean doCalculations(){
		if (chart!=null){
			myStats = statPakSource.getStatPak();			
			boolean munch = myStats.doCalculations(chart.getXArray(), chart.getYArray(), chart);
			if (munch)
				chart.munch();
		}
		return false;
	}
	
	/**draw on the chart*/
	// This draws the colored line segments for the three different regression models that will
	// be dumped below.  In this module, lines are drawn over the range of x values, compare with
	// the method in ScatterOriDiagnostics.
	public void drawOnChart(Graphics g){
		if (myStats!=null)
			myStats.drawOnChart(g, chart);
		
	}
	/**print on the chart*/
	public void printOnChart(Graphics g){
		drawOnChart(g);
	}
	
	public String writeOnChart(){
		return myStats.flst();		
	}
	/**
	@param String commandName specifies the field from the statPak to return
	@return double the value requested or MesquiteDouble.unassigned
	*/
	public double getValue(String commandName) {
		if (commandName.equals("getCorrelationCoefficient")) {
			return myStats.getCorrelationCoefficient();
		}
		if (commandName.equals("getPValue")) {
			return myStats.getPValue();
		}
		else return MesquiteDouble.unassigned;
	}
	
	/**to inform ChartExtra that cursor has just entered point*/
	public void cursorEnterPoint(int point, int exactPoint, Graphics g){
		//Debugg.println("enter point " + point);
	}
	/**to inform ChartExtra that cursor has just exited point*/
	public void cursorExitPoint(int point, int exactPoint, Graphics g){
		//Debugg.println("exit point " + point);
	}
	/**to inform ChartExtra that cursor has just touched point*/
	public void cursorTouchPoint(int point, int exactPoint, Graphics g){
		//Debugg.println("touch point " + point);
	}


}