/* AC3 Mesquite module */

package mesquite.ac3.AC3ScatterResXDiagnostics;


import java.awt.*;
import java.applet.Applet;
import java.util.*;

import mesquite.ac3.lib.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;

public class AC3ScatterResXDiagnostics extends AC3ScatterAsst  {
	Vector extras;
	public boolean startJob(String arguments, Object condition, boolean hiredByName) {
		extras = new Vector();
		return true;
	}
	
	public String getLegendText(){
		return "";
	}
	public boolean canHireMoreThanOnce () {
		return false;
	}

	public boolean getUserChooseable(){
		return false;
	}
	public ChartExtra createExtra(MesquiteChart chart){
		ChartExtra s =  new SRXDExtra(this, chart);
		extras.addElement(s);
		return s;
	}
	
	public String getName(){
		return "Scattergram Diagnostics for residuals vs. contrasts for AC3";
		}

	public String getExplanation() {
		return "Calculates summary statistics for a plot of contrast residuals against x-value contrasts.";
	}
	
        /*.................................................................................................................*/
         public String getAuthors() {
                return "Franois Gao, Jonathan Corcy, Michel Laurin (taken from PDAP)" ;
         }
        /*.................................................................................................................*/
	/** returns module version */
         public String getVersion() {
                return "1.08";
         }
	/*.................................................................................................................*/
  	 public boolean isPrerelease() {
		return false;
   	 }
	/*.................................................................................................................*/
	public void endJob(){
		if (extras != null) {
			for (int i=0; i<extras.size(); i++){
				ChartExtra extra = (ChartExtra)extras.elementAt(i);
				extra.turnOff();
			}
		}
		super.endJob();
	}
} //class ScatterResXDiagnostics


class SRXDExtra extends ChartExtra {
	oriStatPak myStats;
	
	public SRXDExtra(MesquiteModule ownerModule, MesquiteChart chart){
		super(ownerModule, chart);
	}
	
	/**Do any calculations needed*/
	public boolean doCalculations(){
		if (chart!=null){
			if (chart.isMinimumYConstrained() || chart.isMaximumYConstrained()){
		      		chart.deConstrainMinimumX();
				chart.deConstrainMaximumX();
				chart.deConstrainMinimumY();
				chart.deConstrainMaximumY();
				//chart.deConstrainMinimumZ();
				//chart.deConstrainMaximumZ();
			}
			myStats = new oriStatPak();    
			boolean munch = myStats.doCalculations(chart.getXArray(), chart.getYArray(), chart);
			if (munch)
				chart.munch();			
		}
		return false;
	}
	
	/**draw on the chart*/
	public void drawOnChart(Graphics g){
		if (myStats!=null)
			myStats.drawOnChart(g, chart);
	}

	/**print on the chart*/
	public void printOnChart(Graphics g){
		drawOnChart(g);
	}
	
	public String writeOnChart(){
		return myStats.flst();		
	}
	
	/**to inform ChartExtra that cursor has just entered point*/
	public void cursorEnterPoint(int point, int exactPoint, Graphics g){
		//Debugg.println("enter point " + point);
	}
	/**to inform ChartExtra that cursor has just exited point*/
	public void cursorExitPoint(int point, int exactPoint, Graphics g){
		//Debugg.println("exit point " + point);
	}
	/**to inform ChartExtra that cursor has just touched point*/
	public void cursorTouchPoint(int point, int exactPoint, Graphics g){
		//Debugg.println("touch point " + point);
	}
	
// simplified version (maybe remove?)
public class oriStatPak extends AC3StatPak{

         double     mean;      // only for y variable
         double     cov;
         double     corr;
         int        number;     //total number of data points
         
	// several fields are overridden         
	protected oriStatPak()
	{
		min = new double[]  {MesquiteInteger.infinite , MesquiteInteger.infinite};
		max = new double[] { MesquiteInteger.negInfinite, MesquiteInteger.negInfinite};
		variance = new double[] {0.0, 0.0};
		mean = 0.0;
		number = 0;
	}
	/** This is the main calculating procedure */
	public boolean doCalculations(NumberArray xValues, NumberArray yValues, MesquiteChart chart){
		getBounds(xValues,yValues);
		getMean(yValues, xValues);   // xValues not calculated, just used for flagging bad y values.
		mean /= number;
		getVars(xValues,yValues);
		variance[0] /= number-1;
		variance[1] /= number-1;
		corr /= number-1;
		cov = corr;
		if ((variance[0] != 0) && (variance[1] != 0))
         		corr = corr/Math.sqrt(variance[0]*variance[1]);
      		else
         		corr = 0;
		return false;
	}

	/** since min and max are zero based, maybe the names data0 and data1 
	will be less confusing */
	public void getBounds(NumberArray data0, NumberArray data1){
		for (int i = 0; i < data0.getSize(); i++)	
		if (data0.isCombinable(i) && data1.isCombinable(i)) {
			number++;
			if (data0.getDouble(i) > max[0])
				max[0] = data0.getDouble(i);
			if (data1.getDouble(i) > max[1])
				max[1] = data1.getDouble(i);
			if (data0.getDouble(i) < min[0])
				min[0] = data0.getDouble(i);
			if (data1.getDouble(i) < min[1])
				min[1] = data1.getDouble(i);
		}
	}

	/** Dummy */
	public double getPValue() {
		return 0.0;  // no stats here (yet)
	}

	/** Calculate the mean of the y values (contrast residuals) 
	@param: NumberArray yData */
	public void getMean(NumberArray yData, NumberArray xData){
		for (int i = 0; i < yData.getSize(); i++)
		if (yData.isCombinable(i) && xData.isCombinable(i))  // If x is bad, so is y, so don't count it 
			mean += yData.getDouble(i);
	}

	/** This method calculates the variances 
	(honoring the assumption that mean(x) = 0)
	@param: NumberArray data0
	@param: NumberArray data1 */
	public void getVars(NumberArray data0, NumberArray data1) {
		for (int i = 0; i < data0.getSize(); i++)
		if (data0.isCombinable(i) && data1.isCombinable(i)) {
			double dev0 = data0.getDouble(i);
			double dev1 = data1.getDouble(i)-mean;    // y values have a real mean
			variance[0] += dev0*dev0;
			variance[1] += dev1*dev1;
			corr += dev0*dev1;
		}
	}

	/*............................................................*/
	/** This method returns the string that is displayed on the bottom of the window 
	@return string */
	public String flst() {
		StringBuffer results = new StringBuffer(500);
		results.append("Summary statistics for residuals, which assume a true mean of zero.\n\n");
		results.append("Number of contrasts: " + number);
		results.append("\nMinimum of X,Y coordinates: " + min[0] + ", " + min[1]);
		results.append("\nMaximum of X,Y coordinates: " + max[0] + ", " + max[1]);
		results.append("\nMean of X,Y coordinates: zero, " + mean);
		results.append("\nVariance of X,Y coordinates: " + variance[0] + ", " + variance[1]);
		results.append("\nCovariance: " + cov);
		results.append("\nPearson Product-Moment Correlation Coefficient: " + corr + "\n");
		return results.toString();
   	}

	/** This method returns the string for the legend window.
	@return String the legend string for this window */	
	public String getLegendText() {
		StringBuffer results = new StringBuffer(100);
		results.append("Number of contrasts: " + number);
		results.append("\nPearson Product-Moment Correlation Coefficient: " + corr + "\n");
		return results.toString();
	}
	
	public void setDrawingOption(int option){
		//stub for now
	}
		
	/**draw on the chart*/
	public void drawOnChart(Graphics g, MesquiteChart chart){
		Charter c = chart.getCharter();
		int startX = c.xToPixel(min[0],chart);
		int startY =  c.yToPixel(0,chart);
		int endX = c.xToPixel(max[0],chart);
		int endY = c.yToPixel(0,chart);
		g.setColor(Color.black);
		g.drawLine(startX, startY,endX,endY);
		// insert code for thicker lines here
	}
	}
}
