/* Mesquite module */

package mesquite.ac3.AC3ScatterRootCIPI;

import java.awt.*;
import java.applet.Applet;
import java.util.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;
import mesquite.ac3.lib.*;
import mesquite.cont.lib.*;
import JSci.maths.statistics.*;    // for p-value calculations


public class AC3ScatterRootCIPI extends ScatterRegisterableAsst  {
	Vector extras;
	RootRegressionCalculator rootStatTask;
	AC3TreeWinAsstC chartModule;
	public boolean startJob(String arguments, Object condition, boolean hiredByName) {
		extras = new Vector();
		rootStatTask = (RootRegressionCalculator)hireNamedEmployee(RootRegressionCalculator.class, "#RootReconstructions");
		if (rootStatTask == null)
			return sorry(getName() + " couldn't start because no calculating module for root regression obtained.");
		return true;
	}
	
	/*-----------------------------------------*/
	/** */
	public String getLegendText(){
		return "";
	}
	
	/*-----------------------------------------*/
	/** Avoid problems with multiple hiring */
	public boolean canHireMoreThanOnce () {
		return false;
	}

	/*-----------------------------------------*/
	/** 
	@param AC3TreeWinAsstc
	*/
	public void setChartModule(AC3TreeWinAsstC module){
		chartModule = module;
        rootStatTask.setChartModule(module);
	}

	/*-----------------------------------------*/
	/**
	@param MesquiteChart 
	@return ChartExtra
	creates a new chart extra; also attaches the chart to
	the rootStatTask which needs to draw
	*/
	public ChartExtra createExtra(MesquiteChart chart){
		rootStatTask.setChart(chart);
		ChartExtra s = new SRDExtra(this, chart);
		extras.addElement(s);
		return s;
	}	

	
	/*-----------------------------------------*/
	/**
	@return boolean false
	*/
	public boolean getUserChooseable(){
		return false;
	}
	
	/*-----------------------------------------*/
	/**
	@return String
	*/
	public String getName(){
		return "Scattergram Root Reconstruction Confidence Intervals for AC3";
	}
    /*.................................................................................................................*/
    public String getAuthors() {
    	return "Franois Gao, Jonathan Corcy, Michel Laurin (taken from PDAP)" ;
    }
    /*.................................................................................................................*/
	/** 
	@return String module version
	*/
    public String getVersion() {
    	return "1.08";
    }
	/*.................................................................................................................*/
  	public boolean isPrerelease() {
		return false;
   	}
	/*-----------------------------------------*/
	public void endJob(){
		if (extras != null) {
			for (int i=0; i<extras.size(); i++){
				ChartExtra extra = (ChartExtra)extras.elementAt(i);
				extra.turnOff();
			}
		}
		super.endJob();
	}

} //end class scatterRootCIPI


/*-----------------------------------------*/
class SRDExtra extends ChartExtra {
	CharacterDistribution cd1;
	CharacterDistribution cd2;
	Tree tree;
    MesquiteInteger dfReduce;
	public SRDExtra(MesquiteModule ownerModule, MesquiteChart chart){
		super(ownerModule, chart);
	}
    /*-----------------------------------------*/
	/**
	@param CommandRecord
	@return Boolean always false
	collects the characters and the tree and passes them to the rootStatTask to calculate
	statistics.
	*/
	public boolean doCalculations(){ 
		if (((AC3ScatterRootCIPI)ownerModule).chartModule == null)
			return false;
		cd1 = ((AC3ScatterRootCIPI)ownerModule).chartModule.getObserved1();
		cd2 = ((AC3ScatterRootCIPI)ownerModule).chartModule.getObserved2();
		tree = ((AC3ScatterRootCIPI)ownerModule).chartModule.getTree();
        dfReduce = ((AC3ScatterRootCIPI)ownerModule).chartModule.getDFReduction();
		// Here the root diagonstics module should be consulted to calculate what is needed
		NumberArray result = new NumberArray(1);
		MesquiteString resultString = new MesquiteString("");
		AC32CTStatPak rootStats = ((AC3ScatterRootCIPI)ownerModule).rootStatTask.getStatPak();
		rootStats.setDrawingOption(mesquite.ac3.lib.AC32CTStatPak.ROOTCI);
		((AC3ScatterRootCIPI)ownerModule).rootStatTask.calculateNumbers(tree,cd1,cd2,result,resultString);
		return false;
	}
	
    /*-----------------------------------------*/
	/**
	@param Graphics awt representation of the display pane/page
	tells the rootStatTask's statPak to draw the appropriate 'target' display
	*/
	public void drawOnChart(Graphics g){
		AC32CTStatPak rootStats = ((AC3ScatterRootCIPI)ownerModule).rootStatTask.getStatPak();
		if (rootStats == null)
			return;
		rootStats.drawOnChart(g,chart);
	}
		
    /*-----------------------------------------*/
	/**
	@param Graphics awt representation of the output page
	pass through entry for printing the chart*/
	public void printOnChart(Graphics g){
		drawOnChart(g);
	}
	
    /*-----------------------------------------*/
    /** This returns the string to dump on the text pane of the AC3 window.  It just gets the string returned by
        the rootStatPak. */
	public String writeOnChart(){
		AC32CTStatPak rootStats = ((AC3ScatterRootCIPI)ownerModule).rootStatTask.getStatPak();
		if (rootStats != null)
			return rootStats.flst();
		else
			return "Error: no root statistics were calculated.";
	}
	
    /*-----------------------------------------*/
	/** Entrypoint to inform ChartExtra that cursor has just entered a plot on the scatterplot.  Future use?*/
	public void cursorEnterPoint(int point, int exactPoint, Graphics g){
		//Debugg.println("enter point " + point);
	}
    /*-----------------------------------------*/
	/** Entrypoint to inform ChartExtra that cursor has just exited a plot on the scatterplot.  Future use?*/
	public void cursorExitPoint(int point, int exactPoint, Graphics g){
		//Debugg.println("exit point " + point);
	}
	/**to inform ChartExtra that cursor has just touched point*/
	public void cursorTouchPoint(int point, int exactPoint, Graphics g){
		//Debugg.println("touch point " + point);
	}
}



