/* AC3 Mesquite module */

package mesquite.ac3.AC3ScatterRootRegCIPI;
/*~~  */

import java.awt.*;
import java.util.*;

import mesquite.ac3.lib.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;


public class AC3ScatterRootRegCIPI extends ScatterRegisterableAsst  {
	Vector extras;
	RootRegressionCalculator rootStatTask;
	AC3TreeWinAsstC chartModule;
	public boolean startJob(String arguments, Object condition, boolean hiredByName) {
		extras = new Vector();
		rootStatTask = (RootRegressionCalculator)hireNamedEmployee(RootRegressionCalculator.class, "#RootedRegression");
		if (rootStatTask == null)
			return sorry(getName() + " couldn't start because no calculating module for root regression obtained.");
		return true;
	}
	
        /*.................................................................................................................*/
	public String getLegendText(){
		return "";
	}
        /*.................................................................................................................*/
	public boolean getUserChooseable(){
		return false;
	}
        /*.................................................................................................................*/
	public boolean canHireMoreThanOnce () {
		return false;
	}

    /*.................................................................................................................*/
	public void setChartModule(AC3TreeWinAsstC module){
		chartModule = module;
        rootStatTask.setChartModule(module);
	}

    /*.................................................................................................................*/
	public ChartExtra createExtra(MesquiteChart chart){
		rootStatTask.setChart(chart);
		ChartExtra s = new SRDExtra(this, chart);
		extras.addElement(s);
		return s;
	}
	
        /*.................................................................................................................*/
	public String getName(){
		return "Tip Scattergram with Root Regression Confidence and Prediction Intervals for AC3";
	}
		
        /*.................................................................................................................*/
         public String getAuthors() {
                return "Franois Gao, Jonathan Corcy, Michel Laurin (taken from PDAP)" ;
         }
        /*.................................................................................................................*/
	/** returns module version */
         public String getVersion() {
                return "1.08";
         }
	/*.................................................................................................................*/
  	 public boolean isPrerelease() {
		return false;
   	 }
   	 /** This makes sure all the extras are turned off if the job is ending */
	public void endJob(){
		if (extras != null) {
			for (int i=0; i<extras.size(); i++){
				ChartExtra extra = (ChartExtra)extras.elementAt(i);
				extra.turnOff();
			}
		}
		super.endJob();
	}
}


class SRDExtra extends ChartExtra {
	CharacterDistribution cd1;
	CharacterDistribution cd2;
	Tree tree;
    MesquiteInteger dfReduction;
	public SRDExtra(MesquiteModule ownerModule, MesquiteChart chart){
		super(ownerModule, chart);
	}
	/* Peter: Note this is a different convention from that I'm suggesting for the other Scatter Diagnostics.  Your convention is related
	/**Do any calculations needed*/
	public boolean doCalculations(){ 
	// here you can put the main calculations. 
		if (((AC3ScatterRootRegCIPI)ownerModule).chartModule == null)
			return false;
		cd1 = ((AC3ScatterRootRegCIPI)ownerModule).chartModule.getObserved1();
		cd2 = ((AC3ScatterRootRegCIPI)ownerModule).chartModule.getObserved2();
		tree = ((AC3ScatterRootRegCIPI)ownerModule).chartModule.getTree();
        dfReduction = ((AC3ScatterRootRegCIPI)ownerModule).chartModule.getDFReduction();
		// Here the root diagnostics module should be consulted to calculate what is needed
		NumberArray result = new NumberArray(1);
		MesquiteString resultString = new MesquiteString("");
		((AC3ScatterRootRegCIPI)ownerModule).rootStatTask.calculateNumbers(tree,cd1,cd2,result,resultString);
		AC32CTStatPak rootStats = ((AC3ScatterRootRegCIPI)ownerModule).rootStatTask.getStatPak();
		rootStats.setDrawingOption(AC3StatPak.REGRESSIONCIPI); 
		//Need to store results so as to be called later for drawing & writing
		return false;
	}
	
	/**draw on the chart*/
	// This tells the statpak to draw the OLS regression line and curves representing the confidence and prediation intervals.
	public void drawOnChart(Graphics g){
		AC32CTStatPak rootStats = ((AC3ScatterRootRegCIPI)ownerModule).rootStatTask.getStatPak();
		if (rootStats == null)
			return;
		rootStats.drawOnChart(g,chart);
}
	/**print on the chart*/
	public void printOnChart(Graphics g){
		drawOnChart(g);
	}
	
	public String writeOnChart(){
		AC32CTStatPak rootStats = ((AC3ScatterRootRegCIPI)ownerModule).rootStatTask.getStatPak();
		if (rootStats != null)
			return rootStats.flst();
		else
			return "";
	}
	
	/**to inform ChartExtra that cursor has just entered point*/
	public void cursorEnterPoint(int point, int exactPoint, Graphics g){
		//Debugg.println("enter point " + point);
	}
	/**to inform ChartExtra that cursor has just exited point*/
	public void cursorExitPoint(int point, int exactPoint, Graphics g){
		//Debugg.println("exit point " + point);
	}
	/**to inform ChartExtra that cursor has just touched point*/
	public void cursorTouchPoint(int point, int exactPoint, Graphics g){
		//Debugg.println("touch point " + point);
	}
}



