/* AC3 Mesquite module */

package mesquite.ac3.CIReport;
/*~~  */

import mesquite.ac3.lib.*;
import mesquite.lib.*;
import mesquite.lib.characters.CharacterDistribution;
import mesquite.lib.duties.CharSourceCoordObed;


/** */
public class CIReport extends AC3FileReporter {
	private AC32CTStatPak statPak;


	/*.................................................................................................................*/
    public boolean startJob(String arguments, Object condition, boolean hiredByName) {
        return true;  
    }

 	/*...............................................................................*/
 	public void setStatPak(AC3StatPak s){
 		if (s instanceof AC32CTStatPak)
 			this.statPak = (AC32CTStatPak)s;
 		else
 			MesquiteMessage.printStackTrace("Wrong statPak passed to CIReport.setStatPak");
 	}
 	
                
    /*.................................................................................................................*/
	/** Always true, because an CI file always has tables.
	*/
    public boolean hasTables() {
    	return true;
    }

    /*.................................................................................................................*/
    /**
     *  ask for formatting options 
     * @param dataSelected boolean true to indicate saving only the selected columns of a character matrix 
     * @param taxaSelected boolean true to indicate saving only the selected taxa
     * @return boolean true when button pressed?
     * */
    public boolean getReportOptions(boolean dataSelected, boolean taxaSelected){
        MesquiteInteger buttonPressed = new MesquiteInteger(1);
        AC3ReporterDialog reportDialog = new AC3ReporterDialog(this,containerOfModule(), "CI Report Options", buttonPressed);
        reportDialog.addDefaultPanels(dataSelected, taxaSelected,hasTables());
        boolean ok = (reportDialog.query(dataSelected,taxaSelected,convertSpaces)==0);
        reportDialog.dispose();
        return ok;
    }


    /*.................................................................................................................*/
	// This actually writes the report.
    public void writeReport(MesquiteFile file,CharacterDistribution data1,CharacterDistribution data2,Tree theTree, String arguments) { //todo: this should be passed scripting
		if (!getReportOptions(false,false))
			return;
		String output = reportToString(data1,data2,theTree);
		String name = getProject().getHomeFileName();
		if (name==null)
			name = "untitled.ci";
		else 
			name = stripNex(name) + ".ci";
		saveReportFile(output,arguments,name);
    }

    /*.................................................................................................................*/
	/** This actually writes the report. */
    public String reportToString(CharacterDistribution data1,CharacterDistribution data2,Tree theTree) { 
        if (statPak == null) 
           return "No CI data generated because no RootStatPak available";
		return statPak.CIString(getLineEnding(),getTableDelimiter(),convertSpaces);
    }
    
    /*.................................................................................................................*/
    public String reportToString(CharacterDistribution data1,CharacterDistribution data2,Tree theTree,String lineDelimiter, String columnDelimiter) { 
        if (statPak == null) 
           return "No CI data generated because no RootStatPak available";
		return statPak.CIString(lineDelimiter,columnDelimiter,convertSpaces);
	}    

    /*.................................................................................................................*/
    public String getName() {
        return "CI (AC3)";
    }
    /*.................................................................................................................*/
    public String getAuthors() {
        return "Franois Gao, Jonathan Corcy, Michel Laurin (taken from PDAP)";
    }
    /*.................................................................................................................*/
    public String getVersion() {
        return "1.08";
    }
	/*.................................................................................................................*/
  	public boolean isPrerelease() {
		return false;
   	}
	/*.................................................................................................................*/
	public boolean isSubstantive(){
		return false;
	}   	 
    /*.................................................................................................................*/
    /** returns an explanation of what the module does.*/
    public String getExplanation() {
        return "Generates CI format report files as described in Garland et al. 1999" ;
    }
    /*.................................................................................................................*/
    /** returns current parameters, for logging etc..*/
    public String getParameters() {
        return "" ;
    }

	public String reportContrastToString(CharacterDistribution data1, CharSourceCoordObed data2, Tree theTree) {
		// TODO Auto-generated method stub
		return null;
	}

	public String reportContrastToString(CharacterDistribution data1, CharSourceCoordObed data2, Tree theTree, String lineDelimiter, String columnDelimiter) {
		// TODO Auto-generated method stub
		return null;
	}
}

