/* AC3 Mesquite module */
package mesquite.ac3.ContrastResiduals;
/*~~  */

import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.ac3.lib.*;
import mesquite.categ.lib.CategoricalDistribution;
import mesquite.cont.lib.*;


/* ======================================================================== */
/** This class is responsible for calculating the residuals from a regression of contrasts against 
        contrasts.  As with any such regression, it must go through the origin.
        This is a bivariate contrast statistic.  As such, it needs to prune its tree to handle missing
        data. */
public class ContrastResiduals extends ResidualsCalculator {
	private Tree lastUsedTree = null;
	private long lastTreeVersion=-1;
	private int absolute = ContrastForChar.CONTRAST;
	private boolean statsOK = true;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, boolean hiredByName) {
		statsOK = true;
 		return true;
  	 }

	/*.................................................................................................................*/
 	/** returns current parameters, for logging etc...
 	see comments for  setOption 
 	@return String*/
    	 public String getParameters() {
		if (absolute == ContrastForChar.ABSCONTRAST)
			return "(absolute value)";
		else
			return "(raw)";
   	 }
	/*.................................................................................................................*/
 	 public void initialize(Tree tree){
 	 }

	/** Allows the calculation option to be set and checked */
	public void setOption(int option){
		if ((option == ContrastForChar.ABSCONTRAST) ||
		     (option == ContrastForChar.CONTRAST))
			absolute = option;	
		else
			MesquiteMessage.warnProgrammer("ContrastResiduals got a bad option: " + option);			
	}
	
	
	//private int numNodes;
	private boolean [] validNodeLeft = null;
	private boolean [] validNodeRight = null;
	
	// is it char1/char2 or left vs. right ?
	// Changes here!!!!
	// Construction in progress for future release
	private void pruneTest(Tree tree, ContinuousDistribution data1, ContinuousDistribution data2, int node, MesquiteBoolean good){
		for (int daughter = tree.firstDaughterOfNode(node); tree.nodeExists(daughter); daughter = tree.nextSisterOfNode(daughter))
			pruneTest(tree, data1, data2, daughter, good);
		if (tree.nodeIsInternal(node)) {
		   int left = tree.firstDaughterOfNode(node);
		   int right = tree.lastDaughterOfNode(node);
		   if (validNodeLeft[left] && validNodeRight[left] && validNodeLeft[right] && validNodeRight[right]) { // all good
		   	validNodeLeft[node] = true;
		   	validNodeRight[node] = true;
		   }
		   else if (validNodeLeft[right] && validNodeRight[right]) {   // good values for both from right, no value from left
 			validNodeRight[node] = true;
			validNodeLeft[node] = true;  // was false
		   	}
		   else if (validNodeLeft[left] && validNodeRight[left]) {    // good values for both from left, no value from right
			validNodeLeft[node] = true;  // was true
		        validNodeRight[node] = true;
		        }
		   else { // both bad - remove the node
			validNodeLeft[node] = false;
			validNodeRight[node] = false;
		   }
		}
		else {
			boolean nodeOk = (MesquiteDouble.isCombinable(data1.getState(tree.taxonNumberOfNode(node),0)) &&
			                            MesquiteDouble.isCombinable(data2.getState(tree.taxonNumberOfNode(node),0)));
			if (nodeOk) {
				validNodeLeft[node] = true;
				validNodeRight[node] = true;
			}
			else {
				validNodeLeft[node] = false;
				validNodeRight[node] = false;
				good.setValue(false);
			}
		}
	}
	
	/** Actually does the pruning, based on the validNodeLeft & Right */
	private void pruneTree(MesquiteTree tree, boolean validLeft[], boolean validRight[], int node) {
	    if (tree.nodeIsInternal(node)) {
                if (validLeft[node] && validRight[node]) {
	            for (int daughter = tree.firstDaughterOfNode(node); tree.nodeExists(daughter); daughter = tree.nextSisterOfNode(daughter)) {
		       pruneTree(tree,validLeft,validRight,daughter);
		    }
		}
                else if (!validLeft[node] && validRight[node]) {
	             int left = tree.firstDaughterOfNode(node);
		     tree.deleteClade(left,false);
		     pruneTree(tree,validLeft,validRight,tree.lastDaughterOfNode(node));
		}
		else if (!validRight[node] && validLeft[node]) {
		      int right = tree.lastDaughterOfNode(node);
		      tree.deleteClade(right,false);
		      pruneTree(tree,validLeft,validRight,tree.firstDaughterOfNode(node));
		   }
	   }
	   else {   // tip - anything to do??
	   }
	}
	
	/** Prune the tree by removing nodes that aren't supported by both data sets */
	private Tree dataPruneTree(Tree tree, CategoricalDistribution observedStates1, ContinuousDistribution data2) {
		// Don't copy the tree if all data is present
		//MesquiteBoolean test = new MesquiteBoolean(true);
		//int root = tree.getRoot();
		//TreeVector trees;
		validNodeLeft = new boolean[tree.getNumNodeSpaces()];
		validNodeRight = new boolean[tree.getNumNodeSpaces()];		
		
		//pruneTest(tree,data1,data2,root,test);
		//if (test.getValue())			
		return tree;
	//	else {
	//		MesquiteTree newTree = (MesquiteTree)tree.cloneTree();
	//		numNodes = tree.getNumNodeSpaces();
	//		pruneTree(newTree,validNodeLeft,validNodeRight,root);
	//		MesquiteProject bar = getProject();
	//		TreesManager manager = (TreesManager)bar.getCoordinatorModule().findElementManager(TreeVector.class);
	//		if (manager != null) {
	//		    int numLists = manager.getNumberTreeBlocks(newTree.getTaxa());
	//		    if (numLists == 1) {
	//		        trees = manager.getTreeBlock(newTree.getTaxa(),0);
	//		        trees.addElement(newTree,false);
	//		    }
	//		 }
			//TreeVector foo = bar.storeTree((Frame)null, (Tree)null,(CommandRecord)null);
	//		return newTree;
	//	}
	}

	/** positivize by flipping the sign of corresponding elements of data1 and data2 if the data1 element is negative */
	private void positivize(Tree tree, double[] data1,double [] data2,int node) {
		for (int daughter = tree.firstDaughterOfNode(node); tree.nodeExists(daughter); daughter = tree.nextSisterOfNode(daughter))
			positivize(tree, data1, data2, daughter);
			if (MesquiteDouble.isCombinable(data1[node]) && MesquiteDouble.isCombinable(data2[node])) {
				if (data1[node] < 0) {	
					data1[node] *= -1;
					data2[node] *= -1;
				}
			}
	}

	/** generate sums, sums of squares and cross product sums for statistical calculations */
	private void getSums(Tree tree, 
					 double[] data1, 
					 double[] data2,
				 	 MesquiteDouble sx,
					 MesquiteDouble sx2,
					 MesquiteDouble sy,
					 MesquiteDouble sxy,
					 int node)  {
		if (tree.nodeIsInternal(node)){   // only interior nodes
			if (MesquiteDouble.isCombinable(data1[node]) && // calculate only for nodes with good data
			     MesquiteDouble.isCombinable(data2[node])) {
				sx.add(data1[node]);
				sx2.add(data1[node]*data1[node]);
				sy.add(data2[node]);
				sxy.add(data1[node]*data2[node]); 
			}
			getSums(tree,data1,data2,sx,sx2,sy,sxy,tree.firstDaughterOfNode(node));
			getSums(tree,data1,data2,sx,sx2,sy,sxy,tree.lastDaughterOfNode(node));
		}
	}
	
	/** Currently all this does is calculate the slope of the regression line, all the partial sums
	 are tossed.  */
	private void calculateSlope(Tree tree, double [] data1, double [] data2, MesquiteDouble conSlope) {
		MesquiteDouble sx = new MesquiteDouble(0.0);
		MesquiteDouble sx2 = new MesquiteDouble(0.0);
		MesquiteDouble sy = new MesquiteDouble(0.0);
		MesquiteDouble sxy = new MesquiteDouble(0.0);
		int root = tree.getRoot();
		
		positivize(tree,data1,data2,root);
		getSums(tree,data1,data2,sx,sx2,sy,sxy,root);
		// stash away the contrast slope for further use
		if (sx2.getValue() != 0)  
			conSlope.setValue(sxy.getValue()/sx2.getValue());  // otherwise leave "unassigned"
	}

	/** This calculates residuals from the through the origin regression.  Note that these
	residuals will not generally sum to zero.  The method calculates them in the obvious manner */
	private void calculateResiduals(Tree tree, 
							double [] contrasts1, 
							double [] contrasts2, 
							NumberArray result,
							MesquiteDouble slope, 
							int node)  {
		if (tree.nodeIsInternal(node)) {  // only interior nodes
			if (MesquiteDouble.isCombinable(contrasts2[node]) &&
			    MesquiteDouble.isCombinable(contrasts1[node])) {
				if (absolute == ContrastForChar.ABSCONTRAST)	
					result.setValue(node,Math.abs(contrasts2[node] - slope.getValue()*contrasts1[node]));
				else
					result.setValue(node,contrasts2[node] - slope.getValue()*contrasts1[node]);
			}
			else
				result.setValue(node,MesquiteDouble.unassigned);
			for (int daughter = tree.firstDaughterOfNode(node); tree.nodeExists(daughter); daughter = tree.nextSisterOfNode(daughter))
				calculateResiduals(tree, contrasts1, contrasts2, result, slope, daughter);
		}
	}

	/*.................................................................................................................*/
    /** 
     * 
     * @param tree
     * @param charDistribution1
     * @param charDistribution2
     * @param result
     * @param resultString
     * @param commandRec If a CommandRecord is passed, this accepts it
     * This method checks arguments, calls contrast calculators to get the residuals, then
     * calls methods to get the regression slope and the residuals, and returns them in result
     */
	 public void calculateNumbers(Tree tree, 
		     CharacterDistribution charDistribution1, 
		     CharacterDistribution charDistribution2, 
		     NumberArray result, 
		     MesquiteString resultString){

UnivariateContrastCalculator continuousCalculator;
AC3ContrastCalculator categoricalCalculator;
NumberArray	contrastWithNodes=null;
CategoricalDistribution observedStates1 = null;
ContinuousDistribution observedStates2 = null;
MesquiteDouble conSlope = new MesquiteDouble();
int root = tree.getRoot();
double [] contrasts1;
double [] contrasts2;
//BivariateContrastCalculator calculator; 
Tree dataReducedTree;

if (result==null || tree == null)
return;
if (!(charDistribution1 instanceof CategoricalDistribution) ||!(charDistribution2 instanceof ContinuousDistribution))
return;
result.zeroArray();
observedStates1 = (CategoricalDistribution)charDistribution1;
observedStates2 = (ContinuousDistribution)charDistribution2;

lastUsedTree = tree;
lastTreeVersion = tree.getVersionNumber();
dataReducedTree = dataPruneTree(tree,observedStates1,observedStates2);                 // tree pruned to handle missing data
if ((observedStates1 != null) && (observedStates2 != null)) {
//calculator = new BivariateContrastCalculator(dataReducedTree, observedStates1, observedStates2);
//contrasts1 = calculator.contrastCalculation();
categoricalCalculator= new AC3ContrastCalculator(tree,observedStates1);
continuousCalculator = new UnivariateContrastCalculator(tree,observedStates2);
contrasts1 = categoricalCalculator.contrastCalculation();

if (!continuousCalculator.Ok())
MesquiteMessage.warnProgrammer("Bad contrast calculation in ConstrastResiduals X trait");
contrasts2 = continuousCalculator.contrastCalculation();
double [] stdDev=continuousCalculator.getStdDev();
if(categoricalCalculator.getNodesToRecalculate()!=null){
contrastWithNodes=categoricalCalculator.getNodesToRecalculate().cloneArray();
double [] resul=continuousCalculator.correctedStdDevCalculation(contrastWithNodes);
for (int i=0;i<resul.length;i+=2)
stdDev[(int)resul[i]]=resul[i+1];
for(int k=0;k<contrasts1.length;k++){
if(MesquiteDouble.isCombinable(contrasts1[k]) && stdDev[k]!=0)
contrasts1[k]=contrasts1[k]/stdDev[k];
}
double[] res=continuousCalculator.contrastToRecalculate(contrastWithNodes);
for (int j=0;j<res.length;j+=2)
contrasts2[(int)res[j]]=(double)res[j+1];
}
calculateSlope(dataReducedTree,contrasts1,contrasts2,conSlope);
calculateResiduals(dataReducedTree,contrasts1,contrasts2,result,conSlope,root);
}
/*if (!calculator.Ok())
MesquiteMessage.warnProgrammer("Bad contrast calculation in ConstrastResiduals X trait");
contrasts2 = calculator.getContrasts2();
calculateSlope(dataReducedTree,contrasts1,contrasts2,conSlope);
calculateResiduals(dataReducedTree,contrasts1,contrasts2,result,conSlope,root);
}*/
if (resultString!=null)
resultString.setValue("Contrast residuals");
}


	/*.................................................................................................................*/
  	 public Class getCharacterClass() {
  	 	return ContinuousState.class;
  	 }
	/*.................................................................................................................*/
  	 public CompatibilityTest getCompatibilityTest() {
  	 	return new ContinuousStateTest();
  	 }
	/*.................................................................................................................*/
    	 public String getExplanation() {
		return "Calculates the regression residuals of Felsenstein's contrasts between two characters";
   	 }
   
	/*.................................................................................................................*/
    	 public String getName() {
		return "Contrast Residuals for AC3";
   	 }
	/*.................................................................................................................*/
  	 public String getVersion() {
		return "1.08";
   	 }
	/*.................................................................................................................*/
  	 public boolean isPrerelease() {
		return false;
   	 }
   	 
	/*.................................................................................................................*/
  	 public String getAuthors() {
		return "Franois Gao, Jonathan Corcy, Michel Laurin (based on PDAP)";
   	 }
}
