/* AC3 Mesquite module */

package mesquite.ac3.GrafenTransform;
/*~~  */

import mesquite.lib.*;
import mesquite.lib.duties.*;

/** ======================================================================== */
public class GrafenTransform extends BranchLengthsAlterer {
    int [] depth;
    double resultNum;
    /*.................................................................................................................*/
    public boolean startJob(String arguments, Object condition, boolean hiredByName) {		
        return true;
    }

    /*.................................................................................................................*/
    /**
     * @param tree
     * @param resultString passed in, never filled
     * @param commandRec
     * @return returns true because this assumes the calling code is always responsible for notifying listeners of tree
     *
     * Real entry point for the transform - a wrapper around the recursive method doTransform
     *
     */
    public  boolean transformTree(AdjustableTree tree, MesquiteString resultString){
        doTransform(tree, tree.getRoot());
        return true;
    }

    /**
     * 
     * @param tree
     * @param node
     * 
     * This procedure sets all the branch lengths to those dictated by
     * arbitrary method.  The distance from the tips to the current node is
     * equal to one less than the number of tips descending from that node.
     */
    private void doTransform(AdjustableTree tree, int root) {
        MesquiteInteger rootCount = new MesquiteInteger();

        depth = new int[tree.getNumNodeSpaces()];
        grafenMe1(tree,root,rootCount);
        grafenMe2(tree,root);
    }

    /**
     * 
     * @param tree
     * @param nd
     * @param mothersCount
     * 
     * This procedure computes the depths recursively.
     * There are two support functions since DOS AC3 needed to handle polytomies specially.
     */
    private void grafenMe1(AdjustableTree tree, int nd, MesquiteInteger mothersCount) { 
        if (!tree.nodeIsInternal(nd)) {
            mothersCount.add(1);            // this is a tip, so there is 1 in the subtree
            depth[nd] =0;                        //set depth to 0}
        }
        else {
            MesquiteInteger myCounts = new MesquiteInteger(0);
            for (int daughter=tree.firstDaughterOfNode(nd); tree.nodeExists(daughter); daughter = tree.nextSisterOfNode(daughter) ) 
                grafenMe1(tree,daughter,myCounts);
            mothersCount.add(myCounts.getValue());
            depth[nd] = myCounts.getValue()-1; //save tips-1 as depth
        }
    }

    /**
     * 
     * @param tree AdjustableTree to transform
     * @param nd int specifying where we are in the tree
     * 
     * This method finishes the transformation by assigning branch lengths.  Since this is a regular
     * Mesquite tree, ZLB's shouldn't get special treatment.
     */
    private void grafenMe2(AdjustableTree tree, int nd) {
        if (tree.nodeIsInternal(nd)) {
            for (int daughter=tree.firstDaughterOfNode(nd); tree.nodeExists(daughter); daughter = tree.nextSisterOfNode(daughter) ) { 
                tree.setBranchLength(daughter,depth[nd]-depth[daughter],false);
                grafenMe2(tree,daughter);
            }
        }
    }

    /*.................................................................................................................*/
    public String getName() {
        return "Branch Lengths Method of Grafen (1989) for AC3";
    }
    /*.................................................................................................................*/
    public String getVersion() {
        return "1.08";
    }
    /*.................................................................................................................*/
    public boolean isPrerelease() {
        return false;
    }

    /*.................................................................................................................*/
    public String getAuthors() {
        return "Franois Gao , Jonathan Corcy, Michel Laurin (taken from PDAP)";
    }

    /*.................................................................................................................*/
    /** returns an explanation of what the module does.*/
    public String getExplanation() {
        return "Adjusts a tree's branch lengths according the method of Grafen (1989)" ;
    }

}