/* AC3 Mesquite module */

package mesquite.ac3.IndepContrastsVariance;
/*~~  */

import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.ac3.lib.*;
import mesquite.categ.lib.*;
import mesquite.cont.lib.*;
import mesquite.lib.duties.*;

/** ======================================================================== */
//Peter: this is a new module (see also NumForTreeWith2Chars in Basic).

public class IndepContrastsVariance extends NumberForCharAndTree {
	ContrastForChar contrastsTask;
	AC32CTStatPakSource statPakTask;
	private NumberArray xArray;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, boolean hiredByName) {

 		contrastsTask = (ContrastForChar)hireNamedEmployee(ContrastForChar.class, "#FelsContrWithChar");
 		if (contrastsTask == null)
 			return sorry(getName() + " couldn't start because no calculator of Felsenstein's contrasts obtained.");
		contrastsTask.setOption(ContrastForChar.CONTRAST);
 		statPakTask = (AC32CTStatPakSource)hireNamedEmployee(AC32CTStatPakSource.class, "#SPOriDiagnostics");
 		if (statPakTask == null)
 			return sorry(getName() + " couldn't start because no calculator of correlation obtained.");
		xArray = new NumberArray(3);
 		return true; //false if no appropriate employees!
 	}
 	public void employeeQuit(MesquiteModule m){
 		if (m==contrastsTask)
 			iQuit();
 	}
 	
   	/** Called to provoke any necessary initialization.  This helps prevent the module's intialization queries to the user from
   	happening at inopportune times (e.g., while a long chart calculation is in mid-progress)*/
   	public void initialize(Tree tree, CharacterDistribution charStatesX){
   	}
	/*.................................................................................................................*/
	/*in future this will farm out the calculation to the modules that deal with appropriate character model*/
	public  void calculateNumber(Tree tree, CharacterDistribution charStatesX, MesquiteNumber result, MesquiteString resultString) {  
    	 	if (result==null)
    	 		return;
		result.setToUnassigned();
		if (tree==null || charStatesX ==null) {
			if (resultString!=null)
				resultString.setValue("Variance in Independent Contrasts can't be calculated because tree or character not supplied");
		}
		else if (!(charStatesX instanceof ContinuousDistribution)) {
			if (resultString!=null)
				resultString.setValue("Variance in Independent Contrasts can't be calculated because character supplied isn't continuous");
		}
		else {
			int numNodes = tree.getNumNodeSpaces();
			xArray.resetSize(numNodes);
			contrastsTask.calculateNumbers(tree, charStatesX, xArray, null);
			AC32CTStatPak myStats = statPakTask.getStatPak(new MesquiteInteger(0));  // df reduction == 0 since we don't care for variances
            myStats.setTree(tree);
            myStats.setObserved1((CategoricalDistribution)charStatesX);
            myStats.setObserved2((ContinuousDistribution)charStatesX);
			myStats.doCalculations(xArray, xArray, null);
			result.setValue(myStats.getVariance(0));
			resultString.setValue("Variance in independent contrasts: " + result);
		}
	}
	/*.................................................................................................................*/
	
	/*.................................................................................................................*/
    	 public String getName() {
		return "Variance in Independent Contrasts for AC3";
   	 }
	/*.................................................................................................................*/
	/** Returns the name of the module in very short form.  For use for column headings and other constrained places.  Unless overridden returns getName()*/
 	public String getVeryShortName(){
 		return "Indep. Contrasts Variance";
 	}
	/*.................................................................................................................*/
 	/** Returns a version number string for the module
     * @return String
 	 */
  	 public String getVersion() {
		return "1.08(1)";
   	 }
	/*.................................................................................................................*/
	/** Marks the module as NOT being a prerelease
     * @return boolean - constant false 
     */
	public boolean isPrerelease() {
		return false;
	}  
    /*.................................................................................................................*/
    /** Returns a string identifying the authors of the module
     * @return String 
     */
  	 public String getAuthors() {
		return "Franois Gao , Jonathan Corcy, Michel Laurin (based on PDAP)";
   	 }
	 	 
	/*.................................................................................................................*/
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Calculates the variance in standardized Felsenstein's independent contrasts of a character on a tree." ;
   	 }
}

