/* AC3 Mesquite module */

package mesquite.ac3.RaiseStatesToPowerChar;
/*~~  */

import java.applet.*;
import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.characters.CharacterData;
import mesquite.lib.duties.*;
import mesquite.cont.lib.*;
import mesquite.lib.table.*;

/** ======================================================================== */
public class RaiseStatesToPowerChar extends ContDataAlterer {
	static final double maxExp = Math.log(Double.MAX_VALUE);
	double power = 1.0;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, boolean hiredByName) {
		return true;
	}
		
   	/** Called to alter data in those cells selected in table*/
   	public boolean alterData(CharacterData data, MesquiteTable table){
   			boolean did=false;
   			boolean valid=true;
			if (!(data instanceof ContinuousData))
				return false;
			double d = MesquiteDouble.queryDouble(containerOfModule(), "Raise to Power", "Raise values in matrix to power:", power);
			if (MesquiteDouble.isCombinable(d))
				power = d;
			else
				return false;
			ContinuousData cData = (ContinuousData)data;
   	 		if (table==null && data !=null){
				// first check
				for (int i=0; i<cData.getNumChars(); i++){
					for (int j=0; j<cData.getNumTaxa(); j++) {
						valid = valid && checkCell(cData, i, j);
					}
				}
				// then transform only if all cells are valid
				if (valid)
					for (int i=0; i<cData.getNumChars(); i++){
						for (int j=0; j<cData.getNumTaxa(); j++) {
								transformCell(cData, i, j);
								did=true;
						}
					}
   	 		}
   	 		else if (table!=null && cData !=null){
	   			if (table.anyCellSelected()) {
	   				// first check
					for (int i=0; i<table.getNumColumns(); i++)
						for (int j=0; j<table.getNumRows(); j++)
							if (table.isCellSelected(i,j)) {
								valid = valid && checkCell(cData, i, j);
							}
					// then transform only if all selected cells are valid
					if (valid)
						for (int i=0; i<table.getNumColumns(); i++)
							for (int j=0; j<table.getNumRows(); j++)
								if (table.isCellSelected(i,j)) {
									transformCell(cData, i, j);
									did = true;
								}
				}
				else if (table.anyRowSelected()) {
					// first check
					for (int j=0; j<table.getNumRows(); j++) {
						if (table.isRowSelected(j))
							for (int i=0; i<table.getNumColumns(); i++) {
								valid = valid && checkCell(cData, i, j);
							}
					}
					// then transform only if all selected cells are valid
					if (valid)
						for (int j=0; j<table.getNumRows(); j++) {
							if (table.isRowSelected(j))
								for (int i=0; i<table.getNumColumns(); i++) {
									transformCell(cData, i, j);
									did = true;
								}
						}
				}
				else if (table.anyColumnSelected()) {
					// first check
					for (int i=0; i<table.getNumColumns(); i++){
						if (table.isColumnSelected(i))
							for (int j=0; j<table.getNumRows(); j++) {
								valid = valid && checkCell(cData, i, j);
							}
					}
					// then transform only if all selected cells are valid
					if (valid)
						for (int i=0; i<table.getNumColumns(); i++){
							if (table.isColumnSelected(i))
								for (int j=0; j<table.getNumRows(); j++) {
									transformCell(cData, i, j);
									did=true;
								}
						}
				}
				else {
					// first check
					for (int i=0; i<cData.getNumChars(); i++){
							for (int j=0; j<cData.getNumTaxa(); j++) {
								valid = valid && checkCell(cData, i, j);
								did=true;
							}
					}
					// then transform only if all selected cells are valid
					if (valid)
						for (int i=0; i<cData.getNumChars(); i++){
							for (int j=0; j<cData.getNumTaxa(); j++) {
								transformCell(cData, i, j);
								did=true;
							}
						}
				}
			}
			if (!valid){
				alert("Values were not exponentially transformed because some values were either nonpositive or too large.");
			}	
			return did;
   	}
   	

	private boolean checkCell(ContinuousData data, int ic, int it){
		int item = 0; //TODO: other items
		
		double state = data.getState(ic,it, item);
		if (MesquiteDouble.isCombinable(state)) 
			if ((state<0) || (power*Math.log(state) > maxExp))
				return false;
			else
				return true;
		else 
			return true;   //if not combinable, transformCell ignores it, so should checkCell
	}
   	
	private void transformCell(ContinuousData data, int ic, int it){
		int item = 0; //TODO: other items
		
		double state = data.getState(ic,it, item);
		if (MesquiteDouble.isCombinable(state)) {
			data.setState(ic,it, item,  Math.exp(power*Math.log(state))); 
		}
	}

	
	/*.................................................................................................................*/
    	 public String getName() {
		return "Raise to Power for AC3";
   	 }
	/*.................................................................................................................*/
  	 public String getVersion() {
		return "1.08";
   	 }
	/*.................................................................................................................*/
  	 public boolean isPrerelease() {
		return false;
   	 }
   	 
	/*.................................................................................................................*/
  	 public String getAuthors() {
		return "Franois Gao , Jonathan Corcy, Michel Laurin (taken from PDAP)";
   	 }
	/*.................................................................................................................*/
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Alters continuous data by raising values to a power." ;
   	 }

 	//modif mesquite2
	public boolean alterData(CharacterData data, MesquiteTable table, UndoReference undoReference) {
		// TODO Auto-generated method stub
		return false;
	}
   	 
}
