package mesquite.ac3.lib;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import mesquite.lib.*;
import mesquite.lib.characters.CharacterHistory;
import mesquite.lib.characters.CharacterState;

import mesquite.categ.lib.CategoricalDistribution;

public class AC3ContrastCalculator extends ContrastCalculator {
	protected Tree tree;

	protected CategoricalDistribution states;

	protected int numNodes;

	protected double[] contrasts = null;

	protected Set availableContrasts;

	protected Set notAvailableNodesToContrast;

	protected List nodesToContrast;

	protected NumberArray nodesToRecalc;

	protected double[] x = null;
	protected double[] tips = null;

	protected int maxState;

	protected boolean statsOk = true; // maybe private

	/*
	 * public boolean startJob(String arguments, Object condition, CommandRecord
	 * commandRec, boolean hiredByName) { return true; }
	 */
	public AC3ContrastCalculator(Tree theTree,
			CategoricalDistribution observedStates) {
		if (theTree == null || observedStates == null) {
			MesquiteMessage
			.printStackTrace("A null value was passed to the constructor for AcdcpContrastCalculator");
			tree = null;
			observedStates = null;

		} else {
			tree = theTree;
			states = observedStates;
			numNodes = tree.getNumNodeSpaces();
			maxState = states.getMaxState();
			checkArrays();
			getCategoricalStatesValues();
		}
	}

	protected void checkArrays() {
		if (contrasts == null || contrasts.length != numNodes) {
			contrasts = new double[numNodes];
			x = new double[numNodes];
			tips = new double[numNodes];
			for (int i = 0; i < numNodes; i++)
				contrasts[i] = MesquiteDouble.unassigned;
			availableContrasts = new HashSet();
			notAvailableNodesToContrast = new HashSet();
			nodesToContrast = new ArrayList();
			nodesToRecalc = new NumberArray(numNodes);
		} else {
			DoubleArray.deassignArray(contrasts);
			DoubleArray.zeroArray(x);
			DoubleArray.deassignArray(tips);
		}
	}

	public boolean Ok() {
		return statsOk;
	}

	/* ................................................................................................................. */

	/**
	 * This returns the array of contrast values for those modules that need
	 * them
	 * 
	 * @return double[] - the array of contrast values calculated here
	 */
	public void getCategoricalStatesValues(){
		/**
		 * ----getting categorical states for 10 different characters (should be
		 * faster)--*
		 */
		double[] table2power = { 1, 2, 4, 8, 16, 32, 64, 128, 256, 512 };
		for (int i = 0; i < numNodes; i++) {
			double state = states.getState(i);
			
			if (!MesquiteLong.isInfinite((long) state) && state <= 512)
				for (int j = 0; j < 10; j++) {
					if (state == table2power[j]) {
						x[i] = j;
						break;
					}
					if (state < table2power[j]) {
						x[i] = MesquiteDouble.unassigned;
						break;
					}
				}
			else
				x[i] = MesquiteDouble.unassigned;
		}
		/** --------getting categorical states for N different character----* */
		/*
		 * double maxValue=Math.pow(2, maxState); for(int i=0;i<numNodes;i++){
		 * double state=states.getState(i);
		 * if(!MesquiteLong.isInfinite((long)state) && (state%2==0 || state==1) &&
		 * state<=maxValue){ double power=0; while(state>1){ state/=2; power++; }
		 * if (state==1) x[i]=power; else x[i]=MesquiteDouble.unassigned; } else
		 * x[i]=MesquiteDouble.unassigned; }
		 */
	}
	public double[] contrastCalculation() {
		int root = tree.getRoot();
		xCalculation(root);
		xExtendedCalculation(availableContrasts.size());
		nodesToRecalculateContinuousContrasts();
		return contrasts;
	}

	protected void nodesToRecalculateContinuousContrasts() {
		if (nodesToContrast.size() != 0) {
			Iterator i = nodesToContrast.iterator();
			for (int j = 0; j < nodesToContrast.size(); j++) {
				nodesToRecalc
				.setValue(j, Integer.parseInt(i.next().toString()));
			}
		} else
			nodesToRecalc = null;
	}

	public NumberArray getNodesToRecalculate() {
		return nodesToRecalc;
	}

	protected boolean isAvailableNodeToContrast(int daughterNode,
			int ancestorNode) {
		int closestAncestor = daughterNode;
		boolean isAvailableNode = true;
		closestAncestor = tree.motherOfNode(closestAncestor);
		if(!MesquiteDouble.isCombinable(contrasts[closestAncestor])){
		while (closestAncestor != ancestorNode) {
			if (!MesquiteDouble.isCombinable(contrasts[closestAncestor]))
				closestAncestor = tree.motherOfNode(closestAncestor);
			else {
				isAvailableNode = false;
				break;
			}
		}
		}
		else
			isAvailableNode = false;
		return isAvailableNode;
	}


	protected void xExtendedCalculation(int numberOfAvailableContrasts) {
		if (numberOfAvailableContrasts >= 2) {
			Iterator i = availableContrasts.iterator();
			int highestNode = -1;
			while (i.hasNext()) {
				int availableContrast = Integer.parseInt(i.next().toString());
				Iterator j = availableContrasts.iterator();
				while (j.hasNext()) {
					int mostRecentCommonAncestor = tree.mrca(availableContrast,
							Integer.parseInt(j.next().toString()));
					if (!notAvailableNodesToContrast.contains(new Integer(
							mostRecentCommonAncestor))
							&& tree.nodeIsInternal(mostRecentCommonAncestor)) {
						if (!MesquiteDouble
								.isCombinable(contrasts[mostRecentCommonAncestor])) {
							if (mostRecentCommonAncestor > highestNode)
								highestNode = mostRecentCommonAncestor;
						}
					}
				}
			}
			if (highestNode != -1) {
				Iterator i2 = availableContrasts.iterator();
				int[] contrastedNodes = { -1, -1 };
				int[] depth = { MesquiteInteger.infinite,
						MesquiteInteger.infinite };
				int dTA;
				while (i2.hasNext()) {
					int contrastedNode = Integer.parseInt(i2.next().toString());
					if(tree.descendantOf(contrastedNode, highestNode)){
						if (isAvailableNodeToContrast(contrastedNode, highestNode)) {
							dTA = tree.depthToAncestor(contrastedNode, highestNode);
							if (0 < dTA && dTA < depth[0]) {
								depth[0] = dTA;
								contrastedNodes[0] = contrastedNode;
								if (depth[0] < depth[1]) {
									depth[0] = depth[1];
									depth[1] = dTA;
									contrastedNodes[0] = contrastedNodes[1];
									contrastedNodes[1] = contrastedNode;
								}
							}
						}
						else
							i2.remove();

					}
				}
				if (tree.nodeExists(contrastedNodes[0])
						&& tree.nodeExists(contrastedNodes[1])) {
					 if(contrastedNodes[0]<contrastedNodes[1]){
					 contrasts[highestNode]=x[contrastedNodes[0]]-x[contrastedNodes[1]];
					 nodesToContrast.add(new Integer(highestNode));
						nodesToContrast.add(new Integer(contrastedNodes[0]));
						nodesToContrast.add(new Integer(contrastedNodes[1]));
					 }
					 else{
					 contrasts[highestNode]=x[contrastedNodes[1]]-x[contrastedNodes[0]];
					 nodesToContrast.add(new Integer(highestNode));
						nodesToContrast.add(new Integer(contrastedNodes[1]));
						nodesToContrast.add(new Integer(contrastedNodes[0]));
					 }
					 availableContrasts.remove(new Integer(contrastedNodes[0]));
					availableContrasts.remove(new Integer(contrastedNodes[1]));
					xExtendedCalculation(availableContrasts.size());
				} else {
					notAvailableNodesToContrast.add(new Integer(highestNode));
					xExtendedCalculation(availableContrasts.size());
				}
			} else
				return;
		} else
			return;
	}

	protected void xCalculation(int node) {
		for (int daughter = tree.firstDaughterOfNode(node); tree
		.nodeExists(daughter); daughter = tree
		.nextSisterOfNode(daughter))
			xCalculation(daughter);
		if (tree.nodeIsInternal(node)) {
			int left = tree.firstDaughterOfNode(node);
			int right = tree.lastDaughterOfNode(node);
			if (MesquiteDouble.isCombinable(x[left])
					&& MesquiteDouble.isCombinable(x[right])) { // good contrast
				contrasts[node] = x[left] - x[right];
			}
			//if only left state of node is terminal and available
			else if (MesquiteDouble.isCombinable(x[left])
					) {
				availableContrasts.add(new Integer(left));
				contrasts[node] = MesquiteDouble.unassigned;
			}
			// if only right state of node is terminal and available
			else if (MesquiteDouble.isCombinable(x[right])
					) {
				availableContrasts.add(new Integer(right));
				contrasts[node] = MesquiteDouble.unassigned;
			}
			/** method to use if you want to contrast between terminal taxa only
			// if only left state of node is terminal and available
			else if (MesquiteDouble.isCombinable(x[left])
					&& tree.nodeIsTerminal(left)) {
				availableContrasts.add(new Integer(left));
				contrasts[node] = MesquiteDouble.unassigned;
			}
			// if only right state of node is terminal and available
			else if (MesquiteDouble.isCombinable(x[right])
					&& tree.nodeIsTerminal(right)) {
				availableContrasts.add(new Integer(right));
				contrasts[node] = MesquiteDouble.unassigned;
			}*/
			// if no state of node is available
			else {
				contrasts[node] = MesquiteDouble.unassigned;
			}
		} else {
			contrasts[node] = MesquiteDouble.unassigned;
		}
	}

	/** Just returns the node values (tip and interior), without recalculating */
	public double[] getNodeValues() {
		return x;
	}
	
	public String[] getAllStates(){
		String[] statesOfNodes=new String[numNodes];
		for (int i=0;i<statesOfNodes.length;i++){
			if(tree.nodeIsInternal(i)){
				CharacterState statesOfNode=states.getCharacterState(states.getCharacterState(), i);
				statesOfNodes[i]=statesOfNode.toString();
			}
		}
		return statesOfNodes;
	}
	
	public double[] fillTips(int node) {
	for (int daughter = tree.firstDaughterOfNode(node); tree.nodeExists(daughter); daughter = tree.nextSisterOfNode(daughter))
			fillTips(daughter);
	if (tree.nodeIsInternal(node)) {
			tips[node] = MesquiteDouble.unassigned;

		}
		else {
		tips[node] = x[node];
		}
		return tips;	
	}

	protected void vCalculation(int node) {
		// no need
		return;
	}

	protected double[] xExtendedCalculation(NumberArray nodesToRecalc) {
	return null;
	}

	protected double[] DevExtendedCalculation(NumberArray nodesToRecalc) {
		// TODO Auto-generated method stub
		return null;
	}
		}
