/* AC3 Mesquite module  */

package mesquite.ac3.lib;

import java.awt.*;
/* ======================================================================== */

import mesquite.lib.ColorDistribution;
import mesquite.lib.CommandRecord;
import mesquite.lib.MainThread;
import mesquite.lib.MesquiteFile;
import mesquite.lib.MesquiteModule;
import mesquite.lib.StringUtil;
import mesquite.lib.Tree;
import mesquite.lib.characters.CharacterDistribution;
import mesquite.lib.duties.CharSourceCoordObed;

public abstract class AC3FileReporter extends MesquiteModule {
	// copied from FileInterpreter - maybe make a common superclass?
	public static final int CURRENTDELIMITER=0;
	public static final int MACOSDELIMITER=1;
	public static final int WINDOWSDELIMITER=2;
	public static final int UNIXDELIMITER=3;
	protected int lineDelimiter = CURRENTDELIMITER;
	public boolean writeOnlySelectedData = false;
	public boolean writeOnlySelectedTaxa = false;
	// and two extensions...
	public static final int TAB = 0;
	public static final int COMMA = 1;
	public static final int SPACE = 2;
	public static final int NEWLINE = 3;
	protected int tableDelimiter = TAB;   // we can change this default....
    public boolean convertSpaces = true;  //convert spaces to '_' in tip names
    //  it may be completely unnecessary to save the state of convertSpaces, but better safe...
    private boolean savedConvertSpaces = true;  


   	public Class getDutyClass() {
   	 	return AC3FileReporter.class;
   	}

 	public String getDutyName() {
 		return "Writes a AC3 report to a file";
   	}

/*.................................................................................................................*/
	/** also borrowed from FileInterpreter */
	public String getLineEnding() {
 		if (lineDelimiter == CURRENTDELIMITER) 
 			return StringUtil.lineEnding();
 		else if (lineDelimiter == MACOSDELIMITER) 
 			return "\r";
 		else if (lineDelimiter == WINDOWSDELIMITER) 
 			return "\r\n";
 		else if (lineDelimiter == UNIXDELIMITER) 
 			return "\n";
 		return StringUtil.lineEnding();
   	}
   	 
   	public String getTableDelimiter() {
   	 	switch (tableDelimiter) {
   	 		case TAB: return("\t");
   	 		case COMMA: return(",");
   	 		case SPACE: return(" ");
   	 		case NEWLINE: return getLineEnding();
   	 	default: return "\t";
   	 	}
   	}
    

   	 
/*.................................................................................................................*/
	protected String stripNex(String name) {
		String extension = ".nex";
 		if (name == null)
 			return null;
 		int length = name.length();
 		if (length<extension.length())
 			return name;
 		String extSuffix = name.substring(length-extension.length(), length);
 		if (extSuffix.equalsIgnoreCase(extension)) 
 			return name.substring(0, length-extension.length());
 		else
 			return name;
   	 }
   	 
/*........................................................................................*/
	public abstract boolean hasTables();

/*.................................................................................................................*/
   	public void saveReportFile(String output, String arguments, String suggestedFileName) {
    //check arguments for filename and directory
		String tempFileName=null;
		String tempDirectoryName=null;
   	 	String token = parser.getFirstToken(arguments);
   	 	while (token !=null){
   	 		if (token.equalsIgnoreCase("file")){
   	 			parser.getNextToken(); // =
   	 			tempFileName = parser.getNextToken();
   	 		}
   	 		else if (token.equalsIgnoreCase("directory")){
   	 			parser.getNextToken(); // =
   	 			tempDirectoryName = parser.getNextToken();
   	 			if (tempDirectoryName != null && !tempDirectoryName.endsWith(MesquiteFile.fileSeparator)) //(tempDirectoryName.length()-1) != MesquiteFile.fileSeparator.charAt(0))
   	 				tempDirectoryName += MesquiteFile.fileSeparator;
   	 		}
   	 		token = parser.getNextToken();
   	 	}
   	 	if (StringUtil.blank(tempFileName)){
	   	 	String queryMessage = "Export to file";
			MainThread.setShowWaitWindow(false);
			FileDialog fdlg= new FileDialog(containerOfModule().getParentFrame(), queryMessage, FileDialog.SAVE);   // Save File dialog box
			if (suggestedFileName == null)
				suggestedFileName = "untitled";
			fdlg.setFile(suggestedFileName);
			
			fdlg.setBackground(ColorDistribution.light[0]);
			fdlg.setVisible(true);
			MainThread.setShowWaitWindow(true);
			tempFileName=fdlg.getFile();
			tempDirectoryName=fdlg.getDirectory();
		}
		if (!StringUtil.blank(tempFileName)) {
			if (StringUtil.blank(tempDirectoryName))
				tempDirectoryName = "";
			logln("Writing report file to " + tempDirectoryName+tempFileName);
			MesquiteFile.putFileContents(tempDirectoryName+tempFileName, output, true);
			
			
			/*This is optional; i.e. useful for debugging
			FileDisplayer displayer = (FileDisplayer)hireEmployee(new CommandRecord(false), FileDisplayer.class, null);
			if (displayer!=null)
				displayer.showFile(tempDirectoryName+tempFileName, 2000,true); 
			*/
		}
	}
    
    /**
     * Converts space characters to underscores '_'
     * @param nName String to convert, presumibly the name of a node
     * @return copy of nName with spaces converted
     */
    protected String spaceConvertString(String nName){
        int spacePos = nName.indexOf(" ");
        if (spacePos == -1)
            return nName;
        else {
            StringBuffer workBuffer = new StringBuffer(nName);
            while (spacePos != -1){
                workBuffer.setCharAt(spacePos,'_');
                spacePos = nName.indexOf(" ",spacePos+1);
            }
            return workBuffer.toString();
        }
    }
    
    /**
     * Saves state of convertSpaces before updating
     * @param newValue
     */
    public void pushConvertSpaces(boolean newValue){
        savedConvertSpaces = convertSpaces;
        convertSpaces = newValue;
    }
    

    /**
     * no need to return the value, just restore it
     */
    public void popConvertSpaces(){
        convertSpaces = savedConvertSpaces;
    }
    
  	public abstract void writeReport(MesquiteFile file, CharacterDistribution data1, CharacterDistribution data2, Tree theTree, String arguments);

	public abstract String reportToString(CharacterDistribution data1, CharacterDistribution data2, Tree theTree);
	
	public abstract String reportToString(CharacterDistribution data1, CharacterDistribution data2, Tree theTree, String lineDelimiter, String columnDelimiter);
	
	public abstract String reportContrastToString(CharacterDistribution data1, CharSourceCoordObed data2, Tree theTree);
	
	public abstract String reportContrastToString(CharacterDistribution data1, CharSourceCoordObed data2, Tree theTree, String lineDelimiter, String columnDelimiter);

}
