/* AC3 Mesquite */


package mesquite.ac3.lib;

import mesquite.lib.*;
import mesquite.categ.lib.CategoricalDistribution;
import mesquite.cont.lib.*;

public abstract class ContrastCalculator {

	protected Tree tree;
	protected ContinuousDistribution states;
	protected int numNodes;
	protected double [] contrasts = null;
	protected double [] rawContrasts = null;
	protected double [] x = null;
	protected double [] vPrime = null;
	protected double [] stdDev = null;
	protected double [] height = null;
	protected double [] weightPassThrough = null;   // needed for parents of nodes with missing data
	protected boolean weightedBranches = true;
	protected boolean statsOk = true;  
	// maybe private

	public boolean Ok () {
		return statsOk;
	}
	
	/*.................................................................................................................*/
	/** implements the notion of weighted branches for contrast calculations. */
	/** this needs to be reworked for missing data or removed */
	protected double branchWeight (int node){
		double weight;
		if (weightedBranches){
			if (node == tree.getRoot()){  /*asking for weight across root for unrooted case*/
//				int left = tree.firstDaughterOfNode(node);
//				int right = tree.lastDaughterOfNode(node);
//				if (tree.branchLengthUnassigned(left))
//					weight=tree.getBranchLength(right);
//				else if (tree.branchLengthUnassigned(right))
//					weight=tree.getBranchLength(left);
//				else
//					weight=tree.getBranchLength(left) +tree.getBranchLength(right);
				weight = 0.0;
			}
			else if (tree.branchLengthUnassigned(node))   //give warning?
				weight=1.0;
			else
				weight=tree.getBranchLength(node,1.0); // does same as above
			}
		else
			weight=1.0;
			
		//if (weight==0)  //doing this breaks zero-length branch polytomies
		//	weight=1.0;  //give warning?
		return(weight);
	}

	/** This returns the array of contrast values for those modules that need them 
	@return double[] - the array of contrast values calculated here */
	public double [] contrastCalculation() {
		int root = tree.getRoot();
		vCalculation(root);
		xCalculation(root);
		return contrasts;
	}	
	/*modified here*/
	public double[] contrastToRecalculate(NumberArray nodesToRecalc){
		double[] result=xExtendedCalculation(nodesToRecalc);
		return result;
	}
	abstract protected double[] xExtendedCalculation(NumberArray nodesToRecalc);
	abstract protected double[] DevExtendedCalculation(NumberArray nodesToRecalc);
	
	//end modif
	abstract protected void vCalculation(int node);
	
	abstract protected void xCalculation(int node);



	/** This returns the array of raw contrast values for those modules that need them 
	@return double[] - the array of raw contrast values calculated here or null for failure */
	public double [] rawContrastCalculation() {
		int root = tree.getRoot();
		vCalculation(root);
		xCalculation(root);
		return rawContrasts;
	}
	

	/** This returns the array of contrast values for those modules that need them 
	@return double[] - the array of raw contrast values calculated here or null for failure */
	public double [] nodeValueCalculation() {
		int root = tree.getRoot();
		vCalculation(root);
		xCalculation(root);
		return x;
	}
	

	/** This returns the array of contrast values for those modules that need them 
	@return double[] - the array of raw contrast values calculated here or null for failure */
	public double [] vPrimeCalculation() {
		int root = tree.getRoot();
		vCalculation(root);
		return vPrime;
	}

	public abstract double [] fillTips(int node);

	/** This returns the array of contrast values for those modules that need them 
	@return double[] - the array of raw contrast values calculated here or null for failure */
	public double [] stdDevCalculation() {
		int root = tree.getRoot();
		vCalculation(root);
		return stdDev;
	}
	
	public double [] correctedStdDevCalculation(NumberArray nodesToRecalc){
		double[] result=DevExtendedCalculation(nodesToRecalc);
		return result;
	}

	/** Just returns the node values (tip and interior), without recalculating */
	public double [] getNodeValues() {
		return x;
	}

	/** Just returns the standard deviation array, without recalculating */
	public double [] getStdDev() {
		return stdDev;
	}

	/** Just returns the vPrime array, without recalculating */
	public double [] getVPrime() {
		return vPrime;
	}

}